@@ -1,4 +1,4 @@
-/** 
+/**
  * @namespace   biew_addons
  * @file        addons/tools/eval.c
  * @brief       A simple mathematical expression evaluator in C.
@@ -597,7 +597,7 @@ static void CalculatorFunc(void)
   twUseWin(ewnd);
   while(1)
   {
-   ret = xeditstring(estr,NULL,76);
+   ret = xeditstring(estr,NULL,76,NULL);
    if(ret == KE_ESCAPE || ret == KE_F(10)) break;
    else
      if(ret == KE_ENTER)
