@@ -1,4 +1,4 @@
-/** 
+/**
  * @namespace   biew
  * @file        bconsole.c
  * @brief       This file contains low level BIEW console functions.
@@ -27,11 +27,11 @@
 #include "colorset.h"
 #include "biewutil.h"
 #include "tstrings.h"
+#include "search.h"
 #include "bconsole.h"
 #include "biewlib/pmalloc.h"
 #include "biewlib/kbd_code.h"
 
-tAbsCoord Width,Height;
 extern TWindow * ErrorWnd;
 
 #define MAXINPUT 79
@@ -50,8 +50,6 @@ void __FASTCALL__ initBConsole( unsigned long vio_flg,unsigned long twin_flg )
     twDestroy();
     exit(EXIT_FAILURE);
   }
-  Width = tvioWidth;
-  Height = tvioHeight;
 }
 
 void __FASTCALL__ termBConsole( void )
@@ -60,12 +58,16 @@ void __FASTCALL__ termBConsole( void )
 }
 
 /**
-   read the next keyboard character 
+   read the next keyboard character
 */
-static int __NEAR__ __FASTCALL__ getkey(int hard)
+static int __NEAR__ __FASTCALL__ getkey(int hard, void (*func)(void))
 {
- return KB_freq ? KB_Buff[--KB_freq] : 
-                  GetEvent(hard ? hard > 1 ? drawAsmEdPrompt : drawEditPrompt : drawEmptyPrompt,NULL);
+ return KB_freq ? KB_Buff[--KB_freq] :
+                  GetEvent( func ? func : hard ? hard > 1 ? 
+			    drawAsmEdPrompt : drawEditPrompt : drawEmptyPrompt,
+			    func ? NULL : hard ? hard > 1 ? 
+			    EditAsmActionFromMenu: NULL: NULL,
+			    NULL);
 }
 
 static tBool __NEAR__ __FASTCALL__ ungotkey(int keycode)
@@ -79,28 +81,18 @@ static tBool __NEAR__ __FASTCALL__ ungotkey(int keycode)
   return ret;
 }
 
-/**
-   Allows the user to edit a string with only certain characters allowed -
-   Returns True if ESC was not pressed, FALSE is ESC was pressed.
-*/
-
-tBool __FASTCALL__ editstring(char *s,const char *legal,unsigned maxlength)
+int __FASTCALL__ xeditstring(char *s,const char *legal,unsigned maxlength, void (*func)(void))
 {
-  return eeditstring(s,legal,&maxlength,1,NULL,__ESS_ENABLEINSERT,NULL) != KE_ESCAPE;
-}
-
-int __FASTCALL__ xeditstring(char *s,const char *legal,unsigned maxlength)
-{
-  return eeditstring(s,legal,&maxlength,1,NULL,__ESS_ENABLEINSERT,NULL);
+  return eeditstring(s,legal,&maxlength,1,NULL,__ESS_ENABLEINSERT,NULL,func);
 }
 
 #define isSpace(val) (((int)val+1)%3 ? 0 : 1)
 #define isFirstD(pos) (isSpace(pos-1))
 #define isSecondD(pos) (isSpace(pos+1))
 
-static unsigned char insert = UCHAR_MAX;
+static tBool insert = True;
 
-int __FASTCALL__ eeditstring(char *s,const char *legal, unsigned *maxlength,unsigned _y,unsigned *stx,unsigned attr,char *undo)
+int __FASTCALL__ eeditstring(char *s,const char *legal, unsigned *maxlength,unsigned _y,unsigned *stx,unsigned attr,char *undo, void (*func)(void))
 {
  int c;
  unsigned len = attr & __ESS_HARDEDIT ? *maxlength : attr & __ESS_NON_C_STR ? _y : strlen(s);
@@ -138,14 +130,14 @@ int __FASTCALL__ eeditstring(char *s,const char *legal, unsigned *maxlength,unsi
           twGotoXY(len + 2,y);
         }
       for(i = len; i < *maxlength;i++)
-         twPutChar((attr & __ESS_FILLER_7BIT) == __ESS_FILLER_7BIT ? ' ' : '');
+         twPutChar((attr & __ESS_FILLER_7BIT) == __ESS_FILLER_7BIT ? TWC_DEF_FILLER : TWC_MED_SHADE);
     }
   }
   twRefreshLine(twUsedWin(),y);
   twGotoXY(pos + 1, y);
   func_getkeys = attr & __ESS_HARDEDIT ? attr & 0x0020 ? 2 : 1 : 0;
   new_keycode:
-  c = getkey(func_getkeys);
+  c = getkey(func_getkeys, func);
   lastkey = c;
   attr |= __ESS_NOREDRAW;
   switch(c)
@@ -190,11 +182,17 @@ int __FASTCALL__ eeditstring(char *s,const char *legal, unsigned *maxlength,unsi
                            }
                        }
                        break;
-   case KE_DEL : if (pos < len && !(attr & __ESS_HARDEDIT) && !ashex)
+   case KE_DEL : if (pos < len && !(attr & __ESS_HARDEDIT))
                     {
                       attr &= ~__ESS_NOREDRAW;
                       memmove(&s[pos], &s[pos+1], len - pos);
                       len--;
+                         if(ashex)
+                           if(!isSpace(pos) && pos > 2)
+                           {
+                             memmove(&s[pos], &s[pos+2], len - pos);
+                             len-=2;
+                           }
                     }
                     break;
    case KE_CTL_BKSPACE:
@@ -208,14 +206,14 @@ int __FASTCALL__ eeditstring(char *s,const char *legal, unsigned *maxlength,unsi
    case KE_INS :   if(attr & __ESS_ENABLEINSERT &&
                       !(attr & __ESS_HARDEDIT) &&
                       !ashex)
-                            insert = insert ? 0 : UCHAR_MAX;
+                            insert = insert ? False : True;
                     twSetCursorType(attr & __ESS_ENABLEINSERT ? insert ? TW_CUR_NORM : TW_CUR_SOLID : TW_CUR_NORM);
                     break;
    case KE_ENTER  : break;
    case KE_ESCAPE : len = 0; break;
    default     :
                   /** FunKey trough int 16 returned XX00 exclude Ctrl0-CtrlZ */
-                  if(!(c & 0x00FF) || c == KE_TAB || c == KE_BKSPACE) { c = KE_ENTER; break; }
+                  if(!(c & 0x00FF) || c == KE_TAB || c == KE_SHIFT_TAB || c == KE_BKSPACE) { c = KE_ENTER; break; }
                   if ( legal == 0 || strchr(legal, c) != NULL )
                   {
                     attr &= ~__ESS_NOREDRAW;
@@ -230,7 +228,7 @@ int __FASTCALL__ eeditstring(char *s,const char *legal, unsigned *maxlength,unsi
                         len++;
                       }
                      }
-                     else 
+                     else
                        if (pos >= len) len++;
                      if(len > *maxlength) len = *maxlength;
                      s[pos++] = c;
@@ -268,8 +266,8 @@ TWindow *__FASTCALL__ PleaseWaitWnd( void )
 {
    TWindow *w,*usd;
    usd = twUsedWin();
-   w = CrtDlgWndnls(" Busy ",14,1);
-   twGotoXY(1,1); twPutS("Please Wait...");
+   w = CrtDlgWndnls(SYSTEM_BUSY,14,1);
+   twGotoXY(1,1); twPutS(PLEASE_WAIT);
    twUseWin(usd);
    return w;
 }
@@ -279,60 +277,123 @@ void __FASTCALL__ MemOutBox(const char *user_msg)
   ErrMessageBox(user_msg," Not enough memory! ");
 }
 
+struct percent_data
+{
+  time_t   _time;
+  time_t   prev_time;
+  unsigned _percents;
+  tBool    is_first;
+};
+
+static long __FASTCALL__ PercentWndCallBack(TWindow *it,unsigned event, unsigned long param, void *data)
+{
+  struct percent_data *my_data;
+  UNUSED(param);
+  UNUSED(data);
+  switch(event)
+  {
+    case WM_CREATE:
+                     if((my_data=malloc(sizeof(struct percent_data))) != NULL)
+                     {
+                        twSetUsrData(it,my_data);
+                     }
+                     break;
+    case WM_DESTROY:
+                     my_data = twGetUsrData(it);
+                     if(my_data) free(my_data);
+                     break;
+     default: break;
+  }
+  return 0L;
+}
+
+
 TWindow *__FASTCALL__ PercentWnd(const char *text,const char *title)
 {
   TWindow *ret,*usd;
   static time_t sttime;
+  struct percent_data* my_data;
   usd = twUsedWin();
-  ret = WindowOpen(1,1,53,6,TWS_FRAMEABLE | TWS_NLSOEM);
+  twcRegisterClass("PERCENT_WND", __CS_ORDINAL, PercentWndCallBack);
+  ret = twCreateWinEx(1,1,53,6,TWS_FRAMEABLE | TWS_NLSOEM,NULL,"PERCENT_WND");
   twCentredWin(ret,NULL);
   twSetColorAttr(dialog_cset.main);
   twSetFrameAttr(ret,TW_UP3D_FRAME,dialog_cset.main);
   twSetTitleAttr(ret,title,TW_TMODE_CENTER,dialog_cset.title);
-  twSetFooterAttr(ret," [ ESC ] - Abort ",TW_TMODE_RIGHT,dialog_cset.footer);
+  twSetFooterAttr(ret," [ Ctrl-Break ] - Abort ",TW_TMODE_RIGHT,dialog_cset.footer);
   twClearWin();
   twGotoXY(1,1); twPutS(text);
   twinDrawFrameAttr(1,2,52,4,TW_DN3D_FRAME,dialog_cset.main);
   twShowWin(ret);
   twUseWin(usd);
   time(&sttime);
-  twSetUsrData(ret,&sttime);
+  my_data = twGetUsrData(ret);
+  if(my_data)
+  {
+     my_data->_time = my_data->prev_time = sttime;
+     my_data->_percents = 0;
+     my_data->is_first = True;
+  }
   return ret;
 }
 
 tBool __FASTCALL__ ShowPercentInWnd(TWindow *pw,unsigned percents)
 {
   TWindow *usd;
-  unsigned cells,remaind;
-  time_t *sttime,curtime,deltat;
+  unsigned cells,remaind, prev_prcnt = 0;
+  time_t sttime =0,curtime,deltat, prev_time = 0;
   struct tm *tm;
+  struct percent_data* my_data;
+  tBool is_first = True;
   char outb[50];
+  tBool ret;
   usd = twUsedWin();
   twUseWin(pw);
-  if(percents > 100) percents = 100;
-  cells = percents/2;
-  remaind = percents%2;
-  memset(outb,TWC_FL_BLK,cells);
-  if(remaind) outb[cells++] = TWC_LF_HBLK;
-  if(cells < sizeof(outb)) memset(&outb[cells],TWC_DEF_FILLER,sizeof(outb)-cells);
-  twDirectWrite(2,3,outb,sizeof(outb));
-  sttime = twGetUsrData(pw);
+  my_data = twGetUsrData(pw);
+  if(my_data)
+  {
+    prev_prcnt = my_data->_percents;
+    sttime = my_data->_time;
+    prev_time = my_data->prev_time;
+    is_first = my_data->is_first;
+  }
+  if(percents != prev_prcnt || is_first)
+  {
+    if(percents > 100) percents = 100;
+    cells = percents/2;
+    remaind = percents%2;
+    memset(outb,TWC_FL_BLK,cells);
+    if(remaind) outb[cells++] = TWC_LF_HBLK;
+    if(cells < sizeof(outb)) memset(&outb[cells],TWC_DEF_FILLER,sizeof(outb)-cells);
+    twDirectWrite(2,3,outb,sizeof(outb));
+  }
   time(&curtime);
-  deltat = curtime - (*sttime);
-  tm = localtime(&deltat);
-  strftime(outb,sizeof(outb),"%X",tm);
-  twGotoXY(1,5);
-  twPutS("Elapsed time: ");
-  twPutS(outb);
+  if(prev_time != curtime || is_first)
+  {
+    deltat = curtime - sttime;
+    tm = gmtime(&deltat);
+    strftime(outb,sizeof(outb),"%X",tm);
+    twGotoXY(1,5);
+    twPutS("Elapsed time: ");
+    twPutS(outb);
+  }
+  if(my_data)
+  {
+    my_data->_percents = percents;
+    my_data->is_first = False;
+    my_data->prev_time = curtime;
+  }
   twUseWin(usd);
-  return !IsKbdTerminate();
+  ret = !IsKbdTerminate();
+  CleanKbdTermSig();
+  return ret;
 }
 
 TWindow * __FASTCALL__ WindowOpen(tAbsCoord x1,tAbsCoord y1,tAbsCoord x2,tAbsCoord y2,unsigned flags)
 {
   TWindow *ret;
   ret = twCreateWin(x1,y1,x2-x1+1,y2-y1+1,flags);
-  if(!ret) { ErrMessageBox("Out of memory",NULL); exit(EXIT_FAILURE); }
+  if(!ret) { MemOutBox(NULL); exit(EXIT_FAILURE); }
   return ret;
 }
 
@@ -360,10 +421,7 @@ static TWindow * __NEAR__ __FASTCALL__ _CreateWindowDD(const char * title,tAbsCo
  return win;
 }
 
-static TWindow * __NEAR__ __FASTCALL__ _CreateWindowDDnls(const char * title,tAbsCoord x2,tAbsCoord y2)
-{
-  return _CreateWindowDD(title,x2,y2,True);
-}
+#define _CreateWindowDDnls(title,x2,y2) (_CreateWindowDD(title,x2,y2,True))
 
 TWindow * __FASTCALL__ CrtDlgWnd(const char * title,tAbsCoord width,tAbsCoord height )
 {
@@ -446,7 +504,8 @@ TWindow * __FASTCALL__ CreateEditor(tAbsCoord X1,tAbsCoord Y1,tAbsCoord X2,tAbsC
  return ret;
 }
 
-static void __NEAR__ __FASTCALL__ __MB(const char * text,const char * title,ColorAttr base,ColorAttr frame)
+static void __NEAR__ __FASTCALL__ __MB(const char * text,const char * title,
+                                       ColorAttr base,ColorAttr frame)
 {
  unsigned slen,tlen;
  slen = strlen(text) + 3;
@@ -463,75 +522,43 @@ static void __NEAR__ __FASTCALL__ __MB(const char * text,const char * title,Colo
  twPutS(text);
 }
 
-void __FASTCALL__ TMessageBox(const char * text,const char * title)
+static void __NEAR__ __FASTCALL__ __MessageBox(const char * text,const char * title,
+                                               ColorAttr base,ColorAttr frame)
 {
  TWindow *prev;
  unsigned evt;
  prev = twUsedWin();
  twUseWin(ErrorWnd);
- __MB(text,title,dialog_cset.main,dialog_cset.title);
+ __MB(text,title,base,frame);
  do
  {
-   evt = GetEvent(drawEmptyPrompt,ErrorWnd);
+   evt = GetEvent(drawEmptyPrompt,NULL,ErrorWnd);
  }
- while(!(evt == KE_ESCAPE || evt == KE_F(10)));
+ while(!(evt == KE_ESCAPE || evt == KE_F(10) || evt == KE_SPACE || evt == KE_ENTER));
  twHideWin(ErrorWnd);
  twUseWin(prev);
- twResizeWin(ErrorWnd,Width,Height);
+ twResizeWin(ErrorWnd,tvioWidth,tvioHeight); /* It for reserving memory */
+}
+
+
+void __FASTCALL__ TMessageBox(const char * text,const char * title)
+{
+ __MessageBox(text,title ? title : "",dialog_cset.main,dialog_cset.title);
 }
 
 void __FASTCALL__ NotifyBox(const char * text,const char * title)
 {
- TWindow *prev;
- unsigned evt;
- if(title == NULL) title = NOTE_MSG;
- prev = twUsedWin();
- twUseWin(ErrorWnd);
- __MB(text,title,notify_cset.main,notify_cset.border);
- do
- {
-   evt = GetEvent(drawEmptyPrompt,ErrorWnd);
- }
- while(!(evt == KE_ESCAPE || evt == KE_F(10)));
- twHideWin(ErrorWnd);
- twUseWin(prev);
- twResizeWin(ErrorWnd,Width,Height);
+ __MessageBox(text,title ? title : NOTE_MSG,notify_cset.main,notify_cset.border);
 }
 
 void __FASTCALL__ ErrMessageBox(const char * text,const char * title)
 {
- TWindow *prev;
- unsigned evt;
- if(title == NULL) title = ERROR_MSG;
- prev = twUsedWin();
- twUseWin(ErrorWnd);
- __MB(text,title,error_cset.main,error_cset.border);
- do
- {
-   evt = GetEvent(drawEmptyPrompt,ErrorWnd);
- }
- while(!(evt == KE_ESCAPE || evt == KE_F(10)));
- twHideWin(ErrorWnd);
- twUseWin(prev);
- twResizeWin(ErrorWnd,Width,Height);
+ __MessageBox(text,title ? title : ERROR_MSG,error_cset.main,error_cset.border);
 }
 
 void __FASTCALL__ WarnMessageBox(const char * text,const char * title)
 {
- TWindow *prev;
- unsigned evt;
- if(title == NULL) title = WARN_MSG;
- prev = twUsedWin();
- twUseWin(ErrorWnd);
- __MB(text,title,warn_cset.main,warn_cset.border);
- do
- {
-   evt = GetEvent(drawEmptyPrompt,ErrorWnd);
- }
- while(!(evt == KE_ESCAPE || evt == KE_F(10)));
- twHideWin(ErrorWnd);
- twUseWin(prev);
- twResizeWin(ErrorWnd,Width,Height);
+ __MessageBox(text,title ? title : WARN_MSG,warn_cset.main,warn_cset.border);
 }
 
 void __FASTCALL__ errnoMessageBox(const char *text,const char *title,int __errno__)
@@ -542,27 +569,37 @@ void __FASTCALL__ errnoMessageBox(const char *text,const char *title,int __errno
 }
 
 static void __NEAR__ __FASTCALL__ PaintLine(unsigned i,const char *name,
-                                            unsigned width,tBool isOrdinal,
+                                            unsigned width,unsigned mord_width,
+                                            tBool isOrdinal,
                                             tBool useAcc,tBool is_hl)
 {
   size_t namelen;
   char buffer[__TVIO_MAXSCREENWIDTH];
   memset(buffer,TWC_DEF_FILLER,sizeof(buffer));
-  namelen = strlen(name);
+  namelen = name?strlen(name):0;
   if(isOrdinal)
   {
     char * endptr;
-    endptr = strrchr(name,'');
+    endptr = name?strrchr(name,LB_ORD_DELIMITER):NULL;
     if(endptr)
     {
-      unsigned len;
+      unsigned len, rlen;
+      // write name
       len = endptr - name;
-      memcpy((char *)buffer,name,len);
-      buffer[width - 6] = '@';
-      memcpy((char *)&buffer[width - 5],&name[len + 1],min(5,namelen-(len+1)));
+      rlen = len;
+      if(len > width - mord_width-1)
+          rlen = width - mord_width-3;
+      memcpy(buffer,name,rlen);
+      if(len > rlen) memcpy(buffer+rlen,"..", 2);           // using 2 dots now -XF
+      // write ordinal. it's left aligned now -XF
+      buffer[width - mord_width - 1] = '@';
+      len = rlen = namelen - (len+1);
+      if(rlen > mord_width) rlen = mord_width - 2;
+      memcpy(&buffer[width - mord_width], endptr+1, rlen);
+      if(len > rlen) memcpy(buffer+width-mord_width+rlen,"..", 2);
     }
   }
-  else memcpy((char *)buffer,name,min(namelen,width));
+  else if(name) memcpy((char *)buffer,name,min(namelen,width));
   if(useAcc)
   {
     char *st,*ends,*ptr;
@@ -604,73 +641,40 @@ static void __NEAR__ __FASTCALL__ PaintLine(unsigned i,const char *name,
 static void __NEAR__ __FASTCALL__ Paint(TWindow *win,const char ** names,
                                         unsigned nlist,unsigned start,
                                         unsigned height,unsigned width,
+                                        unsigned mord_width, 
                                         tBool isOrdinal,tBool useAcc,
                                         unsigned cursor)
 {
- unsigned i;
+ unsigned i, pos = 0;
  twUseWin(win);
  twFreezeWin(win);
  width -= 3;
+ if (height>2 && height<nlist)
+     pos = 1 + (start+cursor)*(height-2)/nlist;
  for(i = 0;i < height;i++)
  {
    twSetColorAttr(menu_cset.main);
    twGotoXY(1,i + 1);
-   if(i == 0) twPutChar(start ? TWC_UP_ARROW : TWC_DEF_FILLER);
-   else
-     if(i == height - 1) twPutChar(start + height < nlist ? TWC_DN_ARROW : TWC_DEF_FILLER);
+   if (i == 0)
+       twPutChar(start ? TWC_UP_ARROW : TWC_DEF_FILLER);
+   else if(i == height-1)
+       twPutChar(start + height < nlist ? TWC_DN_ARROW : TWC_DEF_FILLER);
+   else if (i == pos)
+       twPutChar(TWC_THUMB);
+   else twPutChar(TWC_DEF_FILLER);
    twGotoXY(2,i + 1);
    twPutChar(TWC_SV);
    twSetColorAttr(menu_cset.item.active);
-   PaintLine(i,names[i + start],width,isOrdinal,useAcc,cursor == i);
+   PaintLine(i,names[i + start],width,mord_width,isOrdinal,useAcc,cursor == i);
  }
  twRefreshWin(win);
 }
 
 static char byNam;
 
-tBool __FASTCALL__ _lb_searchtext(const char *str,const char *tmpl,unsigned searchlen,tBool Case,tBool Word)
+tBool __FASTCALL__ _lb_searchtext(const char *str,const char *tmpl,unsigned searchlen,const int *cache, unsigned flg)
 {
-  unsigned slen,tlen;
-  tBool __found;
-  tBool cond;
-  unsigned start;
-  char fbuff[21];
-  char cbuff[21],ch,ch1;
-  __found = False;
-  slen = strlen(str);
-  tlen = searchlen;
-  cond = False;
-  for(start = 0;start < slen;start++)
-  {
-   ch = str[start];
-   ch1 = tmpl[0];
-   if(!Case) { ch = toupper(ch); ch1 = toupper(ch1); }
-   if(ch == ch1)
-   {
-    if(tlen > 1)
-    {
-      unsigned tmp;
-      tmp = min(tlen - 1,slen-start);
-      memcpy(fbuff,&str[start+1],tmp);
-      memcpy(cbuff,&tmpl[1],tmp);
-      if(!Case)
-      {
-         memupr((void *)fbuff,tmp);
-         memupr((void *)cbuff,tmp);
-      }
-      if(memcmp(fbuff,cbuff,tmp) == 0) cond = True;
-    }
-    else cond = True;
-   }
-   if(Word && cond)
-   {
-     ch = start ? str[start-1] : ' ';
-     ch1 = start + tlen < slen ? str[start + tlen] : ' ';
-     if(!(isseparate(ch) && isseparate(ch1))) cond = False;
-   }
-   if(cond) { __found = True; break; }
-  }
-  return __found;
+  return strFind(str, strlen(str), tmpl, searchlen, cache, flg) ? True : False;
 }
 
 static tCompare __FASTCALL__ listcompare(const void __HUGE__ *v1,const void __HUGE__ *v2)
@@ -684,8 +688,8 @@ static tCompare __FASTCALL__ listcompare(const void __HUGE__ *v1,const void __HU
 
     s1 = *((const char **)v1);
     s2 = *((const char **)v2);
-    o1 = strrchr(s1,'');
-    o2 = strrchr(s2,'');
+    o1 = strrchr(s1,LB_ORD_DELIMITER);
+    o2 = strrchr(s2,LB_ORD_DELIMITER);
     if(o1 && o2)
     {
       unsigned long ord1,ord2;
@@ -694,9 +698,7 @@ static tCompare __FASTCALL__ listcompare(const void __HUGE__ *v1,const void __HU
        strncpy(buff2,o2 + 1,6);
        ord1 = atol(buff1);
        ord2 = atol(buff2);
-       if(ord1 < ord2) ret = -1;
-       else if(ord1 == ord2) ret = 0;
-            else ret = 1;
+       ret = __CmpLong__(ord1,ord2);
     }
     else  ret = stricmp(s1,s2);
   }
@@ -706,11 +708,10 @@ static tCompare __FASTCALL__ listcompare(const void __HUGE__ *v1,const void __HU
 static int __NEAR__ __FASTCALL__ __ListBox(const char ** names,unsigned nlist,unsigned defsel,const char * title,int assel)
 {
  TWindow * wlist;
- unsigned mwidth = strlen(title);
- unsigned i,j,height;
  char *acctable = 0;
+ unsigned i,j,width,height,mwidth = strlen(title);
+ unsigned mordstr_width, mord_width;
  int ret,start,ostart,cursor,ocursor,scursor;
- unsigned width;
  tBool isOrdinal,sf;
  if(!names || !nlist) return -1;
  isOrdinal = True;
@@ -728,7 +729,7 @@ static int __NEAR__ __FASTCALL__ __ListBox(const char ** names,unsigned nlist,un
    for(i = 0;i < nlist;i++)
    {
      unsigned len;
-     len = strlen(names[i]);
+     len = names[i]?strlen(names[i]):0;
      for(j = 0;j < len;j++)
      {
        if(names[i][j] == '~' && names[i][j+1] != '~')
@@ -739,11 +740,37 @@ static int __NEAR__ __FASTCALL__ __ListBox(const char ** names,unsigned nlist,un
      }
    }
  }
- if(!strrchr(names[0],'')) isOrdinal = False;
- if(!isOrdinal) for(i = 0;i < nlist;i++) { j = strlen(names[i]); if(j > mwidth) mwidth = j; }
- else                                    mwidth = tvioWidth-7;
+ if(names[0]) if(!strrchr(names[0],LB_ORD_DELIMITER)) isOrdinal = False;
+ mordstr_width = mord_width = 0;
+ if(!isOrdinal) 
+   for(i = 0;i < nlist;i++) 
+   {
+     j = names[i]?strlen(names[i]):0;
+     if(j > mwidth) mwidth = j;
+   }
+ else
+ {
+   char *ord_delimiter;
+   for(i = 0;i < nlist;i++) 
+   {
+     ord_delimiter = names[i]?strrchr(names[i], LB_ORD_DELIMITER):NULL;
+     if(ord_delimiter)
+     {
+       j = ord_delimiter - names[i];
+       if(j > mordstr_width) mordstr_width = j;
+       j = &names[i][strlen(names[i])] - ord_delimiter;
+       if(j > mord_width) mord_width = j;
+     }
+   }
+   // name now has higher priority than ordinal -XF
+   if(mordstr_width > (unsigned)(tvioWidth-10))
+       mordstr_width = (unsigned)(tvioWidth-10);
+   if(mord_width > (unsigned)(tvioWidth-4)-mordstr_width-1)
+       mord_width = (unsigned)(tvioWidth-4)-mordstr_width-1;
+   mwidth = mordstr_width+mord_width+1;
+ }
  mwidth += 4;
- if(mwidth > (unsigned)(tvioWidth-2)) mwidth = tvioWidth-2;
+ if(mwidth > (unsigned)(tvioWidth-1)) mwidth = tvioWidth-1;         // maximal width increased to tvioWidth-1 -XF
  height = nlist < (unsigned)(tvioHeight - 4) ? nlist : tvioHeight - 4;
  wlist = CrtLstWndnls(title,mwidth-1,height);
  if((assel & LB_SELECTIVE) == LB_SELECTIVE) twSetFooterAttr(wlist," [ENTER] - Go ",TW_TMODE_RIGHT,dialog_cset.selfooter);
@@ -756,21 +783,21 @@ static int __NEAR__ __FASTCALL__ __ListBox(const char ** names,unsigned nlist,un
     ostart = start;
     ocursor = cursor;
  }
- Paint(wlist,names,nlist,(unsigned)start,height,mwidth,isOrdinal,(assel & LB_USEACC) == LB_USEACC,(unsigned)cursor);
+ Paint(wlist,names,nlist,(unsigned)start,height,mwidth,mord_width,isOrdinal,(assel & LB_USEACC) == LB_USEACC,(unsigned)cursor);
  width = mwidth - 3;
  if((assel & LB_SELECTIVE) == LB_SELECTIVE)
  {
    twSetColorAttr(menu_cset.item.focused);
-   PaintLine((unsigned)cursor,names[cursor + start],width,isOrdinal,(assel & LB_USEACC) == LB_USEACC,True);
+   PaintLine((unsigned)cursor,names[cursor + start],width,mord_width,isOrdinal,(assel & LB_USEACC) == LB_USEACC,True);
  }
  sf = False;
  for(;;)
  {
    unsigned ch;
-   ch = GetEvent(isOrdinal ? drawOrdListPrompt : (assel & LB_SORTABLE) ? drawListPrompt : drawSearchListPrompt,wlist);
+   ch = GetEvent(isOrdinal ? drawOrdListPrompt : (assel & LB_SORTABLE) ? drawListPrompt : drawSearchListPrompt,NULL,wlist);
    if(ch == KE_ESCAPE || ch == KE_F(10)) { ret = -1; break; }
    if(ch == KE_ENTER)                    { ret = start + cursor; break; }
-   if(ch != KE_F(7)) scursor = -1;
+   if(ch!=KE_F(7) && ch!=KE_SHIFT_F(7))  scursor = -1;
    switch(ch)
    {
      case KE_F(2):
@@ -783,11 +810,11 @@ static int __NEAR__ __FASTCALL__ __ListBox(const char ** names,unsigned nlist,un
                 goto restart;
               }
               break;
-     case KE_F(4): /** save contest to a disk */
+     case KE_F(4): /** save content to disk */
               {
                 char ofname[256];
                 ofname[0] = 0;
-                if(GetStringDlg(ofname," Save info to the file : "," [ENTER] - Proceed ",NAME_MSG))
+                if(GetStringDlg(ofname," Save info to file : "," [ENTER] - Proceed ",NAME_MSG))
                 {
                   FILE * out;
                   out = fopen(ofname,"wt");
@@ -802,37 +829,42 @@ static int __NEAR__ __FASTCALL__ __ListBox(const char ** names,unsigned nlist,un
                     for(i = 0;i < nlist;i++)
                     {
                       char *p;
-                      p = strchr(names[i],'');
+                      p = names[i]?strchr(names[i],LB_ORD_DELIMITER):NULL;
                       if(p)
                       {
                         *p = 0;
                         fprintf(out,names[i]);
                         for(j = p - names[i];j < 50;j++) fprintf(out," ");
                         fprintf(out," @%s",p+1);
-                        if(p) *p = '';
+                        if(p) *p = LB_ORD_DELIMITER;
                       }
                       else fprintf(out,names[i]);
                       fprintf(out,"\n");
                     }
                     fclose(out);
                   }
-                  else errnoMessageBox("Can not access file",NULL,errno);
+                  else errnoMessageBox(WRITE_FAIL,NULL,errno);
                 }
               }
               break;
      case KE_F(7): /** perform binary search in list */
+     case KE_SHIFT_F(7):
              {
                static char searchtxt[21] = "";
                static unsigned char searchlen = 0;
-               static tBool CaseSens = False,WordOnly = False,Backwarding = False;
-               if(SearchDialog(searchtxt,&searchlen,&CaseSens,&WordOnly,&Backwarding))
+               static unsigned sflg = SF_NONE;
+
+               if (!(ch==KE_SHIFT_F(7) && searchlen) &&
+                   !SearchDialog(SD_SIMPLE,searchtxt,&searchlen,&sflg))
+                   break;
+
                {
-                  int direct;
+                  int direct, cache[UCHAR_MAX+1];
                   tBool found;
                   int ii,endsearch,startsearch;
                   searchtxt[searchlen] = 0;
-                  endsearch = Backwarding ? -1 : nlist;
-                  direct = Backwarding ? -1 : 1;
+                  endsearch = sflg & SF_REVERSE ? -1 : nlist;
+                  direct = sflg & SF_REVERSE ? -1 : 1;
                   startsearch = (assel & LB_SELECTIVE) == LB_SELECTIVE ?
                                  cursor + start :
                                  scursor != -1 ?
@@ -842,12 +874,15 @@ static int __NEAR__ __FASTCALL__ __ListBox(const char ** names,unsigned nlist,un
                   if(startsearch < 0) startsearch = 0;
                   if((assel & LB_SELECTIVE) == LB_SELECTIVE || scursor != -1)
                   {
-                    Backwarding ? startsearch-- : startsearch++;
+                    sflg & SF_REVERSE ? startsearch-- : startsearch++;
                   }
                   found = False;
+                  fillBoyerMooreCache(cache, searchtxt, searchlen, sflg & SF_CASESENS);
                   for(ii = startsearch;ii != endsearch;ii+=direct)
                   {
-                     if(_lb_searchtext(names[ii],searchtxt,searchlen,CaseSens,WordOnly))
+		    if(names[ii])
+		    {
+                     if(_lb_searchtext(names[ii],searchtxt,searchlen,cache,sflg))
                      {
                         start = scursor = ii;
                         if((unsigned)start > nlist - height) start = nlist - height;
@@ -857,6 +892,7 @@ static int __NEAR__ __FASTCALL__ __ListBox(const char ** names,unsigned nlist,un
                         found = True;
                         break;
                      }
+		    }
                   }
                   if(!found) scursor = -1;
                   if(scursor == -1) ErrMessageBox(STR_NOT_FOUND,SEARCH_MSG);
@@ -893,22 +929,23 @@ static int __NEAR__ __FASTCALL__ __ListBox(const char ** names,unsigned nlist,un
    if(start != ostart)
    {
      ostart = start;
-     Paint(wlist,names,nlist,(unsigned)start,height,mwidth,isOrdinal,(assel & LB_USEACC) == LB_USEACC,(unsigned)cursor);
+     Paint(wlist,names,nlist,(unsigned)start,height,mwidth,mord_width,isOrdinal,(assel & LB_USEACC) == LB_USEACC,(unsigned)cursor);
      sf = True;
    }
    if((cursor != ocursor || sf) && (assel & LB_SELECTIVE) == LB_SELECTIVE)
    {
      twSetColorAttr(menu_cset.item.active);
-     PaintLine((unsigned)ocursor,names[ocursor + start],width,isOrdinal,(assel & LB_USEACC) == LB_USEACC,False);
+     PaintLine((unsigned)ocursor,names[ocursor + start],width,mord_width,isOrdinal,(assel & LB_USEACC) == LB_USEACC,False);
      twSetColorAttr(menu_cset.item.focused);
-     PaintLine((unsigned)cursor,names[cursor + start],width,isOrdinal,(assel & LB_USEACC) == LB_USEACC,True);
+     PaintLine((unsigned)cursor,names[cursor + start],width,mord_width,isOrdinal,(assel & LB_USEACC) == LB_USEACC,True);
      ocursor = cursor;
      sf = False;
    }
    if(scursor != -1)
    {
      twSetColorAttr(menu_cset.highlight);
-     if(scursor >= start && (unsigned)scursor < start + height) PaintLine((unsigned)(scursor - start),names[scursor],width,isOrdinal,(assel & LB_USEACC) == LB_USEACC,True);
+     if(scursor >= start && (unsigned)scursor < start + height)
+         PaintLine((unsigned)(scursor - start),names[scursor],width,mord_width,isOrdinal,(assel & LB_USEACC) == LB_USEACC,True);
    }
  }
  Done:
@@ -957,7 +994,7 @@ int __FASTCALL__ PageBox(unsigned width,unsigned height,const void ** __obj,unsi
  for(;;)
  {
    unsigned ch;
-   ch = GetEvent(drawEmptyPrompt,wlist);
+   ch = GetEvent(drawEmptyPrompt,NULL,wlist);
    if(ch == KE_ESCAPE || ch == KE_F(10)) { ret = -1; break; }
    if(ch == KE_ENTER)                    { ret = start; break; }
    switch(ch)
