@@ -1,4 +1,4 @@
-/** 
+/**
  * @namespace   biew
  * @file        biew.c
  * @brief       This file contains entry point of BIEW project.
@@ -13,6 +13,9 @@
  * @author      Nick Kurshev
  * @since       1995
  * @note        Development, fixes and improvements
+ * @author      Kostya Nosov <k-nosov@yandex.ru>
+ * @date        27.11.2000
+ * @note        Changing technology recognition of new-exe files
 **/
 #include <string.h>
 #include <stdio.h>
@@ -31,12 +34,14 @@
 #include "tstrings.h"
 #include "reg_form.h"
 #include "biewutil.h"
+#include "search.h"
 #include "setup.h"
 #include "biewlib/file_ini.h"
 #include "biewlib/kbd_code.h"
 #include "biewlib/biewlib.h"
 #include "biewlib/pmalloc.h"
 
+
 unsigned ArgCount;
 char **  ArgVector;
 unsigned ListFileCount;
@@ -46,13 +51,24 @@ unsigned long LastOffset;
 static tBool UseIniFile=True;
 char biew_help_name[FILENAME_MAX+1] = "";
 char biew_skin_name[FILENAME_MAX+1] = "";
+char biew_syntax_name[FILENAME_MAX+1] = "";
 extern char last_skin_error[];
-char biew_scheme_name[256] = "Build-in";
+char biew_scheme_name[256] = "Built-in";
 static char biew_ini_ver[32];
 unsigned long biew_vioIniFlags = 0L;
 unsigned long biew_twinIniFlags = 0L;
 unsigned long biew_kbdFlags = 0L;
 tBool iniSettingsAnywhere = False;
+tBool fioUseMMF = False;
+tBool iniPreserveTime = False;
+unsigned long headshift = 0L;
+char *ini_name;
+
+TWindow * MainWnd = 0,*HelpWnd = 0,*TitleWnd = 0,*ErrorWnd = 0;
+
+#define SHORT_PATH_LEN __TVIO_MAXSCREENWIDTH-54
+
+char shortname[SHORT_PATH_LEN + 1];
 
 extern REGISTRY_BIN binTable;
 extern REGISTRY_BIN neTable;
@@ -61,6 +77,7 @@ extern REGISTRY_BIN leTable;
 extern REGISTRY_BIN lxTable;
 extern REGISTRY_BIN nlm386Table;
 extern REGISTRY_BIN elf386Table;
+extern REGISTRY_BIN jvmTable;
 extern REGISTRY_BIN coff386Table;
 extern REGISTRY_BIN archTable;
 extern REGISTRY_BIN aoutTable;
@@ -68,6 +85,9 @@ extern REGISTRY_BIN OldPharLapTable;
 extern REGISTRY_BIN PharLapTable;
 extern REGISTRY_BIN rdoffTable;
 extern REGISTRY_BIN rdoff2Table;
+extern REGISTRY_BIN lmfTable;
+extern REGISTRY_BIN mzTable;
+extern REGISTRY_BIN dossysTable;
 
 static REGISTRY_BIN *mainBinTable[] =
 {
@@ -77,6 +97,7 @@ static REGISTRY_BIN *mainBinTable[] =
   &lxTable,
   &nlm386Table,
   &elf386Table,
+  &jvmTable,
   &coff386Table,
   &archTable,
   &aoutTable,
@@ -84,6 +105,9 @@ static REGISTRY_BIN *mainBinTable[] =
   &PharLapTable,
   &rdoffTable,
   &rdoff2Table,
+  &lmfTable,
+  &mzTable,
+  &dossysTable,
   &binTable
 };
 
@@ -167,16 +191,10 @@ void QuickSelectMode( void )
   if(activeMode->init) activeMode->init();
 }
 
-TWindow * MainWnd = 0,*HelpWnd = 0,*TitleWnd = 0,*ErrorWnd = 0;
-
-#define SHORT_PATH_LEN __TVIO_MAXSCREENWIDTH-54
-
-char shortname[SHORT_PATH_LEN + 1];
-
 static void __NEAR__ __FASTCALL__ MakeShortName( void )
 {
   unsigned l;
-  unsigned slen = Width-54;
+  unsigned slen = twGetClientWidth(TitleWnd)-54;
   l = strlen(ArgVector[1]);
   if(l <= slen) strcpy(shortname,ArgVector[1]);
   else
@@ -189,20 +207,27 @@ static void __NEAR__ __FASTCALL__ MakeShortName( void )
   __nls_CmdlineToOem((unsigned char *)shortname,strlen(shortname));
 }
 
-unsigned int bpos = 0;
-
-char FType = NOT_EXE;
-DOSHDR oldh;
-unsigned long headshift = 0L;
-
 unsigned long IsNewExe()
 {
   unsigned long ret;
   char id[2];
-  BMReadBufferEx(id,sizeof(id),0,BM_SEEK_SET);
+  bmReadBufferEx(id,sizeof(id),0,BM_SEEK_SET);
+#if 0
+   /*
+      It is well documented technology, but it correctly working
+      only with normal stubs, i.e. when New EXE header is located at
+      offset > 0x40. However, in PC world exists files with wrong
+      stubs, which are normal for Host OS. Hence biew must recognize
+      them as normal New EXE files, despite the fact that DOS can
+      not execute ones.
+      Fixed by Kostya Nosov <k-nosov@yandex.ru>.
+   */
    if(!( id[0] == 'M' && id[1] == 'Z' &&
-        BMReadWordEx(0x18,BM_SEEK_SET) >= 0x40 &&
-        (ret=BMReadDWordEx(0x3C,BM_SEEK_SET)) > 0x40L)) ret = 0;
+        bmReadWordEx(0x18,BM_SEEK_SET) >= 0x40 &&
+        (ret=bmReadDWordEx(0x3C,BM_SEEK_SET)) > 0x40L)) ret = 0;
+#endif
+   if(!( id[0] == 'M' && id[1] == 'Z' &&
+        (ret=bmReadDWordEx(0x3C,BM_SEEK_SET)) > 0x02L)) ret = 0;
    return ret;
 }
 
@@ -295,7 +320,7 @@ static tBool __NEAR__ __FASTCALL__ LoadInfo( void )
        }
        else
        {
-          errnoMessageBox("Can not open file",NULL,errno);
+          errnoMessageBox(OPEN_FAIL,NULL,errno);
           return False;
        }
    }
@@ -317,40 +342,25 @@ static tBool __NEAR__ __FASTCALL__ LoadInfo( void )
  return True;
 }
 
-
 static void __NEAR__ __FASTCALL__ __detectBinFmt( void )
 {
- unsigned char ctrl[32];
  unsigned i;
- FType = NOT_EXE;
  if(!bmGetFLength())
  {
    detectedFormat = &binTable;
    return;
  }
- bmReadBufferEx(ctrl,sizeof(ctrl),0,BM_SEEK_SET);
- if(ctrl[0] == 0xFF && ctrl[1] == 0xFF && ctrl[2] == 0xFF && ctrl[3] == 0xFF)
- {
-     FType = DOS_SYS;
-     bmReadBufferEx((void  *)&oldh.drv,sizeof(DOSDRIVER),4,BM_SEEK_SET);
- }
- else
- if((ctrl[0] == 'M' && ctrl[1] == 'Z') ||
-    (ctrl[0] == 'Z' && ctrl[1] == 'M'))
- {
-   FType = OLD_EXE;
-   bmReadBufferEx((void  *)&oldh.mz,sizeof(MZHEADER),2,BM_SEEK_SET);
- }
  for(i = 0;i < sizeof(mainBinTable)/sizeof(REGISTRY_BIN *);i++)
  {
-     if(mainBinTable[i]->check_format())
-     {
-       detectedFormat = mainBinTable[i];
-       if(detectedFormat->init) detectedFormat->init();
-       if(i < sizeof(mainBinTable)/sizeof(REGISTRY_BIN *) - 1) FType = NEW_EXE;
-       break;
-     }
+   if(mainBinTable[i]->check_format())
+   {
+     detectedFormat = mainBinTable[i];
+     if(detectedFormat->init) detectedFormat->init();
+     break;
+   }
  }
+ /* Special case: mz initialization */
+ mzTable.check_format();
 }
 
 void PaintTitle( void )
@@ -360,7 +370,7 @@ void PaintTitle( void )
  twGotoXY(1,1);
  twClrEOL();
  twPrintF("File : %s",shortname);
- twGotoXY(Width-43,1);
+ twGotoXY(twGetClientWidth(TitleWnd)-43,1);
  twPrintF("Size : %8lu bytes",BMGetFLength());
  twRefreshWin(TitleWnd);
 }
@@ -376,15 +386,13 @@ static void MyAtExit( void )
   __term_sys();
 }
 
-extern unsigned char search_buff[21];
-extern unsigned char search_len;
-extern tBool search_Case,search_Word,search_Backward;
-char *ini_name;
-        
 tBool isValidIniArgs( void )
 {
   return iniSettingsAnywhere ? True :
-         ArgVector[1] ? strcmp(ArgVector[1],LastOpenFileName) == 0 : False;
+         ArgVector[1] ?
+	 strcmp(ArgVector[1],LastOpenFileName) == 0 ?
+	 biew_mode != UINT_MAX && biew_mode != LastMode ?
+	 False : True : False : False;
 }
 
 static hIniProfile * __NEAR__ __FASTCALL__ load_ini_info( void )
@@ -393,17 +401,24 @@ static hIniProfile * __NEAR__ __FASTCALL__ load_ini_info( void )
   hIniProfile *ini;
   ini_name = getenv("BIEW_INI");
   if(!ini_name) ini_name = __get_ini_name("biew");
-  ini = iniOpenFile(ini_name,NULL);
+  ini = UseIniFile ? iniOpenFile(ini_name,NULL) : NULL;
   biewReadProfileString(ini,"Biew","Setup","HelpName","",biew_help_name,sizeof(biew_help_name));
   biewReadProfileString(ini,"Biew","Setup","SkinName","",biew_skin_name,sizeof(biew_skin_name));
+  biewReadProfileString(ini,"Biew","Setup","SyntaxName","",biew_syntax_name,sizeof(biew_syntax_name));
   biewReadProfileString(ini,"Biew","Search","String","",(char *)search_buff,sizeof(search_buff));
   search_len = strlen((char *)search_buff);
   biewReadProfileString(ini,"Biew","Search","Case","off",tmp,sizeof(tmp));
-  search_Case = stricmp(tmp,"on") == 0 ? True: False;
+  biewSearchFlg = stricmp(tmp,"on") == 0 ? SF_CASESENS : SF_NONE;
   biewReadProfileString(ini,"Biew","Search","Word","off",tmp,sizeof(tmp));
-  search_Word = stricmp(tmp,"on") == 0 ? True : False;
+  if(stricmp(tmp,"on") == 0) biewSearchFlg |= SF_WORDONLY;
   biewReadProfileString(ini,"Biew","Search","Backward","off",tmp,sizeof(tmp));
-  search_Backward = stricmp(tmp,"on") == 0 ? True : False;
+  if(stricmp(tmp,"on") == 0) biewSearchFlg |= SF_REVERSE;
+  biewReadProfileString(ini,"Biew","Search","Template","off",tmp,sizeof(tmp));
+  if(stricmp(tmp,"on") == 0) biewSearchFlg |= SF_WILDCARDS;
+  biewReadProfileString(ini,"Biew","Search","UsePlugin","off",tmp,sizeof(tmp));
+  if(stricmp(tmp,"on") == 0) biewSearchFlg |= SF_PLUGINS;
+  biewReadProfileString(ini,"Biew","Search","AsHex","off",tmp,sizeof(tmp));
+  if(stricmp(tmp,"on") == 0) biewSearchFlg |= SF_ASHEX;
   biewReadProfileString(ini,"Biew","Browser","LastOpen","",LastOpenFileName,4096);
   sprintf(tmp,"%u",LastMode);
   biewReadProfileString(ini,"Biew","Browser","LastMode",tmp,tmp,sizeof(tmp));
@@ -421,6 +436,11 @@ static hIniProfile * __NEAR__ __FASTCALL__ load_ini_info( void )
   if(stricmp(tmp,"yes") == 0) biew_kbdFlags = KBD_NONSTOP_ON_MOUSE_PRESS;
   biewReadProfileString(ini,"Biew","Setup","IniSettingsAnywhere","no",tmp,sizeof(tmp));
   if(stricmp(tmp,"yes") == 0) iniSettingsAnywhere = True;
+  biewReadProfileString(ini,"Biew","Setup","FioUseMMF","no",tmp,sizeof(tmp));
+  if(stricmp(tmp,"yes") == 0) fioUseMMF = True;
+  if(!__mmfIsWorkable()) fioUseMMF = False;
+  biewReadProfileString(ini,"Biew","Setup","PreserveTimeStamp","no",tmp,sizeof(tmp));
+  if(stricmp(tmp,"yes") == 0) iniPreserveTime = True;
   return ini;
 }
 
@@ -432,11 +452,15 @@ static void __NEAR__ __FASTCALL__ save_ini_info( void )
   ini = iniOpenFile(ini_name,NULL);
   biewWriteProfileString(ini,"Biew","Setup","HelpName",biew_help_name);
   biewWriteProfileString(ini,"Biew","Setup","SkinName",biew_skin_name);
+  biewWriteProfileString(ini,"Biew","Setup","SyntaxName",biew_syntax_name);
   biewWriteProfileString(ini,"Biew","Setup","Version",BIEW_VERSION);
   biewWriteProfileString(ini,"Biew","Search","String",(char *)search_buff);
-  biewWriteProfileString(ini,"Biew","Search","Case",search_Case ? "on" : "off");
-  biewWriteProfileString(ini,"Biew","Search","Word",search_Word ? "on" : "off");
-  biewWriteProfileString(ini,"Biew","Search","Backward",search_Backward ? "on" : "off");
+  biewWriteProfileString(ini,"Biew","Search","Case",biewSearchFlg & SF_CASESENS ? "on" : "off");
+  biewWriteProfileString(ini,"Biew","Search","Word",biewSearchFlg & SF_WORDONLY ? "on" : "off");
+  biewWriteProfileString(ini,"Biew","Search","Backward",biewSearchFlg & SF_REVERSE ? "on" : "off");
+  biewWriteProfileString(ini,"Biew","Search","Template",biewSearchFlg & SF_WILDCARDS ? "on" : "off");
+  biewWriteProfileString(ini,"Biew","Search","UsePlugin",biewSearchFlg & SF_PLUGINS ? "on" : "off");
+  biewWriteProfileString(ini,"Biew","Search","AsHex",biewSearchFlg & SF_ASHEX ? "on" : "off");
   biewWriteProfileString(ini,"Biew","Browser","LastOpen",ArgVector[1]);
   sprintf(tmp,"%u",defMainModeSel);
   biewWriteProfileString(ini,"Biew","Browser","LastMode",tmp);
@@ -452,10 +476,17 @@ static void __NEAR__ __FASTCALL__ save_ini_info( void )
   biewWriteProfileString(ini,"Biew","Setup","MouseSens",tmp);
   strcpy(tmp,iniSettingsAnywhere ? "yes" : "no");
   biewWriteProfileString(ini,"Biew","Setup","IniSettingsAnywhere",tmp);
+  strcpy(tmp,fioUseMMF ? "yes" : "no");
+  biewWriteProfileString(ini,"Biew","Setup","FioUseMMF",tmp);
+  strcpy(tmp,iniPreserveTime ? "yes" : "no");
+  biewWriteProfileString(ini,"Biew","Setup","PreserveTimeStamp",tmp);
   if(activeMode->save_ini) activeMode->save_ini(ini);
   iniCloseFile(ini);
 }
 
+static FTime ftim;
+static tBool ftim_ok = False;
+
 int main( int argc, char *argv[] )
 {
  hIniProfile *ini;
@@ -473,7 +504,7 @@ int main( int argc, char *argv[] )
     size_t i;
     show_usage:
     printm("\n"BIEW_VER_MSG"\n");
-    printm(" Usage: biew [OPTIONS] [file...]\n\n");
+    printm(" Usage: biew [OPTIONS] file...\n\n");
     for(i = 0;i < sizeof(biewArg)/sizeof(struct tagbiewArg);i++)
     {
       printm("  %s\t%s\n",biewArg[i].key,biewArg[i].prompt);
@@ -486,31 +517,34 @@ int main( int argc, char *argv[] )
  initBConsole(biew_vioIniFlags,biew_twinIniFlags);
  ErrorWnd = WindowOpen(1,1,50,16,TWS_NONE | TWS_NLSOEM);
  if(ErrorWnd) twSetTitleAttr(ErrorWnd," Error ",TW_TMODE_CENTER,error_cset.border);
- else { printm("fatal error: can not create window"); return EXIT_FAILURE; }
+ else { printm("fatal error: can't create window"); return EXIT_FAILURE; }
  twCentredWin(ErrorWnd,NULL);
  twSetColorAttr(error_cset.main);
  twSetFrameAttr(ErrorWnd,TW_DOUBLE_FRAME,error_cset.border);
- if(skin_err)
- {
-   char sout[256];
-   sprintf(sout,"Detected error in skin file: '%s'",last_skin_error);
-   ErrMessageBox(sout,NULL);
- }
- HelpWnd = WindowOpen(1,Height,Width,Height,TWS_NLSOEM);
+ HelpWnd = WindowOpen(1,tvioHeight,tvioWidth,tvioHeight,TWS_NLSOEM);
  twSetColorAttr(prompt_cset.digit);
  twClearWin();
  twShowWin(HelpWnd);
  if(strcmp(biew_ini_ver,BIEW_VERSION) != 0) Setup();
- TitleWnd = WindowOpen(1,1,Width,1,TWS_NLSOEM);
+ TitleWnd = WindowOpen(1,1,tvioWidth,1,TWS_NONE);
  twSetColorAttr(title_cset.main);
  twClearWin();
  twShowWin(TitleWnd);
  activeMode = mainModeTable[1];
  atexit(MyAtExit);
  retval = EXIT_SUCCESS;
+ if(skin_err)
+ {
+   char sout[256];
+   sprintf(sout,"Error in skin file detected: '%s'",last_skin_error);
+   ErrMessageBox(sout,NULL);
+ }
+ /* We must do it before opening a file because of some RTL has bug
+    when are trying to open already open file with no sharing access */
+ ftim_ok = __OsGetFTime(ArgVector[1],&ftim);
  if(!LoadInfo())
  {
-   iniCloseFile(ini);
+   if(ini) iniCloseFile(ini);
    retval = EXIT_FAILURE;
    goto Bye;
  }
@@ -518,12 +552,11 @@ int main( int argc, char *argv[] )
  init_modes(ini);
  init_addons();
  init_sysinfo();
- iniCloseFile(ini);
- MainWnd = WindowOpen(1,2,Width,Height-1,TWS_NONE);
+ if(ini) iniCloseFile(ini);
+ MainWnd = WindowOpen(1,2,tvioWidth,tvioHeight-1,TWS_NONE);
  twSetColorAttr(browser_cset.main);
  twClearWin();
  PaintTitle();
- Height -= 2;
  if(!isValidIniArgs() || LastOffset > BMGetFLength()) LastOffset = 0;
  twShowWin(MainWnd);
  MainLoop();
@@ -534,6 +567,7 @@ int main( int argc, char *argv[] )
  term_modes();
  if(detectedFormat->destroy) detectedFormat->destroy();
  BMClose();
+ if(iniPreserveTime && ftim_ok) __OsSetFTime(ArgVector[1],&ftim);
  Bye:
  return retval;
 }
@@ -568,7 +602,9 @@ tBool NewSource( void )
   PFree(nlsListFile);
   if(i != -1)
   {
+    if(iniPreserveTime && ftim_ok) __OsSetFTime(ArgVector[1],&ftim);
     BMClose();
+    ftim_ok = __OsGetFTime(ListFile[i],&ftim);
     if(BMOpen(ListFile[i]) == 0)
     {
       ArgVector[1] = ListFile[i];
