@@ -1,4 +1,4 @@
-/** 
+/**
  * @namespace   biewlib
  * @file        biewlib/bbio.h
  * @brief       This file contains prototypes of BBIO technology functions.
@@ -45,6 +45,7 @@
                              & 90% backward from current file position */
 #define BIO_OPT_RBACKSCAN 0x0004
 #define BIO_OPT_DIRMASK  0x000F /**< direction mask */
+#define BIO_OPT_USEMMF   0xFFFF /**< Use mmf instead buffering i/o. This covers all optimizations */
 #define BIO_OPT_NOCACHE  0x8000 /**< disable cache */
 
 #define BIO_SEEK_SET     SEEKF_START /**< specifies reference location from begin of file */
@@ -52,6 +53,46 @@
 #define BIO_SEEK_END     SEEKF_END   /**< specifies reference location from end of file */
 
 typedef void * BGLOBAL;       /**< This is the data type used to represent buffered stream objects */
+
+/*
+   This struct is ordered as it documented in Athlon manual
+   Publication # 22007 Rev: D
+*/
+/** Virtual file buffer structure */
+typedef struct tagvfb
+{
+ int             handle;    /**< file handle */
+ unsigned long   FBufStart; /**< logical position of mirror the buffer onto file */
+ char *          MBuffer;   /**< NULL - not buffered i/o */
+ unsigned        MBufLen;   /**< length data, actually contains in buffer */
+ unsigned        MBufSize;  /**< real size of buffer */
+ tBool           updated;   /**< True if buffer contains data, that not pesent in file */
+}vfb;
+
+/** Memory mapped buffer structure */
+typedef struct tagmmb
+{
+ mmfHandle       mmf;       /**< If OS support MMF contains handle of memory-mapped file */
+ void *          mmf_addr;  /**< If OS support MMF contains base address of memory where file is mapped */
+}mmb;
+
+typedef struct tagBFILE
+{
+ unsigned long   FilePos;   /**< current logical position in file */
+ unsigned long   FLength;   /**< real length of the file */
+ char *          FileName;  /**< Real file name of opened file */
+ unsigned        openmode;  /**< mode,that OsOpen this file */
+ int             optimize;  /**< seek optimization */
+ tBool           is_mmf;    /**< indicates that 'mmb' is used */
+ tBool           primary_mmf; /**< If this is set then we have not duplicated handle */
+ union                      /**< cache subsystem */
+ {
+    vfb          vfb;       /**< buffered file */
+    mmb *        mmb;       /**< Pointer to memory mapped file. We must have pointer, but not object!!! It for bioDupEx */
+ }b;
+ tBool           is_eof;    /**< Indicates EOF for buffering streams */
+}BFILE;
+
 extern struct tagBFILE bNull; /**< Stream associated with STDERR */
 
                    /** Opens existed file and buffered it
@@ -62,7 +103,7 @@ extern struct tagBFILE bNull; /**< Stream associated with STDERR */
                      * @note                  Returns bNull if opening is fail.
                      * @warning               Carefully use parameter - buffSize.
                      *                        If you created new file with 0 bytes
-                     *                        of length, and value of buffSize = 
+                     *                        of length, and value of buffSize =
                      *                        UINT_MAX, then i/o will be unbuffered.
                      *                        It better to use values < UINT_MAX.
                      *                        Value UINT_MAX better to use for readonly
@@ -70,7 +111,7 @@ extern struct tagBFILE bNull; /**< Stream associated with STDERR */
                      *                        memory entire.
                      * @see                   bioClose
                     **/
-BGLOBAL               __FASTCALL__ bioOpen(const char * fname,unsigned openmode,unsigned buffSize);
+BGLOBAL               __FASTCALL__ bioOpen(const char * fname,unsigned openmode,unsigned buffSize,unsigned optimization);
 
                    /** Changes size of opened file.
                      * @return                True if operation was succesfully performed
@@ -111,7 +152,7 @@ unsigned long         __FASTCALL__ bioFLength(BGLOBAL bioFile);
                      *                        file position is out of buffer.
                      * @see                   bioReRead
                     **/
-tBool                 __FASTCALL__ bioFlush(BGLOBAL bioFile); 
+tBool                 __FASTCALL__ bioFlush(BGLOBAL bioFile);
 
                    /** Reads one byte from stream.
                      * @return                Readed byte
@@ -120,7 +161,7 @@ tBool                 __FASTCALL__ bioFlush(BGLOBAL bioFile);
                      *                        incremented after operation.
                      * @see                   bioWriteByte bioReadWord bioReadDWord bioReadBuffer
                     **/
-uint8_t               __FASTCALL__ bioReadByte(BGLOBAL bioFile);
+tUInt8               __FASTCALL__ bioReadByte(BGLOBAL bioFile);
 
                    /** Reads two bytes from stream.
                      * @return                Readed word
@@ -129,7 +170,7 @@ uint8_t               __FASTCALL__ bioReadByte(BGLOBAL bioFile);
                      *                        incremented after operation.
                      * @see                   bioWriteWord bioReadByte bioReadDWord bioReadBuffer
                     **/
-uint16_t              __FASTCALL__ bioReadWord(BGLOBAL bioFile);
+tUInt16              __FASTCALL__ bioReadWord(BGLOBAL bioFile);
 
                    /** Reads four bytes from stream.
                      * @return                Readed double word
@@ -138,7 +179,7 @@ uint16_t              __FASTCALL__ bioReadWord(BGLOBAL bioFile);
                      *                        incremented after operation.
                      * @see                   bioWriteDWord bioReadByte bioReadWord bioReadBuffer
                     **/
-uint32_t              __FASTCALL__ bioReadDWord(BGLOBAL bioFile);
+tUInt32              __FASTCALL__ bioReadDWord(BGLOBAL bioFile);
 
                    /** Reads specified number of bytes from stream.
                      * @return                True if operation was succesfully performed
@@ -156,7 +197,7 @@ tBool                 __FASTCALL__ bioReadBuffer(BGLOBAL bioFile,void * buffer,u
                      * @param bioFile         handle of opened stream
                      * @see                   bioFlush
                     **/
-tBool                 __FASTCALL__ bioReRead(BGLOBAL bioFile); 
+tBool                 __FASTCALL__ bioReRead(BGLOBAL bioFile);
 
                    /** Positions logical file pointer at the specified position.
                      * @return                True if operation was succesfully performed
@@ -196,7 +237,7 @@ unsigned long         __FASTCALL__ bioTell(BGLOBAL bioFile);
                      *                        incremented after operation.
                      * @see                   bioReadByte bioWriteWord bioWriteWord bioWriteBuffer
                     **/
-tBool                 __FASTCALL__ bioWriteByte(BGLOBAL bioFile,uint8_t bVal);
+tBool                 __FASTCALL__ bioWriteByte(BGLOBAL bioFile,tUInt8 bVal);
 
                    /** Writes two bytes to stream.
                      * @return                True if operation was succesfully performed
@@ -206,7 +247,7 @@ tBool                 __FASTCALL__ bioWriteByte(BGLOBAL bioFile,uint8_t bVal);
                      *                        incremented after operation.
                      * @see                   bioReadWord bioWriteWord bioWriteWord bioWriteBuffer
                     **/
-tBool                 __FASTCALL__ bioWriteWord(BGLOBAL bioFile,uint16_t wVal);
+tBool                 __FASTCALL__ bioWriteWord(BGLOBAL bioFile,tUInt16 wVal);
 
                    /** Writes four bytes to stream.
                      * @return                True if operation was succesfully performed
@@ -216,7 +257,7 @@ tBool                 __FASTCALL__ bioWriteWord(BGLOBAL bioFile,uint16_t wVal);
                      *                        incremented after operation.
                      * @see                   bioReadDWord bioWriteWord bioWriteWord bioWriteBuffer
                     **/
-tBool                 __FASTCALL__ bioWriteDWord(BGLOBAL bioFile,uint32_t dwVal);
+tBool                 __FASTCALL__ bioWriteDWord(BGLOBAL bioFile,tUInt32 dwVal);
 
                    /** Writes specified number of bytes opened to stream.
                      * @return                True if operation was succesfully performed
