@@ -1,4 +1,4 @@
-/** 
+/**
  * @namespace   biewlib
  * @file        biewlib/file_ini.c
  * @brief       This file contains implementation of .ini files services.
@@ -53,6 +53,10 @@ const char iniLegalSet[] = " _0123456789"
 
 #define IS_VALID_NAME(name) (strspn(name,iniLegalSet) == strlen(name))
 #define IS_SECT(str,ch) (str[0] == ch)
+#define FiisSection( str ) (IS_SECT(str,'['))
+#define FiisSubSection( str ) (IS_SECT(str,'<'))
+#define FiisCommand( str ) (IS_SECT(str,'#'))
+#define FiisItem(str) (!(FiisSection(str) || FiisSubSection(str) || FiisCommand(str)))
 
 static char * __NEAR__ __FASTCALL__ GETS(char *str,unsigned num,BGLOBAL h)
 {
@@ -110,22 +114,22 @@ void __FASTCALL__ FiAErrorCL(int nError) { FiAError(nError,FinCurrString[FiFileP
 
 static const char * list[] = {
  "No errors",
- "Can not open file (bad '#include' statement?).",
- "Too many opened files.",
+ "Can't open file (bad '#include' statement?).",
+ "Too many open files.",
  "Memory exhausted.",
- "Opened 'if' (missing '#endif').",
+ "Open 'if' (missing '#endif').",
  "Missing 'if' for 'endif' statement.",
  "Missing 'if' for 'else' statement.",
  "Unknown '#' directive.",
  "Syntax error in 'if' statement.",
- "Expected opened section or subsection or invalid string.",
+ "Expected open section or subsection, or invalid string.",
  "Bad character on line (possible lost comment).",
  "Bad variable in 'set' or 'delete' statement.",
  "Bad value of variable in 'set' statement.",
  "Unrecognized name of variable in 'delete' statement.",
- "Detected undefined variable (case sensivity?).",
+ "Undefined variable detected (case sensivity?).",
  "Missing 'if' for 'elif' statement.",
- "Opened variable on line (use even number of '%' characters).",
+ "Open variable on line (use even number of '%' characters).",
  "Lost or mismatch character '=' in assigned expression.",
  "",
  "User error."
@@ -169,7 +173,7 @@ FiHandler __FASTCALL__ FiOpen( const char * filename)
   char * activeFile;
   FiHandler ret;
   /* Try to load .ini file entire into memory */
-  ret = bioOpen(filename,FO_READONLY | SO_DENYWRITE,UINT_MAX);
+  ret = bioOpen(filename,FO_READONLY | SO_DENYWRITE,UINT_MAX,BIO_OPT_USEMMF);
   if(ret == &bNull )
   {
     FiAError(__FI_BADFILENAME,0);
@@ -212,26 +216,6 @@ void __FASTCALL__ FiClose(FiHandler h)
   bioClose(h);
 }
 
-tBool       __FASTCALL__ FiisSection( const char * str )
-{
-  return IS_SECT(str,'[');
-}
-
-tBool       __FASTCALL__ FiisSubSection( const char * str )
-{
-  return IS_SECT(str,'<');
-}
-
-tBool       __FASTCALL__ FiisCommand( const char * str )
-{
-  return IS_SECT(str,'#');
-}
-
-tBool       __FASTCALL__ FiisItem( const char *str)
-{
-  return !(FiisSection(str) || FiisSubSection(str) || FiisCommand(str));
-}
-
 static unsigned int __NEAR__ __FASTCALL__ __GetLengthBrStr(const char * src,char obr,char cbr)
 {
  char *ends;
@@ -274,35 +258,12 @@ static char * __NEAR__ __FASTCALL__ __GetBrStrName(const char * src,char * store
  return store;
 }
 
-unsigned int  __FASTCALL__ FiGetLengthBracketString( const char * str )
-{
-  return __GetLengthBrStr(str,'"','"');
-}
-
-char * __FASTCALL__ FiGetBracketString(const char * str,char * store)
-{
-  return __GetBrStrName(str,store,'"','"');
-}
-
-char *    __FASTCALL__ FiGetSectionName(const char * src,char * store)
-{
-  return __GetBrStrName(src,store,'[',']');
-}
-
-char *    __FASTCALL__ FiGetSubSectionName(const char * src,char * store)
-{
-  return __GetBrStrName(src,store,'<','>');
-}
-
-unsigned int  __FASTCALL__ FiGetLengthSection( const char * src )
-{
-  return __GetLengthBrStr(src,'[',']');
-}
-
-unsigned int  __FASTCALL__ FiGetLengthSubSection( const char * src )
-{
-  return __GetLengthBrStr(src,'<','>');
-}
+#define FiGetLengthBracketString( str ) (__GetLengthBrStr(str,'"','"'))
+#define FiGetBracketString( str, store) (__GetBrStrName(str,store,'"','"'))
+#define FiGetSectionName( src, store) (__GetBrStrName(src,store,'[',']'))
+#define FiGetSubSectionName( src, store) (__GetBrStrName(src,store,'<','>'))
+#define FiGetLengthSection( src ) (__GetLengthBrStr(src,'[',']'))
+#define FiGetLengthSubSection( src ) (__GetLengthBrStr(src,'<','>'))
 
 unsigned int  __FASTCALL__ FiGetLengthItem( const char * src )
 {
@@ -728,8 +689,8 @@ char * __FASTCALL__ FiGetNextString(FiHandler h, char * str,unsigned int size,ch
       szTrimLeadingSpace(str);
       /* kill all spaces around punctuations */
       /*
-         Correct me: spaces will sqweezed even inside " " brackets.
-         But it languag depended.
+         Correct me: spaces will be sqweezed even inside " " brackets.
+         But it is language dependent.
          For .ini files it is not significant.
       */
       sret = str;
@@ -986,10 +947,18 @@ tBool __FASTCALL__ FiCommandProcessorStd( const char * cmd )
  {
    char * bracket;
    char _v[FI_MAXSTRLEN+1];
+   char pfile[FILENAME_MAX+1],*pfp,*pfp2;
    bracket = __FiCBString(&str.str[str.iptr]);
    FiExpandAllVar(bracket,_v);
    fdeb_save = fi_Debug_Str;
-   (*FiFileProcessor)(_v);
+   /* make path if no path specified */
+   strcpy(pfile,FiFileNames[FiFilePtr-1]);
+   pfp=strrchr(pfile,'\\');
+   pfp2=strrchr(pfile,'/');
+   pfp=max(pfp,pfp2);
+   if(pfp && !(strchr(_v,'\\') || strchr(_v,'/'))) strcpy(pfp+1,_v);
+   else    strcpy(pfile,_v);
+   (*FiFileProcessor)(pfile);
    fi_Debug_Str = fdeb_save;
    PFREE(bracket);
    goto Exit_CP;
@@ -1181,22 +1150,21 @@ tBool __FASTCALL__ FiStringProcessorStd(char * curr_str)
     else
     if(FiisItem(curr_str))
     {
-      char buffer[FI_MAXSTRLEN+1],bufferv[FI_MAXSTRLEN+1];
+      char buffer[FI_MAXSTRLEN+1];
       tBool retval;
       IniInfo info;
-      item = __FiCItem(curr_str);
+      FiExpandAllVar(curr_str,buffer);
+      item = __FiCItem(buffer);
       retval = False;
       if(item[0])
       {
-       val = __FiCValue(curr_str);
-       FiExpandAllVar(val,buffer);
-       FiExpandAllVar(item,bufferv);
+       val = __FiCValue(buffer);
        if(curr_sect) info.section = curr_sect;
        else info.section = "";
        if(curr_subsect) info.subsection = curr_subsect;
        else info.subsection = "";
-       info.item = bufferv;
-       info.value = buffer;
+       info.item = item;
+       info.value = val;
        retval = (*proc)(&info);
        PFREE(val);
       }
@@ -1315,7 +1283,7 @@ static tCompare __FASTCALL__ __full_compare_cache(const void __HUGE__ *v1,const
   tCompare i_ret;
   c1 = (const ini_cache __HUGE__ *)v1;
   c2 = (const ini_cache __HUGE__ *)v2;
-  iflg = c1->flags == c2->flags ? 0 : c1->flags > c2->flags ? 1 : -1;
+  iflg = __CmpLong__(c1->flags,c2->flags);
   i_ret = strcmp(c1->item,c2->item);
   return i_ret ? i_ret : iflg;
 }
@@ -1439,10 +1407,7 @@ static tBool __FASTCALL__ __buildCache(IniInfo *ini)
 
 static void __FASTCALL__ __iter_destroy(void __HUGE__ *it);
 
-static void __destroyCache(linearArray *it)
-{
-   la_IterDestroy(it,__iter_destroy);
-}
+#define __destroyCache(it) (la_IterDestroy(it,__iter_destroy))
 
 static void __FASTCALL__ __iter_destroy(void __HUGE__ *it)
 {
@@ -1652,7 +1617,7 @@ unsigned __FASTCALL__ iniReadProfileString(hIniProfile *ini,const char *section,
    return ret;
 }
 
-#define HINI_HEADER "; This file generated automatically by BIEWLIB.\n; WARNING: Any changes made by hands can be lost at following program start.\n"
+#define HINI_HEADER "; This file generated automatically by BIEWLIB.\n; WARNING: Any changes made by hands may be lost at following program start.\n"
 
 static FILE * __NEAR__ __FASTCALL__ __makeIni(hIniProfile *ini)
 {
