@@ -1,4 +1,4 @@
-/** 
+/**
  * @namespace   biewlib
  * @file        biewlib/kbd_code.h
  * @brief       This file includes OS depended keyboard codes.
@@ -25,7 +25,7 @@
   #if defined(__WIN32__) && defined(_MSC_VER)
     #define __OS_KEYBOARD <biewlib/sysdep/ia32/win32/kbd_code.h>
   #else
-    #define __OS_KEYBOARD <biewlib/sysdep/##__MACHINE__/##__OS__/kbd_code.h>
+    #define __OS_KEYBOARD <biewlib/sysdep/__MACHINE__/__OS__/kbd_code.h>
   #endif
   #include __OS_KEYBOARD
 #endif
