@@ -1,4 +1,4 @@
-/** 
+/**
  * @namespace   biewlib
  * @file        biewlib/sysdep/__os_dep.h
  * @brief       This file contains all operating system depended part of BIEW project.
@@ -80,6 +80,16 @@ extern int       __FASTCALL__ __kbdTestKey( unsigned long flg );
                     **/
 extern int       __FASTCALL__ __kbdGetShiftsKey( void );
 
+                   /** Describes info about input events.
+                     * @param head            Pointer to header to be filled
+                     * @param text            Pointer to formatted text to be filled
+                     * @return                0 if ESCAPE's functionality key pressed;
+                                              1 if regulare event occured
+                                              -1 if functionality is not implemented(supported).
+                    **/
+extern int       __FASTCALL__ __inputRawInfo(char *head, char *text);
+
+
 #if __WORDSIZE == 16
 typedef unsigned char tAbsCoord; /**< This is the data type used to represent screen-related coordinates */
 #else
@@ -135,8 +145,8 @@ extern int       __FASTCALL__ __MsGetBtns( void );
 
 /* VIDEO subsystem handling */
 
-typedef uint8_t  ColorAttr; /**< This is the data type used to represent attributes of color */
-typedef uint8_t  t_vchar;   /**< This is the data type used to represent video character */
+typedef tUInt8 ColorAttr; /**< This is the data type used to represent attributes of color */
+typedef tUInt8 t_vchar;   /**< This is the data type used to represent video character */
 
 /** Internal structure of video buffer */
 typedef struct tag_tvioBuff
@@ -277,6 +287,24 @@ extern void      __FASTCALL__ __term_sys( void );
                     **/
 extern void      __FASTCALL__ __OsYield( void );
 
+                   /** Gets ctrl-break signal
+                     * @return                True if occured; False - otherwise
+                     * @note                  Function does not differ soft and
+                     *                        hard c-break.
+                     * @warning               After getting signal by program
+                                              semaphore is not being cleaned.
+                    **/
+extern tBool     __FASTCALL__ __OsGetCBreak( void );
+
+                   /** Sets control-break signal
+                     * @return                none
+                     * @param state           indicates new state of
+                                              control-break semaphore
+                     * @note                  Function does not differ soft and
+                     *                        hard c-break.
+                    **/
+extern void      __FASTCALL__ __OsSetCBreak( tBool state );
+
                    /** Builds OS specific name of initializing file
                      * @return                fully qualified name of .ini file
                      * @param progname        indicates name of program, that will be used as part of .ini file name
@@ -428,7 +456,7 @@ extern int       __FASTCALL__ __OsRename(const char *oldname,const char *newname
                      * @param handle          handle of opened stream
                      * @param buff            pointer to memory where will be stored readed information
                      * @param size            specifies number of bytes to be readed from file
-                     * @note                  Function reads specified number of bytes 
+                     * @note                  Function reads specified number of bytes
                      *                        from current file position. File
                      *                        position is changed after reading
                      *                        on specified number of bytes.
@@ -441,7 +469,7 @@ extern int       __FASTCALL__ __OsRead(int handle,void *buff,unsigned size);
                      * @param handle          handle of opened stream
                      * @param buff            pointer to memory to be written
                      * @param size            specifies number of bytes to be writed to file
-                     * @note                  Function writes specified number of bytes 
+                     * @note                  Function writes specified number of bytes
                      *                        at current file position. File
                      *                        position is changed after writing
                      *                        on specified number of bytes.
@@ -449,6 +477,112 @@ extern int       __FASTCALL__ __OsRead(int handle,void *buff,unsigned size);
                     **/
 extern int       __FASTCALL__ __OsWrite(int handle,const void *buff,unsigned size);
 
+/** Structure for storing and setting file time information */
+typedef struct tagFTime
+{
+   unsigned long acctime; /**< contains last access time */
+   unsigned long modtime; /**< constains modification time */
+}FTime;
+                   /** Gets last access and modification time for given file.
+                     * @return                True if success False otherwise
+                     * @param name            specified name of file
+                     * @param data            pointer to memory where information will be stored
+                     * @see                   __OsSetFTime
+                    **/
+extern tBool      __FASTCALL__ __OsGetFTime(const char *name,FTime *data);
+
+                   /** Sets last access and modification time for given file.
+                     * @return                True if success False otherwise
+                     * @param name            specified name of file
+                     * @param data            pointer to memory where information is stored
+                     * @see                   __OsGetFTime
+                    **/
+extern tBool      __FASTCALL__ __OsSetFTime(const char *name,const FTime *data);
+
+/*
+   First edition of Memory Mapped File Support (For modern OS).
+   Note: MMF does not support preffered memory addresses and many other features
+         of standard mmap technology.
+   In this release it would be better to use this technique only for read only
+   operations.
+*/
+
+typedef void * mmfHandle;
+
+                   /** Opens existed file and mapped it into memory.
+                     * @return                handle of opened stream if successful, NULL otherwise
+                     * @param name            name of existed file to be opened
+                     * @param mode            combination of FO_* and SO_* flags
+                     * @see                   __mmfClose
+                    **/
+mmfHandle          __FASTCALL__ __mmfOpen(const char *fname,int mode);
+
+                   /** Flushes memory mapped file to the physical file.
+                     * @return                True if successful, False otherwise
+                     * @param mh              handle of opened stream
+                     * @see                   __mmfOpen __mmfClose __mmfSync
+                    **/
+tBool              __FASTCALL__ __mmfFlush(mmfHandle mh);
+
+                   /** Synchronizes memory mapped file with the physical file.
+                     * @param mh              handle of opened stream
+                     * @return                mh if successful, NULL otherwise.
+                     * @note                  This function performs both operations
+                                              writting to the file of dirty
+                                              areas and rereading of the file
+                                              back into the memory. On failure
+                                              it closed all handles and freed
+                                              mmfHandle structure.
+                     * @see                   __mmfFlush
+                    **/
+mmfHandle          __FASTCALL__ __mmfSync(mmfHandle mh);
+
+                   /** Changes access to the memory mapping of file.
+                     * @return                True if successful, False otherwise
+                     * @param mh              handle of opened stream
+                     * @param flags           combination of FO_* and SO_* flags
+                     * @see                   __mmfOpen __mmfClose
+                    **/
+tBool              __FASTCALL__ __mmfProtect(mmfHandle mh,int flags);
+
+                   /** Changes size of existed file mapping.
+                     * @return                True if successful, False otherwise
+                     * @param mh              handle of opened stream
+                     * @param size            new size of mapping in bytes
+                     * @note                  May be not supported by some OS.
+                     * @warning               If mapping is truncated, the data from
+                     *                        the new end of file to the original
+                     *                        end of the file are lost.
+                     * @see                   __mmfOpen __mmfClose __mmfProtect
+                    **/
+tBool              __FASTCALL__ __mmfResize(mmfHandle mh,long size);
+
+                   /** Closes existed file mapping.
+                     * @return                None
+                     * @param mh              handle of stream to be closed
+                     * @see                   __mmfOpen
+                    **/
+void               __FASTCALL__ __mmfClose(mmfHandle mh);
+
+                   /** Return address in memory where file mapping is assigned.
+                     * @return                Assigned address of file mapping or -1 on error
+                     * @param mh              handle of stream to be closed
+                     * @see                   __mmfOpen
+                    **/
+void *             __FASTCALL__ __mmfAddress(mmfHandle mh);
+
+                   /** Returns size of opened MMF stream.
+                     * @return                Length in bytes of opened MMF stream
+                     * @param mh              handle of stream
+                     * @see                   __mmfOpen __mmfAddress
+                    **/
+long              __FASTCALL__ __mmfSize(mmfHandle mh);
+
+                   /** Returns workability of MMF subsystem.
+                     * @return                True if mmf is workable on given OS
+                    **/
+tBool             __FASTCALL__ __mmfIsWorkable( void );
+
 /* National Language Support */
 
            /** Checks whether the specified character is OEM pseudographical symbol */
@@ -466,7 +600,7 @@ extern void      __FASTCALL__ __nls_PrepareOEMForTVio(tvioBuff *it,unsigned size
                      * @return                none
                      * @param str             buffer to be converted
                      * @param size            size of buffer in bytes
-                     * @see                   __nls_CmdlineToOem
+                     * @see                   __nls_CmdlineToOem __nls_CmdlineToFs
                     **/
 extern void      __FASTCALL__ __nls_OemToOsdep(unsigned char *str,unsigned size);
 
@@ -477,10 +611,18 @@ extern void      __FASTCALL__ __nls_OemToOsdep(unsigned char *str,unsigned size)
                      * @depricated            This function only used in Win32
                      *                        where codepages of console and command
                      *                        line is differ.
-                     * @see                   __nls_OemToOsdep
+                     * @see                   __nls_OemToOsdep __nls_CmdlineToFs
                     **/
 extern void      __FASTCALL__ __nls_CmdlineToOem(unsigned char *str,unsigned size);
 
+                   /** Converts buffer from OEM codepage to currently used by OS's file system.
+                     * @return                none
+                     * @param str             buffer to be converted
+                     * @param size            size of buffer in bytes
+                     * @see                   __nls_OemToOsdep __nls_CmdlineToOem
+                    **/
+extern void      __FASTCALL__ __nls_OemToFs(unsigned char *str,unsigned size);
+
 #ifdef __cplusplus
 }
 #endif
