@@ -1,4 +1,4 @@
-/** 
+/**
  * @namespace   biewlib
  * @file        biewlib/sysdep/ia16/win/nls.c
  * @brief       This file contains implementation of OEM codepages support for Win16
@@ -20,7 +20,12 @@
 void __FASTCALL__ __nls_OemToOsdep(unsigned char *buff,unsigned len)
 {
   OemToAnsiBuff(buff,buff,len);
-};
+}
+
+void __FASTCALL__ __nls_OemToFs(unsigned char *buff,unsigned len)
+{
+  OemToAnsiBuff(buff,buff,len);
+}
 
 void __FASTCALL__ __nls_CmdlineToOem(unsigned char *buff,unsigned len)
 {
