@@ -1,4 +1,4 @@
-/** 
+/**
  * @namespace   biewlib
  * @file        biewlib/sysdep/ia32/cpu_info.c
  * @brief       This file contains function for retrieving CPU information for
@@ -27,12 +27,12 @@
 #include "biewlib/biewlib.h"
 
 #define CPU_CLONE     0x000F
-#define HAVE_FPU      0x8000
-#define HAVE_CPUID    0x4000
-#define HAVE_MMX      0x2000
-#define HAVE_SSE      0x1000
+#define __HAVE_FPU    0x8000
+#define __HAVE_CPUID  0x4000
+#define __HAVE_MMX    0x2000
+#define __HAVE_SSE    0x1000
 
-#if defined(__GNUC__) && defined(NDEBUG) && !defined(__BEOS__)
+#if !defined(__DISABLE_ASM) && (defined(__GNUC__) && defined(NDEBUG))
 
 static unsigned __NEAR__ __FASTCALL__ __cpu_type( void )
 {
@@ -82,7 +82,7 @@ static unsigned __NEAR__ __FASTCALL__ __cpu_type( void )
       "	jz	0f\n"
       "	orl	$0x2000, %0\n"
 "0:\n"
-      "	testl   $0x200000, %%edx\n"
+      "	testl   $0x2000000, %%edx\n"
       "	jz	1f\n"
       "	orl	$0x1000, %0\n"
 "1:\n"
@@ -727,13 +727,22 @@ static unsigned long __NEAR__ __FASTCALL__ __SSEOPS_std(volatile unsigned *count
 #define __ASMPART_DEFINED 1
 #include "biewlib/sysdep/ia16/cmn_ix86.c"
 
+#elif defined(__WATCOMC__) && __WATCOMC__ >= 1100
+
+#include "biewlib/sysdep/ia32/cpu_info.wc"
+
+#define __ASMPART_DEFINED 1
+#include "biewlib/sysdep/ia16/cmn_ix86.c"
+
+#elif defined(__WATCOMC__) && defined(__QNX4__)
+
+#include "biewlib/sysdep/ia32/qnx/cpu_info.qnx"
+
+#define __ASMPART_DEFINED 1
+#include "biewlib/sysdep/ia16/cmn_ix86.c"
+
 #else
 
-#include <stdio.h>
-
-void __FillCPUInfo(char *buff,unsigned cbBuff,void (*percent_callback)(int))
-{
-	sprintf(buff,"\n\n\n\n\n         CPU info currently N/A under current compiled BIEW");
-}
+#include "biewlib/sysdep/generic/cpu_info.c"
 
 #endif
