@@ -1,4 +1,4 @@
-/** 
+/**
  * @namespace   biewlib
  * @file        biewlib/sysdep/ia32/win32/nls.c
  * @brief       This file contains implementation of OEM codepages support
@@ -18,7 +18,7 @@
 
 #include "biewlib/biewlib.h"
 
-extern tBool win32_use_ansi;
+tBool win32_use_ansi;
 
 void __FASTCALL__ __nls_OemToOsdep(unsigned char *buff,unsigned len)
 {
@@ -28,6 +28,10 @@ void __FASTCALL__ __nls_OemToOsdep(unsigned char *buff,unsigned len)
  }
 }
 
+void __FASTCALL__ __nls_OemToFs(unsigned char *buff,unsigned len)
+{
+}
+
 void __FASTCALL__ __nls_CmdlineToOem(unsigned char *buff,unsigned len)
 {
    CharToOemBuff(buff,buff,len);
