@@ -1,4 +1,4 @@
-/** 
+/**
  * @namespace   biew
  * @file        fileutil.c
  * @brief       This file contains file utilities of BIEW project.
@@ -36,6 +36,8 @@
 #include "biewlib/twin.h"
 #include "biewlib/kbd_code.h"
 
+extern tBool fioUseMMF;
+
 static tBool ChSize( void )
 {
  long psize,tile = 0;
@@ -48,22 +50,24 @@ static tBool ChSize( void )
     if(psize > 0)
     {
        tBool ret;
+       int my_errno = 0;
        char *fname = BMName();
        BGLOBAL bHandle;
        bHandle = biewOpenRW(fname,BBIO_SMALL_CACHE_SIZE);
        if(bHandle == &bNull)
        {
          err:
-         errnoMessageBox("Can not change size",NULL,errno);
+         errnoMessageBox(RESIZE_FAIL,NULL,my_errno);
          return False;
        }
        ret = bioChSize(bHandle,psize);
+       my_errno = errno;
        bioClose(bHandle);
        if(ret == False) goto err;
        BMReRead();
        return ret;
     }
-    else ErrMessageBox("New length has invalid value",NULL);
+    else ErrMessageBox("Invalid new length",NULL);
   }
  }
  return False;
@@ -81,7 +85,7 @@ static tBool __NEAR__ __FASTCALL__ InsBlock(BGLOBAL bHandle,unsigned long start,
    if(!buffer) return 0;
    if(!bioChSize(bHandle,oflen+psize))
    {
-     ErrMessageBox("Can not expand file",NULL);
+     ErrMessageBox(EXPAND_FAIL,NULL);
      PFREE(buffer);
      return False;
    }
@@ -139,7 +143,7 @@ static tBool __NEAR__ __FASTCALL__ DelBlock(BGLOBAL bHandle,unsigned long start,
    PFREE(buffer);
    if(!bioChSize(bHandle,oflen+psize))
    {
-     ErrMessageBox("Can not truncate file",NULL);
+     ErrMessageBox(TRUNC_FAIL,NULL);
      PFREE(buffer);
    }
    return True;
@@ -157,14 +161,14 @@ static tBool InsDelBlock( void )
     BGLOBAL bHandle;
     char *fname;
     fpos = BMGetCurrFilePos();
-    if(start > BMGetFLength()) { ErrMessageBox("Can not start outside the file",NULL); return 0; }
+    if(start > BMGetFLength()) { ErrMessageBox("Start is outside of file",NULL); return 0; }
     if(!psize) return 0;
     if(psize < 0) if(start+labs(psize) > BMGetFLength()) { ErrMessageBox("Use change size operation instead of block deletion",NULL); return 0; }
     fname = BMName();
     bHandle = biewOpenRW(fname,BBIO_SMALL_CACHE_SIZE);
     if(bHandle == &bNull)
     {
-      errnoMessageBox("Can not open file",NULL,errno);
+      errnoMessageBox(OPEN_FAIL,NULL,errno);
     }
     else
     {
@@ -179,6 +183,7 @@ static tBool InsDelBlock( void )
 }
 
 static char ff_fname[FILENAME_MAX+1] = "biew.$$$";
+static char xlat_fname[FILENAME_MAX+1];
 static unsigned long ff_startpos = 0L,ff_len = 0L;
 
 static void __NEAR__ __FASTCALL__ printObject(FILE *fout,unsigned obj_num,char *oname,int oclass,int obitness,unsigned long size)
@@ -229,7 +234,7 @@ static void __NEAR__ __FASTCALL__ printHdr(FILE * fout,REGISTRY_BIN *fmt)
 static unsigned __NEAR__ __FASTCALL__ printHelpComment(char *buff,MBuffer codebuff,DisasmRet *dret)
 {
   unsigned len,j;
-   if(dis_severity > DISCOMSEV_NONE) 
+   if(dis_severity > DISCOMSEV_NONE)
    {
      len = 3+strlen(dis_comments);
      strcat(buff,dis_comments);
@@ -239,8 +244,8 @@ static unsigned __NEAR__ __FASTCALL__ printHelpComment(char *buff,MBuffer codebu
    for(j = 0;j < dret->codelen;j++)
    {
       memcpy((char *)&buff[len],(char *)Get2Digit(codebuff[j]),2);
-      len += 2; 
-   }   
+      len += 2;
+   }
    buff[len] = 0;
    return len;
 }
@@ -275,7 +280,7 @@ static tBool FStore( void )
  DumpMode = True;
  ff_startpos = BMGetCurrFilePos();
  if(!ff_len) ff_len = BMGetFLength() - ff_startpos;
- if(GetFStoreDlg(" Store information to File [F2 - Asm/Bin F3 - comment F4 - structs] ",ff_fname,&flags,&ff_startpos,&ff_len,FILE_PRMT))
+ if(GetFStoreDlg(" Store information to file ",ff_fname,&flags,&ff_startpos,&ff_len,FILE_PRMT))
  {
   endpos = ff_startpos + ff_len;
   endpos = endpos > BMGetFLength() ? BMGetFLength() : endpos;
@@ -300,14 +305,14 @@ static tBool FStore( void )
        if(handle == -1)
        {
           use_err:
-          errnoMessageBox("Can use file",NULL,errno);
+          errnoMessageBox("Can't use file",NULL,errno);
           goto Exit;
        }
        __OsTruncFile(handle,0L);
      }
      __OsClose(handle);
-     _bioHandle = bioOpen(ff_fname,FO_READWRITE | SO_DENYNONE,BBIO_CACHE_SIZE);
-     if(_bioHandle == &bNull)  _bioHandle = bioOpen(ff_fname,FO_READWRITE | SO_COMPAT,BBIO_CACHE_SIZE);
+     _bioHandle = bioOpen(ff_fname,FO_READWRITE | SO_DENYNONE,BBIO_CACHE_SIZE,fioUseMMF ? BIO_OPT_USEMMF : BIO_OPT_DB);
+     if(_bioHandle == &bNull)  _bioHandle = bioOpen(ff_fname,FO_READWRITE | SO_COMPAT,BBIO_CACHE_SIZE,fioUseMMF ? BIO_OPT_USEMMF : BIO_OPT_DB);
      if(_bioHandle == &bNull)  goto use_err;
      crpos = ff_startpos;
      bioSeek(_bioHandle,0L,SEEKF_START);
@@ -320,14 +325,14 @@ static tBool FStore( void )
         rem = (unsigned)min(wsize,4096);
         if(!BMReadBufferEx(tmp_buff,rem,crpos,BM_SEEK_SET))
         {
-          errnoMessageBox("Can not read file",NULL,errno);
+          errnoMessageBox(READ_FAIL,NULL,errno);
           bioClose(_bioHandle);
           goto Exit;
         }
-        real_size = activeMode->convert_cp ? activeMode->convert_cp((char *)tmp_buff,rem) : rem;
+        real_size = activeMode->convert_cp ? activeMode->convert_cp((char *)tmp_buff,rem,True) : rem;
         if(!bioWriteBuffer(_bioHandle,tmp_buff,real_size))
         {
-          errnoMessageBox("Can not write to file",NULL,errno);
+          errnoMessageBox(WRITE_FAIL,NULL,errno);
           bioClose(_bioHandle);
           goto Exit;
         }
@@ -378,7 +383,7 @@ static tBool FStore( void )
      fout = fopen(ff_fname,"wt");
      if(fout == NULL)
      {
-        errnoMessageBox("Can not write to file",NULL,errno);
+        errnoMessageBox(WRITE_FAIL,NULL,errno);
         PFREE(codebuff);
         goto Exit;
      }
@@ -458,11 +463,11 @@ static tBool FStore( void )
           if(obj_class == OC_NOOBJECT)
           {
             unsigned long diff;
-            fprintf(fout,"; L%08lXH-L%08lXH - is no object\n",obj_start,obj_end);
+            fprintf(fout,"; L%08lXH-L%08lXH - no object\n",obj_start,obj_end);
             dret.codelen = min(UCHAR_MAX,obj_end - ff_startpos);
             /** some functions can placed in virtual area of objects
                 mean at end of true data, but before next object */
-             while(func_pa && func_pa >= obj_start && func_pa < obj_end)
+             while(func_pa && func_pa >= obj_start && func_pa < obj_end && func_pa > ff_startpos)
              {
                   diff = func_pa - ff_startpos;
                   if(diff) fprintf(fout,"resb %08lXH\n",diff);
@@ -474,6 +479,11 @@ static tBool FStore( void )
                   func_pa = detectedFormat->GetPubSym(func_name,sizeof(func_name),
                                                     &func_class,ff_startpos,False);
                   func_name[sizeof(func_name)-1] = 0;
+                  if(func_pa == ff_startpos)
+                  {
+                    fprintf(fout,"...Probably internal error of biew...\n");
+                    break;
+                  }
               }
               diff = obj_end - ff_startpos;
               if(diff) fprintf(fout,"resb %08lXH\n",diff);
@@ -482,14 +492,23 @@ static tBool FStore( void )
           }
           if(detectedFormat->GetPubSym && func_pa)
           {
-            if(ff_startpos == func_pa)
+	    int not_silly;
+	    not_silly = 0;
+            while(ff_startpos == func_pa)
             {
+              /* print out here all public labels */
               fprintf(fout,"%s %s:\n"
                           ,GET_FUNC_CLASS(func_class)
                           ,func_name);
               func_pa = detectedFormat->GetPubSym(func_name,sizeof(func_name),
                                                   &func_class,ff_startpos,False);
               func_name[sizeof(func_name)-1] = 0;
+	      not_silly++;
+              if(not_silly > 100)
+              {
+                fprintf(fout,"; [snipped out] ...\n");
+                break;
+              }
             }
           }
        }
@@ -592,7 +611,7 @@ static tBool FStore( void )
        strcat(tmp_buff2 ? tmp_buff2 : tmp_buff,"\n");
        if(fputs(tmp_buff2 ? tmp_buff2 : tmp_buff,fout) == EOF)
        {
-         errnoMessageBox("Can not write to file",NULL,errno);
+         errnoMessageBox(WRITE_FAIL,NULL,errno);
          goto dis_exit;
        }
        if(flags & FSDLG_STRUCTS)
@@ -603,7 +622,7 @@ static tBool FStore( void )
        }
        if(!dret.codelen)
        {
-         ErrMessageBox("Internal Fatal error"," Put structures ");
+         ErrMessageBox("Internal fatal error"," Put structures ");
          goto dis_exit;
        }
        ff_startpos += dret.codelen;
@@ -633,7 +652,7 @@ static tBool FStore( void )
    CloseWnd(progress_wnd);
    BMSeek(cpos,BM_SEEK_SET);
   }
-  else  ErrMessageBox("Start > end position!",NULL);
+  else  ErrMessageBox("Start position > end position!",NULL);
  }
  PFREE(tmp_buff);
  DumpMode = False;
@@ -646,7 +665,7 @@ static tBool FRestore( void )
  tBool ret;
  ret = False;
  flags = FSDLG_NOMODES;
- if(GetFStoreDlg(" Restore information from File ",ff_fname,&flags,&ff_startpos,&ff_len,FILE_PRMT))
+ if(GetFStoreDlg(" Restore information from file ",ff_fname,&flags,&ff_startpos,&ff_len,FILE_PRMT))
  {
    unsigned long flen,lval;
    int handle;
@@ -664,14 +683,14 @@ static tBool FRestore( void )
    if(endpos > ff_startpos)
    {
      unsigned long wsize,cwpos;
-     unsigned short remaind;
+     unsigned remaind;
      void *tmp_buff;
      handle = __OsOpen(ff_fname,FO_READONLY | SO_DENYNONE);
      if(handle == -1) handle = __OsOpen(ff_fname,FO_READONLY | SO_COMPAT);
      if(handle == -1)
      {
         err:
-        errnoMessageBox("Can not open file",NULL,errno);
+        errnoMessageBox(OPEN_FAIL,NULL,errno);
         return False;
      }
      cpos = BMGetCurrFilePos();
@@ -686,22 +705,22 @@ static tBool FRestore( void )
      }
      fname = BMName();
      bHandle = biewOpenRW(fname,BBIO_SMALL_CACHE_SIZE);
-     if(bHandle)
+     if(bHandle != &bNull)
      {
        while(wsize)
        {
          remaind = (unsigned)min(wsize,4096);
          if(__OsRead(handle,tmp_buff,remaind) != remaind)
          {
-           errnoMessageBox("Can not read from file",NULL,errno);
+           errnoMessageBox(READ_FAIL,NULL,errno);
            __OsClose(handle);
            ret = False;
            goto bye;
          }
          bioSeek(bHandle,cwpos,BIO_SEEK_SET);
-         if(bioWriteBuffer(bHandle,tmp_buff,remaind) != remaind)
+         if(!bioWriteBuffer(bHandle,tmp_buff,remaind))
          {
-           errnoMessageBox("Can not write to file",NULL,errno);
+           errnoMessageBox(WRITE_FAIL,NULL,errno);
            ret = False;
            goto bye;
          }
@@ -712,13 +731,13 @@ static tBool FRestore( void )
        bioClose(bHandle);
        BMReRead();
      }
-     else errnoMessageBox("Can not write to file",NULL,errno);
+     else errnoMessageBox(OPEN_FAIL,NULL,errno);
      PFREE(tmp_buff);
      __OsClose(handle);
      BMSeek(cpos,BM_SEEK_SET);
      ret = True;
    }
-   else ErrMessageBox("Start > end position!",NULL);
+   else ErrMessageBox("Start position > end position!",NULL);
  }
  return ret;
 }
@@ -782,11 +801,11 @@ static tBool CryptBlock( void )
    lval = endpos - ff_startpos;
    endpos = lval > flen ? flen + ff_startpos : endpos;
    endpos = endpos > BMGetFLength() ? BMGetFLength() : endpos;
-   if(!pass[0]) { ErrMessageBox("Password can not be empty",NULL); return False; }
+   if(!pass[0]) { ErrMessageBox("Password can't be empty",NULL); return False; }
    if(endpos > ff_startpos)
    {
      unsigned long wsize,cwpos;
-     unsigned short remaind;
+     unsigned remaind;
      char *fname;
      BGLOBAL bHandle;
      void *tmp_buff;
@@ -809,7 +828,7 @@ static tBool CryptBlock( void )
          remaind = (unsigned)min(wsize,4096);
          if(!bioReadBuffer(bHandle,tmp_buff,remaind))
          {
-           errnoMessageBox("Can not read file",NULL,errno);
+           errnoMessageBox(READ_FAIL,NULL,errno);
            ret = False;
            goto bye;
          }
@@ -817,7 +836,7 @@ static tBool CryptBlock( void )
          bioSeek(bHandle,cwpos,BIO_SEEK_SET);
          if(!(bioWriteBuffer(bHandle,tmp_buff,remaind)))
          {
-           errnoMessageBox("Can not write to file",NULL,errno);
+           errnoMessageBox(WRITE_FAIL,NULL,errno);
            ret = False;
            goto bye;
          }
@@ -832,7 +851,225 @@ static tBool CryptBlock( void )
      BMSeek(cpos,BM_SEEK_SET);
      ret = True;
    }
-   else ErrMessageBox("Start > end position!",NULL);
+   else ErrMessageBox("Start position > end position!",NULL);
+ }
+ return ret;
+}
+
+static void __NEAR__ __FASTCALL__ EndianifyBlock(char * buff,unsigned len, int type)
+{
+  unsigned i, step;
+  if(!type) return; /* for now */
+  switch(type)
+  {
+    default:
+#ifdef INT64_C
+    case 3: step = 8;
+            break;
+#endif
+    case 2: step = 4;
+            break;
+    case 1:
+            step = 2;
+            break;
+  }
+  len /= step;
+  len *= step;
+  for(i = 0;i < len;i+=step, buff+=step)
+  {
+    switch(type)
+    {
+      default:
+
+#ifdef INT64_C
+      case 3: *((tUInt64 *)buff) = ByteSwapLL(*((tUInt64 *)buff));
+              break;
+#endif
+
+      case 2: *((tUInt32 *)buff) = ByteSwapL(*((tUInt32 *)buff));
+              break;
+      case 1:
+              *((tUInt16 *)buff) = ByteSwapS(*((tUInt16 *)buff));
+              break;
+    }
+  }
+}
+
+static tBool ReverseBlock( void )
+{
+ unsigned long endpos,cpos,flags;
+ tBool ret;
+ ret = False;
+ ff_startpos = BMGetCurrFilePos();
+ if(!ff_len) ff_len = BMGetFLength() - ff_startpos;
+ flags = FSDLG_USEBITNS;
+ if(GetFStoreDlg(" Endianify block of file ",NULL,&flags,&ff_startpos,&ff_len,NULL))
+ {
+   unsigned long flen,lval;
+   endpos = ff_startpos + ff_len;
+   flen = BMGetFLength();
+   lval = endpos - ff_startpos;
+   endpos = lval > flen ? flen + ff_startpos : endpos;
+   endpos = endpos > BMGetFLength() ? BMGetFLength() : endpos;
+   if(endpos > ff_startpos)
+   {
+     unsigned long wsize,cwpos;
+     unsigned remaind;
+     char *fname;
+     BGLOBAL bHandle;
+     void *tmp_buff;
+     cpos = BMGetCurrFilePos();
+     wsize = endpos - ff_startpos;
+     cwpos = ff_startpos;
+     tmp_buff = PMalloc(4096);
+     if(!tmp_buff)
+     {
+       MemOutBox("temporary buffer initialization");
+       return False;
+     }
+     fname = BMName();
+     bHandle = biewOpenRW(fname,BBIO_SMALL_CACHE_SIZE);
+     if(bHandle != &bNull)
+     {
+       bioSeek(bHandle,ff_startpos,SEEK_SET);
+       while(wsize)
+       {
+         remaind = (unsigned)min(wsize,4096);
+         if(!bioReadBuffer(bHandle,tmp_buff,remaind))
+         {
+           errnoMessageBox(READ_FAIL,NULL,errno);
+           ret = False;
+           goto bye;
+         }
+         EndianifyBlock(tmp_buff,remaind, flags & FSDLG_BTNSMASK);
+         bioSeek(bHandle,cwpos,BIO_SEEK_SET);
+         if(!(bioWriteBuffer(bHandle,tmp_buff,remaind)))
+         {
+           errnoMessageBox(WRITE_FAIL,NULL,errno);
+           ret = False;
+           goto bye;
+         }
+         wsize -= remaind;
+         cwpos += remaind;
+       }
+       bye:
+       bioClose(bHandle);
+       BMReRead();
+     }
+     PFREE(tmp_buff);
+     BMSeek(cpos,BM_SEEK_SET);
+     ret = True;
+   }
+   else ErrMessageBox("Start position > end position!",NULL);
+ }
+ return ret;
+}
+
+static void __NEAR__ __FASTCALL__ TranslateBlock(char * buff,unsigned len, const unsigned char *xlt)
+{
+  unsigned i;
+  for(i = 0;i < len;i++)
+  {
+    buff[i] = __Xlat__(xlt, (int)buff[i]);
+  }
+}
+
+
+static tBool XLatBlock( void )
+{
+ unsigned char xlt[256];
+ unsigned long endpos,cpos,flags;
+ tBool ret;
+ ret = False;
+ ff_startpos = BMGetCurrFilePos();
+ if(!ff_len) ff_len = BMGetFLength() - ff_startpos;
+ flags = FSDLG_NOMODES;
+ if(!xlat_fname[0])
+ {
+   strcpy(xlat_fname,__get_rc_dir("biew"));
+   strcat(xlat_fname,"xlt");
+ }
+ if(GetFStoreDlg(" Table Look-up Translation ",xlat_fname,&flags,&ff_startpos,&ff_len,XLAT_PRMT))
+ {
+   unsigned long flen,lval;
+   endpos = ff_startpos + ff_len;
+   flen = BMGetFLength();
+   lval = endpos - ff_startpos;
+   endpos = lval > flen ? flen + ff_startpos : endpos;
+   endpos = endpos > BMGetFLength() ? BMGetFLength() : endpos;
+   if(endpos > ff_startpos)
+   {
+     unsigned long wsize,cwpos;
+     unsigned remaind;
+     char *fname;
+     BGLOBAL bHandle, xHandle;
+     void *tmp_buff;
+     cpos = BMGetCurrFilePos();
+     wsize = endpos - ff_startpos;
+     cwpos = ff_startpos;
+     /* Parse xlat file */
+     xHandle = biewOpenRO(xlat_fname,BBIO_SMALL_CACHE_SIZE);
+     if(xHandle == &bNull)
+     {
+       ErrMessageBox("Can't open xlat file", NULL);
+       return False;
+     }
+     if(bioFLength(xHandle) != 320)
+     {
+       ErrMessageBox("Size of xlat file is not 320 bytes", NULL);
+       bioClose(xHandle);
+       return False;
+     }
+     bioReadBuffer(xHandle,xlt, 16);
+     if(memcmp(xlt, "Biew Xlat Table.", 16) != 0)
+     {
+       ErrMessageBox("It seems that xlat file is corrupt", NULL);
+       bioClose(xHandle);
+       return False;
+     }
+     bioSeek(xHandle, 0x40, SEEKF_START);
+     bioReadBuffer(xHandle, xlt, 256);
+     bioClose(xHandle);
+     tmp_buff = PMalloc(4096);
+     if(!tmp_buff)
+     {
+       MemOutBox("temporary buffer initialization");
+       return False;
+     }
+     fname = BMName();
+     bHandle = biewOpenRW(fname,BBIO_SMALL_CACHE_SIZE);
+     if(bHandle != &bNull)
+     {
+       bioSeek(bHandle,ff_startpos,SEEK_SET);
+       while(wsize)
+       {
+         remaind = (unsigned)min(wsize,4096);
+         if(!bioReadBuffer(bHandle,tmp_buff,remaind))
+         {
+           errnoMessageBox(READ_FAIL,NULL,errno);
+           ret = False;
+           goto bye;
+         }
+         TranslateBlock(tmp_buff,remaind, xlt);
+         bioSeek(bHandle,cwpos,BIO_SEEK_SET);
+         if(!(bioWriteBuffer(bHandle,tmp_buff,remaind)))
+         {
+           errnoMessageBox(WRITE_FAIL,NULL,errno);
+           ret = False;
+           goto bye;
+         }
+         wsize -= remaind;
+         cwpos += remaind;
+       }
+       bye:
+       bioClose(bHandle);
+       BMReRead();
+     }
+     PFREE(tmp_buff);
+     BMSeek(cpos,BM_SEEK_SET);
+     ret = True;
+   }
+   else ErrMessageBox("Start position > end position!",NULL);
  }
  return ret;
 }
@@ -921,25 +1158,25 @@ static tBool FileInfo( void )
            "Modification time             = %s"
            "Last access time              = %s"
            "Device containing file        = %u\n"
-           "File serial number            = %u\n"
+           "File serial (inode) number    = %u\n"
            "Number of hard links to file  = %u\n"
            "User ID of the file owner     = %u\n"
            "Group ID of the file owner    = %u"
            ,shortname
-           ,detectedFormat->name ? detectedFormat->name : FType == OLD_EXE ? "MZ" : FType == DOS_SYS ? "DOSdriver" : "Not detected"
+           ,detectedFormat->name ? detectedFormat->name : "Not detected"
            ,BMGetFLength()
            ,attr
            ,stimes[0]
            ,stimes[1]
            ,stimes[2]
-           ,statbuf.st_dev
-           ,statbuf.st_ino
-           ,statbuf.st_nlink
-           ,statbuf.st_uid
-           ,statbuf.st_gid);
+           ,(unsigned)statbuf.st_dev
+           ,(unsigned)statbuf.st_ino
+           ,(unsigned)statbuf.st_nlink
+           ,(unsigned)statbuf.st_uid
+           ,(unsigned)statbuf.st_gid);
   do
   {
-    evt = GetEvent(drawEmptyPrompt,wnd);
+    evt = GetEvent(drawEmptyPrompt,NULL,wnd);
   }
   while(!(evt == KE_ESCAPE || evt == KE_F(10)));
   CloseWnd(wnd);
@@ -953,7 +1190,9 @@ static const char * fu_names[] =
   "~Save block as...",
   "~Restore block from...",
   "~Insert/delete block...",
-  "~Crypt/decrypt block..."
+  "~Crypt/decrypt block...", 
+  "~Endianify block...", 
+  "~Xlat block..."
 };
 
 typedef tBool (*FileFunc)( void );
@@ -965,7 +1204,9 @@ static FileFunc fu_funcs[] =
   FStore,
   FRestore,
   InsDelBlock,
-  CryptBlock
+  CryptBlock, 
+  ReverseBlock,
+  XLatBlock
 };
 
 tBool FileUtils( void )
