@@ -30,16 +30,42 @@
 #include "biewlib/kbd_code.h"
 #include "biewlib/biewlib.h"
 
+static char is_msbf=0; /* is most significand byte first */
+static char is_64bit=0;
+
+#define AOUT_HALF(cval) FMT_WORD(cval,is_msbf)
+#define AOUT_WORD(cval) FMT_DWORD(cval,is_msbf)
+#define AOUT_QWORD(cval) FMT_QWORD(cval,is_msbf)
+
 static const char * __NEAR__ __FASTCALL__ aout_encode_hdr(unsigned long info)
 {
-   switch(info & 0x0000FFFFUL)
+   switch(N_MAGIC(AOUT_WORD(&info)))
    {
-     case OMAGIC: return " a.out: Object file ";
-     case NMAGIC: return " a.out: Pure executable ";
-     case ZMAGIC: return " a.out: Demand-paged executable ";
-     case BMAGIC: return " b.out: Object file ";
-     case QMAGIC: return " a.out: 386BSD demand-paged executable ";
-     default:     return " Unknow a.out or b.out format ";
+     case OMAGIC: return " a.out-32: Object file ";
+     case NMAGIC: return " a.out-32: Pure executable ";
+     case ZMAGIC: return " a.out-32: Demand-paged executable ";
+     case BMAGIC: return " b.out-32: Object file ";
+     case QMAGIC: return " a.out-32: 386BSD demand-paged executable ";
+     case CMAGIC: return " a.out-core";
+     case OMAGIC64: return " a.out-64: Object file ";
+     case NMAGIC64: return " a.out-64: Pure executable ";
+     case ZMAGIC64: return " a.out-64: Demand-paged executable ";
+     default:     return " Unknown a.out or b.out format ";
+   }
+}
+
+static const char * __NEAR__ __FASTCALL__ aout_encode_machine(unsigned long info)
+{
+   switch(N_MACHTYPE(AOUT_WORD(&info)))
+   {
+     case 0: return "Old Sun-2";
+     case 1: return "M-68010";
+     case 2: return "M-68020";
+     case 3: return "Sparc";
+     case 100: return "i386";
+     case 151: return "MIPS1(R3000)";
+     case 152: return "MIPS2(R6000)";
+     default:  return "Unknown CPU";
    }
 }
 
@@ -53,27 +79,29 @@ static unsigned long __FASTCALL__ ShowAOutHeader( void )
   bmReadBufferEx(&aout,sizeof(struct external_exec),0,SEEKF_START);
   w = CrtDlgWndnls(aout_encode_hdr(*((unsigned long *)&aout.e_info)),54,7);
   twGotoXY(1,1);
-  twPrintF("Length of text section      = %08lXH\n"
+  twPrintF("Flags & CPU                 = %02XH %s(%s)\n"
+	   "Length of text section      = %08lXH\n"
            "Length of data section      = %08lXH\n"
            "Length of bss area          = %08lXH\n"
            "Length of symbol table      = %08lXH\n"
-           ,*((unsigned long *)&aout.e_text)
-           ,*((unsigned long *)&aout.e_data)
-           ,*((unsigned long *)&aout.e_bss)
-           ,*((unsigned long *)&aout.e_syms));
+	   ,N_FLAGS(*((unsigned long *)&aout.e_info)),aout_encode_machine(*((unsigned long *)&aout.e_info)),is_msbf?"big-endian":"little-endian"
+           ,AOUT_WORD((unsigned long *)&aout.e_text)
+           ,AOUT_WORD((unsigned long *)&aout.e_data)
+           ,AOUT_WORD((unsigned long *)&aout.e_bss)
+           ,AOUT_WORD((unsigned long *)&aout.e_syms));
   twSetColorAttr(dialog_cset.entry);
   twPrintF("Start address               = %08lXH"
-           ,*((unsigned long *)&aout.e_entry));
+           ,AOUT_WORD((unsigned long *)&aout.e_entry));
   twClrEOL(); twPrintF("\n");
   twSetColorAttr(dialog_cset.main);
   twPrintF("Length of text relocation   = %08lXH\n"
            "Length of data relocation   = %08lXH"
-           ,*((unsigned long *)&aout.e_trsize)
-           ,*((unsigned long *)&aout.e_drsize));
+           ,AOUT_WORD((unsigned long *)&aout.e_trsize)
+           ,AOUT_WORD((unsigned long *)&aout.e_drsize));
   while(1)
   {
     keycode = GetEvent(drawEmptyPrompt,NULL,w);
-    if(keycode == KE_ENTER) { fpos = *((unsigned long *)&aout.e_entry); break; }
+    if(keycode == KE_ENTER) { fpos = AOUT_WORD(*((unsigned long *)&aout.e_entry)); break; }
     else
       if(keycode == KE_ESCAPE || keycode == KE_F(10)) break;
   }
@@ -81,11 +109,23 @@ static unsigned long __FASTCALL__ ShowAOutHeader( void )
   return fpos;
 }
 
+static tBool __FASTCALL__ __aout_check_fmt( tUInt32 id )
+{
+  int a32,a64;
+  a32=!(N_BADMAG(id));
+  a64=!(N_BADMAG64(id));
+  if(a64) is_64bit=1;
+  return a32 || a64 || N_MAGIC(id)==CMAGIC;
+}
+
 static tBool __FASTCALL__ aout_check_fmt( void )
 {
-  tUIntFast16 id;
-  id = bmReadWordEx(0,SEEKF_START);
-  return !(N_BADMAG(id));
+  tUInt32 id;
+  id = bmReadDWordEx(0,SEEKF_START);
+  if(__aout_check_fmt(id)) return 1;
+  id=ByteSwapL(id);
+  if(__aout_check_fmt(id)) { is_msbf=1; return 1; }
+  return 0;
 }
 
 static void __FASTCALL__ aout_init_fmt( void ) {}
@@ -94,7 +134,7 @@ static void __FASTCALL__ aout_destroy_fmt( void ) {}
 static int __FASTCALL__ aout_bitness(unsigned long off)
 {
    UNUSED(off);
-   return DAB_USE32;
+   return is_64bit?DAB_USE64:DAB_USE32;
 }
 
 static tBool __FASTCALL__ aout_AddrResolv(char *addr,unsigned long fpos)
