@@ -1,4 +1,4 @@
-/** 
+/**
  * @namespace   biew_plugins_auto
  * @file        plugins/bin/coff386.c
  * @brief       This file contains implementation of COFF-i386 file format decoder.
@@ -14,6 +14,7 @@
  * @since       1999
  * @note        Development, fixes and improvements
 **/
+#include <limits.h>
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -32,10 +33,14 @@
 #include "biewlib/kbd_code.h"
 #include "biewlib/pmalloc.h"
 
+#define COFF_WORD(cval)  ((tUInt16)(*(const tUInt16 *)(const tUInt8 *)cval))
+#define COFF_DWORD(cval) ((tUInt32)(*(const tUInt32 *)(const tUInt8 *)cval))
+
+
 static struct external_filehdr coff386hdr;
 static AOUTHDR coff386ahdr;
 static SCNHDR *coff386so;
-static unsigned short nsections;
+static tUIntFast16 nsections;
 static BGLOBAL coff_cache;
 static unsigned long strings_ptr;
 
@@ -43,7 +48,7 @@ static void __FASTCALL__ coff_ReadPubNameList(BGLOBAL handle,void (__FASTCALL__
 static void __FASTCALL__ coff_ReadPubName(BGLOBAL b_cache,const struct PubName *it,
                            char *buff,unsigned cb_buff);
 static unsigned __FASTCALL__ coff386_GetObjAttr(unsigned long pa,char *name,unsigned cb_name,
-                             unsigned long *start,unsigned long *end,int *class,int *bitness);
+                             unsigned long *start,unsigned long *end,int *_class,int *bitness);
 
 static tBool __NEAR__ __FASTCALL__ FindPubName(char *buff,unsigned cb_buff,unsigned long pa)
 {
@@ -73,12 +78,12 @@ static void __NEAR__ __FASTCALL__ coffReadLongName(BGLOBAL handle,unsigned long
 
 static unsigned long __FASTCALL__ coff386_VA2PA(unsigned long va)
 {
-  unsigned long i;
+  tUIntFast16 i;
   for(i = 0;i < nsections;i++)
   {
-    if(va >= *(unsigned long *)&coff386so[i].s_vaddr && va <= *(unsigned long *)&coff386so[i].s_vaddr+*(unsigned long *)&coff386so[i].s_size)
+    if(va >= COFF_DWORD(coff386so[i].s_vaddr) && va <= COFF_DWORD(coff386so[i].s_vaddr)+COFF_DWORD(coff386so[i].s_size))
     {
-      return *(unsigned long *)&coff386so[i].s_scnptr + (va - *(unsigned long *)&coff386so[i].s_vaddr);
+      return COFF_DWORD(coff386so[i].s_scnptr) + (va - COFF_DWORD(coff386so[i].s_vaddr));
     }
   }
   return 0L;
@@ -86,12 +91,12 @@ static unsigned long __FASTCALL__ coff386_VA2PA(unsigned long va)
 
 static unsigned long __FASTCALL__ coff386_PA2VA(unsigned long pa)
 {
-  unsigned long i;
+  tUIntFast16 i;
   for(i = 0;i < nsections;i++)
   {
-    if(pa >= *(unsigned long *)&coff386so[i].s_scnptr && pa <= *(unsigned long *)&coff386so[i].s_scnptr+*(unsigned long *)&coff386so[i].s_size)
+    if(pa >= COFF_DWORD(coff386so[i].s_scnptr) && pa <= COFF_DWORD(coff386so[i].s_scnptr)+COFF_DWORD(coff386so[i].s_size))
     {
-      return *(unsigned long *)coff386so[i].s_vaddr + (pa - *(unsigned long *)&coff386so[i].s_scnptr);
+      return COFF_DWORD(coff386so[i].s_vaddr) + (pa - COFF_DWORD(coff386so[i].s_scnptr));
     }
   }
   return 0L;
@@ -124,18 +129,18 @@ static void __FASTCALL__ coffObjPaint(TWindow * win,const void ** names,unsigned
           "  [%c] - section type is data\n"
           "  [%c] - section type is bss"
           ,objs->s_name
-          ,*(unsigned long *)&objs->s_paddr
-          ,*(unsigned long *)&objs->s_vaddr
-          ,*(unsigned long *)&objs->s_size
-          ,*(unsigned long *)&objs->s_scnptr
-          ,*(unsigned long *)&objs->s_relptr
-          ,*(unsigned long *)&objs->s_lnnoptr
-          ,*(unsigned short *)&objs->s_nreloc
-          ,*(unsigned short *)&objs->s_nlnno
-          ,*(unsigned long *)&objs->s_flags
-          ,GetBool((*(unsigned long *)&objs->s_flags & STYP_TEXT) == STYP_TEXT)
-          ,GetBool((*(unsigned long *)&objs->s_flags & STYP_DATA) == STYP_DATA)
-          ,GetBool((*(unsigned long *)&objs->s_flags & STYP_BSS) == STYP_BSS)
+          ,COFF_DWORD(objs->s_paddr)
+          ,COFF_DWORD(objs->s_vaddr)
+          ,COFF_DWORD(objs->s_size)
+          ,COFF_DWORD(objs->s_scnptr)
+          ,COFF_DWORD(objs->s_relptr)
+          ,COFF_DWORD(objs->s_lnnoptr)
+          ,COFF_WORD(objs->s_nreloc)
+          ,COFF_WORD(objs->s_nlnno)
+          ,COFF_DWORD(objs->s_flags)
+          ,GetBool((COFF_DWORD(objs->s_flags) & STYP_TEXT) == STYP_TEXT)
+          ,GetBool((COFF_DWORD(objs->s_flags) & STYP_DATA) == STYP_DATA)
+          ,GetBool((COFF_DWORD(objs->s_flags) & STYP_BSS) == STYP_BSS)
           );
  twRefreshFullWin(win);
 }
@@ -160,24 +165,24 @@ static unsigned long __FASTCALL__ coffShowObjects( void )
  unsigned long fpos,off;
  memArray * obj;
  fpos = BMGetCurrFilePos();
- nnames = *(unsigned short *)&coff386hdr.f_nscns;
+ nnames = COFF_WORD(coff386hdr.f_nscns);
  if(!nnames) { NotifyBox(NOT_ENTRY," Objects Table "); return fpos; }
  if(!(obj = ma_Build(nnames,True))) return fpos;
  handle = coff_cache;
  off = sizeof(coff386hdr);
- if(*(unsigned short *)&coff386hdr.f_opthdr) off += *(unsigned short *)&coff386hdr.f_opthdr;
+ if(COFF_WORD(coff386hdr.f_opthdr)) off += COFF_WORD(coff386hdr.f_opthdr);
  bioSeek(handle,off,SEEK_SET);
  if(__coffReadObjects(handle,obj,nnames))
  {
   int ret;
     ret = PageBox(70,13,(const void **)obj->data,obj->nItems,coffObjPaint);
-    if(ret != -1)  fpos = *(unsigned long *)(((SCNHDR *)obj->data[ret])->s_scnptr);
+    if(ret != -1)  fpos = COFF_DWORD(((SCNHDR *)obj->data[ret])->s_scnptr);
  }
  ma_Destroy(obj);
  return fpos;
 }
 
-static const char * __NEAR__ __FASTCALL__ coff386_encode_hdr(unsigned short info)
+static const char * __NEAR__ __FASTCALL__ coff386_encode_hdr(unsigned info)
 {
    switch(info)
    {
@@ -201,10 +206,10 @@ static unsigned long __FASTCALL__ ShowCoff386Header( void )
     v_entry = entry = *(unsigned long *)&coff386ahdr.entry;
     entry = coff386_VA2PA(v_entry);
   }
-  w = CrtDlgWndnls(coff386_encode_hdr(*((unsigned short *)&coff386hdr.f_magic)),54,12);
+  w = CrtDlgWndnls(coff386_encode_hdr(COFF_WORD(coff386hdr.f_magic)),54,12);
   twGotoXY(1,1);
   twPrintF("Number of sections          = %04XH\n"
-           "Time & data stamp           = %08lXH\n"
+           "Time & date stamp           = %08lXH\n"
            "File pointer to symtab      = %08lXH\n"
            "Number of symtab entries    = %08lXH\n"
            "Size of optional header     = %04XH\n"
@@ -214,17 +219,17 @@ static unsigned long __FASTCALL__ ShowCoff386Header( void )
            "  [%c] - Line numbers stripped\n"
            "  [%c] - Local symbols stripped\n"
            "  [%c] - File is 32-bits little endian\n"
-           ,*((unsigned short *)&coff386hdr.f_nscns)
-           ,*((unsigned long *)&coff386hdr.f_timdat)
-           ,*((unsigned long *)&coff386hdr.f_symptr)
-           ,*((unsigned long *)&coff386hdr.f_nsyms)
-           ,*((unsigned short *)&coff386hdr.f_opthdr)
-           ,*((unsigned short *)&coff386hdr.f_flags)
-           ,GetBool((*((unsigned long *)&coff386hdr.f_flags) & F_RELFLG) == F_RELFLG)
-           ,GetBool((*((unsigned long *)&coff386hdr.f_flags) & F_EXEC) == F_EXEC)
-           ,GetBool((*((unsigned long *)&coff386hdr.f_flags) & F_LNNO) == F_LNNO)
-           ,GetBool((*((unsigned long *)&coff386hdr.f_flags) & F_LSYMS) == F_LSYMS)
-           ,GetBool((*((unsigned long *)&coff386hdr.f_flags) & F_AR32WR) == F_AR32WR));
+           ,COFF_WORD(coff386hdr.f_nscns)
+           ,COFF_DWORD(coff386hdr.f_timdat)
+           ,COFF_DWORD(coff386hdr.f_symptr)
+           ,COFF_DWORD(coff386hdr.f_nsyms)
+           ,COFF_WORD(coff386hdr.f_opthdr)
+           ,COFF_WORD(coff386hdr.f_flags)
+           ,GetBool((COFF_WORD(coff386hdr.f_flags) & F_RELFLG) == F_RELFLG)
+           ,GetBool((COFF_WORD(coff386hdr.f_flags) & F_EXEC) == F_EXEC)
+           ,GetBool((COFF_WORD(coff386hdr.f_flags) & F_LNNO) == F_LNNO)
+           ,GetBool((COFF_WORD(coff386hdr.f_flags) & F_LSYMS) == F_LSYMS)
+           ,GetBool((COFF_WORD(coff386hdr.f_flags) & F_AR32WR) == F_AR32WR));
   twSetColorAttr(dialog_cset.entry);
   twPrintF("Entry point                 = %08lXH (VA=%08lXH)"
            ,entry,v_entry);
@@ -232,7 +237,7 @@ static unsigned long __FASTCALL__ ShowCoff386Header( void )
   twSetColorAttr(dialog_cset.main);
   while(1)
   {
-    keycode = GetEvent(drawEmptyPrompt,w);
+    keycode = GetEvent(drawEmptyPrompt,NULL,w);
     if(keycode == KE_ENTER) { fpos = entry; break; }
     else
       if(keycode == KE_ESCAPE || keycode == KE_F(10)) break;
@@ -311,25 +316,23 @@ static const char * __NEAR__ __FASTCALL__ coffEncodeClass(unsigned _class)
 static unsigned __FASTCALL__ coffSymTabNumItems(BGLOBAL handle)
 {
   UNUSED(handle);
-  return (unsigned)(*(unsigned long *)&coff386hdr.f_nsyms);
+  return (unsigned)COFF_DWORD(coff386hdr.f_nsyms);
 }
 
 static tBool  __FASTCALL__ coffSymTabReadItems(BGLOBAL handle,memArray * obj,unsigned nnames)
 {
  unsigned i;
  unsigned length;
- bioSeek(handle,*(unsigned long *)&coff386hdr.f_symptr,SEEKF_START);
+ bioSeek(handle,COFF_DWORD(coff386hdr.f_symptr),SEEKF_START);
  for(i = 0;i < nnames;i++)
  {
    struct external_syment cse;
    char stmp[256];
    memset(stmp,0,sizeof(stmp));
    bioReadBuffer(handle,&cse,sizeof(struct external_syment));
-   if(*(unsigned long *)&cse.e.e.e_zeroes == 0L &&
-      *(unsigned long *)&cse.e.e.e_offset >= 4)
-   {
-     coffReadLongName(handle,*(unsigned long *)&cse.e.e.e_offset,stmp,sizeof(stmp));
-   }
+   if(COFF_DWORD(cse.e.e.e_zeroes) == 0L &&
+      COFF_DWORD(cse.e.e.e_offset) >= 4)
+        coffReadLongName(handle,COFF_DWORD(cse.e.e.e_offset),stmp,sizeof(stmp));
    else
    {
      memcpy(stmp,cse.e.e_name,E_SYMNMLEN);
@@ -340,10 +343,10 @@ static tBool  __FASTCALL__ coffSymTabReadItems(BGLOBAL handle,memArray * obj,uns
    if(length > 38) strcpy(&stmp[38],">>>");
    else  {  memset(&stmp[length],' ',38-length);  stmp[38] = 0; }
    sprintf(&stmp[strlen(stmp)-1],"(%04hX.%08lX) %s.%s"
-           ,*(unsigned short *)&cse.e_scnum
-           ,*(unsigned long *)&cse.e_value
+           ,COFF_WORD(cse.e_scnum)
+           ,(unsigned long)COFF_DWORD(cse.e_value)
            ,coffEncodeClass(cse.e_sclass[0])
-           ,coffEncodeType(*(unsigned short *)&cse.e_type));
+           ,coffEncodeType(COFF_WORD(cse.e_type)));
    if(!ma_AddString(obj,stmp,True)) break;
  }
  return True;
@@ -352,24 +355,24 @@ static tBool  __FASTCALL__ coffSymTabReadItems(BGLOBAL handle,memArray * obj,uns
 static unsigned long __NEAR__ __FASTCALL__ CalcEntryCoff(unsigned long idx,tBool display_msg)
 {
   struct external_syment cse;
-  unsigned short sec_num;
+  tUIntFast16 sec_num;
   unsigned long fpos;
   fpos = 0L;
-  bmSeek(*(unsigned long *)&coff386hdr.f_symptr+idx*sizeof(struct external_syment),BM_SEEK_SET);
+  bmSeek(COFF_DWORD(coff386hdr.f_symptr)+idx*sizeof(struct external_syment),BM_SEEK_SET);
   bmReadBuffer(&cse,sizeof(struct external_syment));
-  sec_num = *(unsigned short *)&cse.e_scnum;
-  if(sec_num && sec_num <= *(unsigned short *)coff386hdr.f_nscns &&
-    ((*(unsigned char *)&cse.e_sclass == C_EXT ||
-      *(unsigned char *)&cse.e_sclass == C_STAT ||
-      *(unsigned char *)&cse.e_sclass == C_HIDDEN) ||
-     (*(unsigned char *)&cse.e_sclass == C_LABEL &&
-      *(unsigned long *)&cse.e_value)))
+  sec_num = COFF_WORD(cse.e_scnum);
+  if(sec_num && sec_num <= COFF_WORD(coff386hdr.f_nscns) &&
+    ((cse.e_sclass[0] == C_EXT ||
+      cse.e_sclass[0] == C_STAT ||
+      cse.e_sclass[0] == C_HIDDEN) ||
+     (cse.e_sclass[0] == C_LABEL &&
+      COFF_DWORD(cse.e_value))))
   {
-    fpos = *(unsigned long *)&coff386so[sec_num-1].s_scnptr+*(unsigned long *)&cse.e_value;
+    fpos = COFF_DWORD(coff386so[sec_num-1].s_scnptr)+COFF_DWORD(cse.e_value);
   }
   else
     if(display_msg)
-      ErrMessageBox("This entry does not refer inside the file",NULL);
+      ErrMessageBox(NO_ENTRY,NULL);
   return fpos;
 }
 
@@ -404,40 +407,36 @@ static linearArray *RelocCoff386 = NULL;
 static tCompare __FASTCALL__ coff386_compare_rels(const void __HUGE__ *e1,const void __HUGE__ *e2)
 {
   const RELOC_COFF386 __HUGE__ *r1,__HUGE__ *r2;
-  tCompare ret;
   r1 = e1;
   r2 = e2;
-  if(r1->offset < r2->offset) ret = -1;
-  else
-    if(r1->offset > r2->offset) ret = 1;
-    else                        ret = 0;
-  return ret;
+  return __CmpLong__(r1->offset,r2->offset);
 }
 
 static void __NEAR__ __FASTCALL__ BuildRelocCoff386( void )
 {
   TWindow * w,*usd;
-  unsigned j,segcount;
+  size_t j,segcount, nr;
   RELOC_COFF386 rel;
   usd = twUsedWin();
   if(!(RelocCoff386 = la_Build(0,sizeof(RELOC_COFF386),MemOutBox))) return;
-  w = CrtDlgWndnls(" System is busy ",49,1);
+  w = CrtDlgWndnls(SYSTEM_BUSY,49,1);
   if(!PubNames) coff_ReadPubNameList(bmbioHandle(),MemOutBox);
   twUseWin(w);
   twGotoXY(1,1);
-  twPutS(" Building reference chains for Coff386 format");
+  twPutS(BUILD_REFS);
   twUseWin(usd);
-  for(segcount = 0;segcount < *(unsigned short *)&coff386hdr.f_nscns;segcount++)
+  for(segcount = 0;segcount < COFF_WORD(coff386hdr.f_nscns);segcount++)
   {
     tBool is_eof;
     is_eof = False;
-    bmSeek(*(unsigned long *)&coff386so[segcount].s_relptr,BM_SEEK_SET);
-    for(j = 0;j < *(unsigned short *)&coff386so[segcount].s_nreloc;j++)
+    bmSeek(COFF_DWORD(coff386so[segcount].s_relptr),BM_SEEK_SET);
+    nr = COFF_WORD(coff386so[segcount].s_nreloc);
+    for(j = 0;j < nr;j++)
     {
       struct external_reloc er;
       bmReadBuffer(&er,sizeof(struct external_reloc));
       if((is_eof = bmEOF()) != 0) break;
-      rel.offset = er.r_vaddr + *(unsigned long *)&coff386so[segcount].s_scnptr;
+      rel.offset = er.r_vaddr + COFF_DWORD(coff386so[segcount].s_scnptr);
       rel.nameoff = er.r_symndx;
       rel.type = er.r_type;
       if(!la_AddData(RelocCoff386,&rel,MemOutBox)) goto next;
@@ -451,40 +450,39 @@ static void __NEAR__ __FASTCALL__ BuildRelocCoff386( void )
 
 static tBool  __NEAR__ __FASTCALL__ coffSymTabReadItemsIdx(BGLOBAL handle,unsigned long idx,
                                             char *name,unsigned cb_name,
-                                            unsigned short *secnum,
+                                            unsigned *secnum,
                                             unsigned long *offset)
 {
  struct external_syment cse;
- if(idx >= *(unsigned long *)&coff386hdr.f_nsyms) return False;
- bioSeek(handle,*(unsigned long *)&coff386hdr.f_symptr + idx*sizeof(struct external_syment),SEEKF_START);
+ if(idx >= COFF_DWORD(coff386hdr.f_nsyms)) return False;
+ bioSeek(handle,COFF_DWORD(coff386hdr.f_symptr) + idx*sizeof(struct external_syment),SEEKF_START);
  bioReadBuffer(handle,&cse,sizeof(struct external_syment));
- if(*(unsigned long *)&cse.e.e.e_zeroes == 0L &&
-    *(unsigned long *)&cse.e.e.e_offset >= 4)
- {
-     coffReadLongName(handle,*(unsigned long *)&cse.e.e.e_offset,name,cb_name);
- }
+ if(COFF_DWORD(cse.e.e.e_zeroes) == 0L &&
+    COFF_DWORD(cse.e.e.e_offset) >= 4)
+          coffReadLongName(handle,COFF_DWORD(cse.e.e.e_offset),name,cb_name);
  else
  {
    memcpy(name,cse.e.e_name,E_SYMNMLEN);
    name[E_SYMNMLEN] = 0;
  }
- *secnum = *(unsigned short *)&cse.e_scnum;
- *offset = *(unsigned long *)&cse.e_value;
+ *secnum = COFF_WORD(cse.e_scnum);
+ *offset = COFF_DWORD(cse.e_value);
  return True;
 }
 
-static tBool __NEAR__ __FASTCALL__ BuildReferStrCoff386(char *str,RELOC_COFF386 *rne,int flags)
+static unsigned long __NEAR__ __FASTCALL__ BuildReferStrCoff386(char *str,RELOC_COFF386 *rne,int flags)
 {
-  unsigned long val,offset,s,e;
-  unsigned short secnum;
-  tBool retval,val_assigned;
+  unsigned long val,offset,s,e,retval;
+  tUIntFast16 secnum;
+  tBool is_idx,val_assigned;
   int c,b;
   char name[256],pubname[256],secname[256];
+  retval = RAPREF_DONE;
   val = bmReadDWordEx(rne->offset,BM_SEEK_SET);
   /* rne->nameoff it's only pointer to name descriptor */
-  retval = coffSymTabReadItemsIdx(coff_cache,rne->nameoff,name,sizeof(name),&secnum,&offset);
+  is_idx = coffSymTabReadItemsIdx(coff_cache,rne->nameoff,name,sizeof(name),&secnum,&offset);
   val_assigned = False;
-  if(retval)
+  if(is_idx)
   {
     if(!offset && secnum)
     {
@@ -492,7 +490,7 @@ static tBool __NEAR__ __FASTCALL__ BuildReferStrCoff386(char *str,RELOC_COFF386
          try resolve some relocations (like .text+1234) with names from
          public name list
       */
-      if(FindPubName(pubname,sizeof(pubname),*(unsigned long *)&coff386so[secnum-1].s_scnptr + val))
+      if(FindPubName(pubname,sizeof(pubname),COFF_DWORD(coff386so[secnum-1].s_scnptr) + val))
       {
         strcat(str,pubname);
         val_assigned = True;
@@ -503,7 +501,7 @@ static tBool __NEAR__ __FASTCALL__ BuildReferStrCoff386(char *str,RELOC_COFF386
     {
       def_val:
       strcat(str,name);
-    }    
+    }
   }
   else
     strcat(str,".?bad?");
@@ -511,6 +509,8 @@ static tBool __NEAR__ __FASTCALL__ BuildReferStrCoff386(char *str,RELOC_COFF386
   {
      strcat(str,"+");
      strcat(str,Get8Digit(val));
+     if(secnum) retval = COFF_DWORD(coff386so[secnum-1].s_scnptr)+val;
+     else       retval = RAPREF_NONE;
   }
   if(rne->type == RELOC_REL32 && (flags & APREF_TRY_LABEL) != APREF_TRY_LABEL)
   {
@@ -521,23 +521,23 @@ static tBool __NEAR__ __FASTCALL__ BuildReferStrCoff386(char *str,RELOC_COFF386
   e = CalcEntryCoff(rne->nameoff,False);
   if(!DumpMode && !EditMode && e && (flags & APREF_TRY_LABEL) == APREF_TRY_LABEL)
                                                   GidAddGoAddress(str,e);
-  return True;
+  return retval;
 }
 
-static int __FASTCALL__ coff386_AppendRef(char *str,unsigned long ulShift,int flags,int codelen,unsigned long r_sh)
+static unsigned long __FASTCALL__ coff386_AppendRef(char *str,unsigned long ulShift,int flags,int codelen,unsigned long r_sh)
 {
   RELOC_COFF386 *rcoff386,key;
-  tBool ret;
+  unsigned long ret;
   char buff[400];
-  ret = False;
+  ret = RAPREF_NONE;
+  if(flags & APREF_TRY_PIC) return RAPREF_NONE;
   if(!PubNames) coff_ReadPubNameList(bmbioHandle(),MemOutBox);
-  if((*(unsigned short *)&coff386hdr.f_flags & F_RELFLG) == F_RELFLG) goto try_pub;
+  if((COFF_WORD(coff386hdr.f_flags) & F_RELFLG) == F_RELFLG) goto try_pub;
   if(!RelocCoff386) BuildRelocCoff386();
   key.offset = ulShift;
   __codelen = codelen;
   rcoff386 = la_Find(RelocCoff386,&key,coff386_compare_rels);
-  if(!rcoff386) ret = False;
-  else          ret = BuildReferStrCoff386(str,rcoff386,flags);
+  if(rcoff386) ret = BuildReferStrCoff386(str,rcoff386,flags);
   try_pub:
   if(!ret && (flags & APREF_TRY_LABEL))
   {
@@ -545,15 +545,15 @@ static int __FASTCALL__ coff386_AppendRef(char *str,unsigned long ulShift,int fl
      {
        strcat(str,buff);
        if(!DumpMode && !EditMode) GidAddGoAddress(str,r_sh);
-       ret = True;
+       ret = RAPREF_DONE;
      }
   }
-  return ret ? RAPREF_DONE : RAPREF_NONE;
+  return flags & APREF_TRY_LABEL ? ret ? RAPREF_DONE : RAPREF_NONE : ret;
 }
 
 static tBool __FASTCALL__ coff386_check_fmt( void )
 {
-  unsigned short id;
+  tUIntFast16 id;
   id = bmReadWordEx(0,SEEKF_START);
   return !(I386BADMAG(id));
 }
@@ -562,10 +562,10 @@ static void __FASTCALL__ coff386_init_fmt( void )
 {
   BGLOBAL main_handle;
   unsigned long s_off = sizeof(coff386hdr);
-  unsigned i;
+  tUIntFast16 i;
   bmReadBufferEx(&coff386hdr,sizeof(struct external_filehdr),0,SEEKF_START);
-  if(*(unsigned short *)&coff386hdr.f_opthdr) bmReadBuffer(&coff386ahdr,sizeof(AOUTHDR));
-  nsections = *(unsigned long *)&coff386hdr.f_nscns;
+  if(COFF_WORD(coff386hdr.f_opthdr)) bmReadBuffer(&coff386ahdr,sizeof(AOUTHDR));
+  nsections = COFF_WORD(coff386hdr.f_nscns);
   if(!(coff386so = PMalloc((unsigned)(sizeof(SCNHDR)*nsections))))
   {
      MemOutBox("Coff386 initialization");
@@ -573,13 +573,13 @@ static void __FASTCALL__ coff386_init_fmt( void )
   }
   main_handle = bmbioHandle();
   if((coff_cache = bioDupEx(main_handle,BBIO_SMALL_CACHE_SIZE)) == &bNull) coff_cache = main_handle;
-  if(*(unsigned short *)&coff386hdr.f_opthdr) s_off += *(unsigned short *)&coff386hdr.f_opthdr;
+  if(COFF_WORD(coff386hdr.f_opthdr)) s_off += COFF_WORD(coff386hdr.f_opthdr);
   bioSeek(coff_cache,s_off,BIO_SEEK_SET);
   for(i = 0;i < nsections;i++)
   {
     bioReadBuffer(coff_cache,&coff386so[i],sizeof(SCNHDR));
   }
-  strings_ptr = *(unsigned long *)&coff386hdr.f_symptr+*(unsigned long *)&coff386hdr.f_nsyms*sizeof(struct external_syment);
+  strings_ptr = COFF_DWORD(coff386hdr.f_symptr)+COFF_DWORD(coff386hdr.f_nsyms)*sizeof(struct external_syment);
 }
 
 static void __FASTCALL__ coff386_destroy_fmt( void )
@@ -594,7 +594,7 @@ static void __FASTCALL__ coff386_destroy_fmt( void )
 static int __FASTCALL__ coff386_bitness(unsigned long off)
 {
   UNUSED(off);
-  return DAB_USE32; 
+  return DAB_USE32;
 }
 
 static tBool __FASTCALL__ coff386_AddrResolv(char *addr,unsigned long cfpos)
@@ -602,14 +602,14 @@ static tBool __FASTCALL__ coff386_AddrResolv(char *addr,unsigned long cfpos)
  /* Since this function is used in references resolving of disassembler
     it must be seriously optimized for speed. */
   tBool bret = True;
-  uint32_t res;
+  tUInt32 res;
   if(cfpos < sizeof(struct external_filehdr))
   {
     strcpy(addr,"coffih:");
     strcpy(&addr[7],Get2Digit(cfpos));
   }
-  else 
-    if((res=coff386_PA2VA(cfpos))!=0) 
+  else
+    if((res=coff386_PA2VA(cfpos))!=0)
     {
       addr[0] = '.';
       strcpy(&addr[1],Get8Digit(res));
@@ -643,25 +643,26 @@ static void __FASTCALL__ coff_ReadPubNameList(BGLOBAL handle,
  unsigned i,nnames;
  struct PubName pn;
  if(!(PubNames = la_Build(0,sizeof(struct PubName),mem_out))) return;
- bioSeek(handle,*(unsigned long *)&coff386hdr.f_symptr,SEEKF_START);
+ bioSeek(handle,COFF_DWORD(coff386hdr.f_symptr),SEEKF_START);
  nnames = coffSymTabNumItems(handle);
  for(i = 0;i < nnames;i++)
  {
    struct external_syment cse;
    unsigned long where;
-   unsigned short sec_num;
+   tUIntFast16 sec_num;
    where = bioTell(handle);
    bioReadBuffer(handle,&cse,sizeof(struct external_syment));
-   sec_num = *(unsigned short *)&cse.e_scnum;
-   if(sec_num && sec_num <= *(unsigned short *)coff386hdr.f_nscns &&
-     (*(unsigned char *)&cse.e_sclass == C_EXT ||
-      *(unsigned char *)&cse.e_sclass == C_STAT ||
-      *(unsigned char *)&cse.e_sclass == C_HIDDEN))
+   sec_num = COFF_WORD(cse.e_scnum);
+   if(sec_num && sec_num <= COFF_WORD(coff386hdr.f_nscns) &&
+     (cse.e_sclass[0] == C_EXT ||
+      cse.e_sclass[0] == C_STAT ||
+      cse.e_sclass[0] == C_HIDDEN ||
+      cse.e_sclass[0] == C_LABEL))
    {
-     if(*(unsigned long *)&cse.e.e.e_zeroes == 0L &&
-        *(unsigned long *)&cse.e.e.e_offset >= 4)
+     if(COFF_DWORD(cse.e.e.e_zeroes) == 0L &&
+        COFF_DWORD(cse.e.e.e_offset) >= 4)
      {
-       pn.nameoff = *(unsigned long *)&cse.e.e.e_offset;
+       pn.nameoff = COFF_DWORD(cse.e.e.e_offset);
        pn.addinfo = 0;
      }
      else
@@ -669,8 +670,8 @@ static void __FASTCALL__ coff_ReadPubNameList(BGLOBAL handle,
        pn.nameoff = where;
        pn.addinfo = E_SYMNMLEN;
      }
-     pn.pa = *(unsigned long *)&coff386so[sec_num-1].s_scnptr+*(unsigned long *)&cse.e_value;
-     pn.attr = *(unsigned char *)&cse.e_sclass == C_EXT ? SC_GLOBAL : SC_LOCAL;
+     pn.pa = COFF_DWORD(coff386so[sec_num-1].s_scnptr)+COFF_DWORD(cse.e_value);
+     pn.attr = cse.e_sclass[0] == C_EXT ? SC_GLOBAL : SC_LOCAL;
      if(!la_AddData(PubNames,&pn,mem_out)) break;
    }
    if(bioEOF(handle)) break;
@@ -687,35 +688,36 @@ static unsigned long __FASTCALL__ coff386_GetPubSym(char *str,unsigned cb_str,un
 }
 
 static unsigned __FASTCALL__ coff386_GetObjAttr(unsigned long pa,char *name,unsigned cb_name,
-                            unsigned long *start,unsigned long *end,int *class,int *bitness)
+                            unsigned long *start,unsigned long *end,int *_class,int *bitness)
 {
-  unsigned i,ret;
+  unsigned ret;
+  tUIntFast16 i;
   *start = 0;
   *end = bmGetFLength();
-  *class = OC_NOOBJECT;
+  *_class = OC_NOOBJECT;
   *bitness = DAB_USE32;
   name[0] = 0;
   ret = 0;
   for(i = 0;i < nsections;i++)
   {
-    if(pa >= *start && pa < *(unsigned long *)&coff386so[i].s_scnptr)
+    if(pa >= *start && pa < COFF_DWORD(coff386so[i].s_scnptr))
     {
       /** means between two objects */
-      *end = *(unsigned long *)&coff386so[i].s_scnptr;
+      *end = COFF_DWORD(coff386so[i].s_scnptr);
       ret = 0;
       break;
     }
-    if(pa >= *(unsigned long *)&coff386so[i].s_scnptr && pa < *(unsigned long *)&coff386so[i].s_scnptr + *(unsigned long *)&coff386so[i].s_size)
+    if(pa >= COFF_DWORD(coff386so[i].s_scnptr) && pa < COFF_DWORD(coff386so[i].s_scnptr) + COFF_DWORD(coff386so[i].s_size))
     {
-      *start = *(unsigned long *)&coff386so[i].s_scnptr;
-      *end = *start + *(unsigned long *)&coff386so[i].s_size;
-      *class = (*(unsigned long *)&coff386so[i].s_flags & STYP_TEXT) == STYP_TEXT ? OC_CODE : OC_DATA;
+      *start = COFF_DWORD(coff386so[i].s_scnptr);
+      *end = *start + COFF_DWORD(coff386so[i].s_size);
+      *_class = (COFF_DWORD(coff386so[i].s_flags) & STYP_TEXT) == STYP_TEXT ? OC_CODE : OC_DATA;
       strncpy(name,(const char *)coff386so[i].s_name,min(sizeof(coff386so[i].s_name),cb_name));
       name[sizeof(coff386so[i].s_name)] = 0;
       ret = i+1;
       break;
     }
-    *start = *(unsigned long *)&coff386so[i].s_scnptr + *(unsigned long *)&coff386so[i].s_size;
+    *start = COFF_DWORD(coff386so[i].s_scnptr) + COFF_DWORD(coff386so[i].s_size);
   }
   return ret;
 }
