@@ -1,4 +1,4 @@
-/** 
+/**
  * @namespace   biew_plugins_auto
  * @file        plugins/bin/lx.c
  * @brief       This file contains implementation of LX (Linear eXecutable) file
@@ -452,7 +452,7 @@ static tBool __NEAR__ __FASTCALL__ __ReadEntriesLX(BGLOBAL handle,memArray *obj)
  unsigned i;
  unsigned char j;
  unsigned char cnt,type;
- unsigned short numobj = 0;
+ tUIntFast16 numobj = 0;
  LX_ENTRY _lxe;
  i = 0;
  while(1)
@@ -493,16 +493,32 @@ static tBool __NEAR__ __FASTCALL__ __ReadEntriesLX(BGLOBAL handle,memArray *obj)
  return True;
 }
 
+static void __FASTCALL__ lxReadPageDesc(BGLOBAL handle,LX_MAP_TABLE *mt,unsigned long pageidx)
+{
+  bioSeek(handle,headshift+lxe.lx.lxObjectPageTableOffset+
+          sizeof(LX_MAP_TABLE)*(pageidx - 1),SEEK_SET);
+  bioReadBuffer(handle,(void *)mt,sizeof(LX_MAP_TABLE));
+}
+
 static unsigned long __NEAR__ __FASTCALL__ __calcPageEntry(LX_MAP_TABLE *mt)
 {
   unsigned long dataoff;
   unsigned long ret;
   dataoff = mt->o32_pagedataoffset << lxe.lx.lxPageOffsetShift;
-  if(mt->o32_pageflags == 1 || mt->o32_pageflags == 5) ret = lxe.lx.lxObjectIterPageOffset + dataoff;
-  else
-    if(mt->o32_pageflags == 0) ret = lxe.lx.lxDataPagesOffset + dataoff;
-    else                      ret = -1;
-  return ret;
+  switch(mt->o32_pageflags)
+  {
+    default:
+    case PAGE_VALID: ret = lxe.lx.lxDataPagesOffset;
+                     break;
+    case PAGE_ITERDATA2: /* This is very undocumented type.
+                            I do not know - how handle it !!!*/
+    case PAGE_ITERDATA: ret = lxe.lx.lxObjectIterPageOffset;
+                        break;
+    case PAGE_RANGE:
+    case PAGE_INVALID:
+    case PAGE_ZEROED:  ret = 0; break;
+  }
+  return ret + dataoff;
 }
 
 static unsigned long __NEAR__ __FASTCALL__ CalcPageEntry(unsigned long pageidx)
@@ -511,16 +527,14 @@ static unsigned long __NEAR__ __FASTCALL__ CalcPageEntry(unsigned long pageidx)
   LX_MAP_TABLE mt;
   if(!pageidx) return -1;
   handle = lx_cache;
-  bioSeek(handle,lxe.lx.lxObjectPageTableOffset + headshift,SEEK_SET);
-  bioSeek(handle,sizeof(LX_MAP_TABLE)*(pageidx - 1),SEEKF_CUR);
-  bioReadBuffer(handle,(void *)&mt,sizeof(LX_MAP_TABLE));
+  lxReadPageDesc(handle,&mt,pageidx);
   return __calcPageEntry((void *)&mt);
 }
 
 static unsigned long __NEAR__ __FASTCALL__ CalcEntryPointLX(unsigned long objnum,unsigned long _offset)
 {
   BGLOBAL handle;
-  unsigned long ret,pshift;
+  unsigned long i,diff;
   LX_OBJECT lo;
   LX_MAP_TABLE mt;
   if(!objnum) return BMGetCurrFilePos();
@@ -528,14 +542,10 @@ static unsigned long __NEAR__ __FASTCALL__ CalcEntryPointLX(unsigned long objnum
   bioSeek(handle,lxe.lx.lxObjectTableOffset + headshift,SEEK_SET);
   bioSeek(handle,sizeof(LX_OBJECT)*(objnum - 1),SEEKF_CUR);
   bioReadBuffer(handle,(void *)&lo,sizeof(LX_OBJECT));
-/*  if((lo.o32_flags & 0x00002000L) == 0x00002000L) USE16 = 0;
-  else                                            USE16 = 0xFF; */
-  bioSeek(handle,lxe.lx.lxObjectPageTableOffset + headshift,SEEKF_START);
-  pshift = _offset % lxe.lx.lxPageSize;
-  bioSeek(handle,sizeof(LX_MAP_TABLE)*(lo.o32_pagemap - 1 + _offset/lxe.lx.lxPageSize),SEEKF_CUR);
-  bioReadBuffer(handle,(void *)&mt,sizeof(LX_MAP_TABLE));
-  ret = __calcPageEntry((void *)&mt) + pshift;
-  return ret;
+  i = _offset / lxe.lx.lxPageSize;
+  diff = _offset - i*lxe.lx.lxPageSize;
+  lxReadPageDesc(handle,&mt,i+lo.o32_pagemap);
+  return __calcPageEntry((void *)&mt) + diff;
 }
 
 static void __FASTCALL__ ReadLXLEImpMod(unsigned long offtable,unsigned num,char *str)
@@ -610,7 +620,7 @@ static unsigned long __NEAR__ __FASTCALL__ CalcEntryBungleLX(unsigned ordinal,tB
   unsigned i;
   unsigned char j;
   unsigned char cnt,type;
-  unsigned short numobj = 0;
+  tUIntFast16 numobj = 0;
   LX_ENTRY lxent;
   unsigned long ret;
   ret = BMGetCurrFilePos();
@@ -660,7 +670,7 @@ static unsigned long __NEAR__ __FASTCALL__ CalcEntryBungleLX(unsigned ordinal,tB
    if(found || is_eof) break;
  }
  if(found) ret = CalcEntryLX((LX_ENTRY *)&lxent);
- else      if(dispmsg) ErrMessageBox("No such entry in the Table",NULL);
+ else      if(dispmsg) ErrMessageBox(NOT_ENTRY,NULL);
  return ret;
 }
 
@@ -672,7 +682,7 @@ unsigned long __FASTCALL__ ShowObjectsLX( void )
  memArray * obj;
  fpos = BMGetCurrFilePos();
  nnames = (unsigned)lxe.lx.lxObjectCount;
- if(!nnames) { NotifyBox("Entry not found"," Objects Table "); return fpos; }
+ if(!nnames) { NotifyBox(NOT_ENTRY," Objects Table "); return fpos; }
  if(!(obj = ma_Build(nnames,True))) return fpos;
  handle = lx_cache;
  bioSeek(handle,lxe.lx.lxObjectTableOffset + headshift,SEEK_SET);
@@ -822,16 +832,15 @@ static void __FASTCALL__ PaintEntriesLX(TWindow * win,const void ** names,unsign
 
 static tBool __FASTCALL__ __ReadMapTblLX(BGLOBAL handle,memArray * obj,unsigned n)
 {
- size_t i;
-  bioSeek(handle,lxe.lx.lxObjectPageTableOffset + headshift,SEEKF_START);
+  size_t i;
   for(i = 0;i < n;i++)
   {
     LX_MAP_TABLE mt;
     char stmp[80];
     if(IsKbdTerminate() || bioEOF(handle)) break;
-    bioReadBuffer(handle,&mt,sizeof(LX_MAP_TABLE));
+    lxReadPageDesc(handle,&mt,i+1);
     sprintf(stmp,"Off=%08lXH Siz=%04hXH Flg:%04hXH=%s",
-                 mt.o32_pagedataoffset,
+                 (unsigned long)mt.o32_pagedataoffset,
                  mt.o32_pagesize,
                  mt.o32_pageflags,
                  (const char *)lxeGetMapAttr(mt.o32_pageflags));
@@ -879,14 +888,14 @@ unsigned long __FASTCALL__ ShowEntriesLX( void )
  unsigned long fpos;
  memArray * obj;
  fpos = BMGetCurrFilePos();
- if(!lxe.lx.lxEntryTableOffset) { NotifyBox("Entry not found"," Entry Table "); return fpos; }
+ if(!lxe.lx.lxEntryTableOffset) { NotifyBox(NOT_ENTRY," Entry Table "); return fpos; }
  handle = lx_cache;
  bioSeek(handle,lxe.lx.lxEntryTableOffset + headshift,SEEK_SET);
  if(!(obj = ma_Build(0,True))) goto exit;
  if(__ReadEntriesLX(handle,obj))
  {
     int ret;
-    if(!obj->nItems) { NotifyBox("Entry not found"," Entry Table "); goto bye; }
+    if(!obj->nItems) { NotifyBox(NOT_ENTRY," Entry Table "); goto bye; }
     ret = PageBox(70,8,(const void **)obj->data,obj->nItems,PaintEntriesLX);
     if(ret != -1)  fpos = LXType == FILE_LX ? CalcEntryLX(obj->data[ret]) : CalcEntryLE(obj->data[ret]);
  }
@@ -935,7 +944,9 @@ static tBool __NEAR__ __FASTCALL__ __ReadResourceGroupLX(BGLOBAL handle,memArray
     sprintf(stmp,"%6hu = ",lxr.nameID);
     if(lxr.typeID < sizeof(ResourceGrNamesLX)/sizeof(char *)) strcat(stmp,ResourceGrNamesLX[lxr.typeID]);
     else  sprintf(&stmp[strlen(stmp)],"Unknown < %04hXH >",lxr.typeID);
-    sprintf(&stmp[strlen(stmp)]," obj=%04hXH.%08lXH",lxr.object,lxr.offset);
+    sprintf(&stmp[strlen(stmp)]," obj=%04hXH.%08lXH"
+                               ,lxr.object
+                               ,(unsigned long)lxr.offset);
     if(!ma_AddString(obj,stmp,True)) break;
  }
  return True;
@@ -1046,11 +1057,118 @@ static unsigned long __FASTCALL__ LXHelp( void )
   return BMGetCurrFilePos();
 }
 
+static unsigned long __FASTCALL__ lxVA2PA(unsigned long va)
+{
+ /* First we must determine object number for given virtual address */
+  BGLOBAL handle;
+  unsigned long i,diff,oidx,rva,pa;
+  LX_OBJECT lo;
+  LX_MAP_TABLE mt;
+  handle = lx_cache;
+  bioSeek(handle,lxe.lx.lxObjectTableOffset + headshift,SEEK_SET);
+  pa = oidx = 0; /* means: error */
+  for(i = 0;i < lxe.lx.lxObjectCount;i++)
+  {
+    bioReadBuffer(handle,(void *)&lo,sizeof(LX_OBJECT));
+    if(lo.o32_base <= va && va < lo.o32_base + lo.o32_size)
+    {
+      oidx = i+1;
+      break;
+    }
+  }
+  /* Secondly we must determine page within object */
+  if(oidx)
+  {
+    rva = va - lo.o32_base;
+    i = rva / lxe.lx.lxPageSize;
+    diff = rva - i*lxe.lx.lxPageSize;
+    lxReadPageDesc(handle,&mt,i+lo.o32_pagemap);
+    pa = __calcPageEntry((void *)&mt) + diff;
+  }
+  return pa;
+}
+
+static unsigned long __FASTCALL__ lxPA2VA(unsigned long pa)
+{
+ /* First we must determine page for given physical address */
+  BGLOBAL handle;
+  unsigned long i,pidx,rva,va,pagentry;
+  LX_OBJECT lo;
+  LX_MAP_TABLE mt;
+  handle = lx_cache;
+  pagentry = va = pidx = 0;
+  for(i = 0;i < lxe.lx.lxPageCount;i++)
+  {
+    lxReadPageDesc(handle,&mt,i+1);
+    pagentry = __calcPageEntry(&mt);
+    if(pagentry <= pa && pa < pagentry + mt.o32_pagesize)
+    {
+      pidx = i+1;
+      break;
+    }
+  }
+  /* Secondly we must determine object number for given physical address */
+  if(pidx)
+  {
+    rva = pa - pagentry + (pidx-1)*lxe.lx.lxPageSize;
+    bioSeek(handle,lxe.lx.lxObjectTableOffset + headshift,SEEK_SET);
+    for(i = 0;i < lxe.lx.lxObjectCount;i++)
+    {
+      bioReadBuffer(handle,(void *)&lo,sizeof(LX_OBJECT));
+      if(lo.o32_pagemap <= pidx && pidx < lo.o32_pagemap + lo.o32_mapsize)
+      {
+        va = lo.o32_base + rva;
+        break;
+      }
+    }
+  }
+  return va;
+}
+
+static int __FASTCALL__ lxBitness(unsigned long pa)
+{
+ /* First we must determine page for given physical address */
+  BGLOBAL handle;
+  unsigned long i,pidx,pagentry;
+  int ret;
+  LX_OBJECT lo;
+  LX_MAP_TABLE mt;
+  handle = lx_cache;
+  pagentry = pidx = 0;
+  ret = DAB_USE16;
+  for(i = 0;i < lxe.lx.lxPageCount;i++)
+  {
+    lxReadPageDesc(handle,&mt,i+1);
+    pagentry = __calcPageEntry(&mt);
+    if(pagentry <= pa && pa < pagentry + mt.o32_pagesize)
+    {
+      pidx = i+1;
+      break;
+    }
+  }
+  /* Secondly we must determine object number for given physical address */
+  if(pidx)
+  {
+    bioSeek(handle,lxe.lx.lxObjectTableOffset + headshift,SEEK_SET);
+    for(i = 0;i < lxe.lx.lxObjectCount;i++)
+    {
+      bioReadBuffer(handle,(void *)&lo,sizeof(LX_OBJECT));
+      if(lo.o32_pagemap <= pidx && pidx < lo.o32_pagemap + lo.o32_mapsize)
+      {
+        ret = (lo.o32_flags & 0x00002000L) == 0x00002000L ? DAB_USE32 : DAB_USE16;
+        break;
+      }
+    }
+  }
+  return ret;
+}
+
 static tBool __FASTCALL__ lxAddressResolv(char *addr,unsigned long cfpos)
 {
  /* Since this function is used in references resolving of disassembler
     it must be seriously optimized for speed. */
   tBool bret = True;
+  tUInt32 res;
   if(cfpos >= headshift && cfpos < headshift + sizeof(LXHEADER))
   {
     strcpy(addr,"LXH :");
@@ -1070,7 +1188,13 @@ static tBool __FASTCALL__ lxAddressResolv(char *addr,unsigned long cfpos)
     strcpy(addr,"LXPD:");
     strcpy(&addr[5],Get4Digit(cfpos - headshift - lxe.lx.lxObjectPageTableOffset));
   }
-  else bret = False;
+  else
+   if((res=lxPA2VA(cfpos))!=0)
+   {
+     addr[0] = '.';
+     strcpy(&addr[1],Get8Digit(res));
+   }
+   else bret = False;
   return bret;
 }
 
@@ -1086,10 +1210,10 @@ REGISTRY_BIN lxTable =
   NULL,
   NULL,
   lxPlatform,
-  NULL,
+  lxBitness,
   lxAddressResolv,
-  NULL,
-  NULL,
+  lxVA2PA,
+  lxPA2VA,
   NULL,
   NULL,
   NULL,
