@@ -1,4 +1,4 @@
-/** 
+/**
  * @namespace   biew_plugins_auto
  * @file        plugins/bin/nlm386.c
  * @brief       This file contains implementation of NLM-32 (Novell Loadable
@@ -122,7 +122,7 @@ static unsigned long __FASTCALL__ ShowNLMHeader( void )
            ,nlm.nlm_flags);
   do
   {
-    keycode = GetEvent(drawEmptyPrompt,w);
+    keycode = GetEvent(drawEmptyPrompt,NULL,w);
     if(keycode == KE_ENTER)
     {
       fpos = nlm.nlm_codeImageOffset + nlm.nlm_codeStartOffset;
@@ -262,7 +262,7 @@ static unsigned long __FASTCALL__ ShowNewNLM( void )
   }
   do
   {
-    keycode = GetEvent(drawEmptyPrompt,w);
+    keycode = GetEvent(drawEmptyPrompt,NULL,w);
     if(keycode == KE_ENTER)
     {
        if(sharedEntry) fpos = sharedEntry + nlm.nlm_codeImageOffset;
@@ -324,7 +324,7 @@ static unsigned long __NEAR__ __FASTCALL__ CalcEntryNLM(unsigned ord,tBool dispm
  if(ret > bmGetFLength())
  {
     ret = fpos;
-    if(dispmsg) ErrMessageBox("Entry point is outside of this file",NULL);
+    if(dispmsg) ErrMessageBox(NO_ENTRY,NULL);
  }
  return ret;
 }
@@ -437,27 +437,17 @@ static linearArray *RelocNlm = NULL;
 static tCompare __FASTCALL__ nlm_compare_s(const void __HUGE__ *e1,const void __HUGE__ *e2)
 {
   const RELOC_NLM __HUGE__ *r1,__HUGE__ *r2;
-  tCompare ret;
   r1 = e1;
   r2 = e2;
-  if(r1->offset < r2->offset) ret = -1;
-  else
-    if(r1->offset > r2->offset) ret = 1;
-    else                      ret = 0;
-  return ret;
+  return __CmpLong__(r1->offset,r2->offset);
 }
 
 static tCompare __FASTCALL__ nlm_compare_f(const void __HUGE__ *e1,const void __HUGE__ *e2)
 {
   const RELOC_NLM __HUGE__ *r1,__HUGE__ *r2;
-  tCompare ret;
   r1 = e1;
   r2 = e2;
-  if(r1->offset < r2->offset) ret = -1;
-  else
-    if(r1->offset > r2->offset) ret = 1;
-    else                        ret = 0;
-  return ret;
+  return __CmpLong__(r1->offset,r2->offset);
 }
 
 static void __NEAR__ __FASTCALL__ BuildRelocNlm( void )
@@ -469,11 +459,11 @@ static void __NEAR__ __FASTCALL__ BuildRelocNlm( void )
   RELOC_NLM rel;
   usd = twUsedWin();
   if(!(RelocNlm = la_Build(0,sizeof(RELOC_NLM),MemOutBox))) return;
-  w = CrtDlgWndnls(" System is busy ",49,1);
+  w = CrtDlgWndnls(SYSTEM_BUSY,49,1);
   if(!PubNames) nlm_ReadPubNameList(bmbioHandle(),MemOutBox);
   twUseWin(w);
   twGotoXY(1,1);
-  twPutS(" Building reference chains for NLM format");
+  twPutS(BUILD_REFS);
   twUseWin(usd);
   /** -- for external references */
   cpos = nlm.nlm_externalReferencesOffset;
@@ -509,13 +499,13 @@ static void __NEAR__ __FASTCALL__ BuildRelocNlm( void )
   CloseWnd(w);
 }
 
-static int __NEAR__ __FASTCALL__ BuildReferStrNLM(char *str,RELOC_NLM*rne)
+static unsigned long __NEAR__ __FASTCALL__ BuildReferStrNLM(char *str,RELOC_NLM*rne,int flags)
 {
   unsigned long val;
-  unsigned char len;
-  int retrf;
+  unsigned long retrf;
   char name[256];
   BGLOBAL b_cache;
+  unsigned char len;
   b_cache = nlm_cache;
   bioSeek(b_cache,rne->nameoff,BM_SEEK_SET);
   retrf = RAPREF_DONE;
@@ -534,20 +524,23 @@ static int __NEAR__ __FASTCALL__ BuildReferStrNLM(char *str,RELOC_NLM*rne)
       strcat(str,name);
     }
     else
-    {
-      strcat(str,"(*this)+");
-      strcat(str,Get8Digit(val));
-      retrf = RAPREF_PROCEEDCMT;
-    }
+     if(!(flags & APREF_SAVE_VIRT))
+     {
+       strcat(str,"(*this)+");
+       strcat(str,Get8Digit(val));
+       retrf = val;
+     }
+     else retrf = RAPREF_NONE;
   }
   return retrf;
 }
 
-static int __FASTCALL__ AppendNLMRef(char *str,unsigned long ulShift,int flags,int codelen,unsigned long r_sh)
+static unsigned long __FASTCALL__ AppendNLMRef(char *str,unsigned long ulShift,int flags,int codelen,unsigned long r_sh)
 {
   RELOC_NLM *rnlm,key;
-  int retrf;
+  unsigned long retrf;
   char buff[400];
+  if(flags & APREF_TRY_PIC) return RAPREF_NONE;
   if(!nlm.nlm_numberOfExternalReferences || nlm.nlm_externalReferencesOffset >= bmGetFLength()) retrf = RAPREF_NONE;
   else
   {
@@ -555,7 +548,7 @@ static int __FASTCALL__ AppendNLMRef(char *str,unsigned long ulShift,int flags,i
     key.offset = ulShift;
     __codelen = codelen;
     rnlm = la_Find(RelocNlm,&key,nlm_compare_f);
-    retrf = rnlm ? BuildReferStrNLM(str,rnlm) : RAPREF_NONE;
+    retrf = rnlm ? BuildReferStrNLM(str,rnlm,flags) : RAPREF_NONE;
   }
   if(!retrf && (flags & APREF_TRY_LABEL))
   {
@@ -605,13 +598,13 @@ static tBool __FASTCALL__ NLMAddrResolv(char *addr,unsigned long cfpos)
  /* Since this function is used in references resolving of disassembler
     it must be seriously optimized for speed. */
   tBool bret = True;
-  uint32_t res;
+  tUInt32 res;
   if(cfpos < sizeof(Nlm_Internal_Fixed_Header))
   {
     strcpy(addr,"nlm32h:");
     strcpy(&addr[7],Get2Digit(cfpos));
   }
-  else 
+  else
     if((res=NLMPA2VA(cfpos)) != 0)
     {
       addr[0] = '.';
@@ -676,13 +669,13 @@ static unsigned long __FASTCALL__ NLMGetPubSym(char *str,unsigned cb_str,unsigne
 }
 
 static unsigned __FASTCALL__ NLMGetObjAttr(unsigned long pa,char *name,unsigned cb_name,
-                      unsigned long *start,unsigned long *end,int *class,int *bitness)
+                      unsigned long *start,unsigned long *end,int *_class,int *bitness)
 {
   unsigned ret;
   UNUSED(cb_name);
   *start = 0;
   *end = bmGetFLength();
-  *class = OC_NOOBJECT;
+  *_class = OC_NOOBJECT;
   *bitness = NLMbitness(pa);
   name[0] = 0;
   if(pa < nlm.nlm_codeImageOffset)
@@ -694,21 +687,21 @@ static unsigned __FASTCALL__ NLMGetObjAttr(unsigned long pa,char *name,unsigned
     if(pa >= nlm.nlm_codeImageOffset && pa < nlm.nlm_codeImageOffset + nlm.nlm_codeImageSize)
     {
       *start = nlm.nlm_codeImageOffset;
-      *class = OC_CODE;
+      *_class = OC_CODE;
       *end = nlm.nlm_codeImageOffset + nlm.nlm_codeImageSize;
       ret = 1;
     }
     else
     if(pa >= nlm.nlm_dataImageOffset && pa < nlm.nlm_dataImageOffset + nlm.nlm_dataImageSize)
     {
-      *class = OC_DATA;
+      *_class = OC_DATA;
       *start = nlm.nlm_dataImageOffset;
       *end = *start + nlm.nlm_dataImageSize;
       ret = 2;
     }
     else
     {
-      *class = OC_NOOBJECT;
+      *_class = OC_NOOBJECT;
       *start = nlm.nlm_dataImageOffset + nlm.nlm_dataImageSize;
       ret = 3;
     }
