@@ -1,4 +1,4 @@
-/** 
+/**
  * @namespace   biew_plugins_auto
  * @file        plugins/bin/pe.c
  * @brief       This file contains implementation of PE (Portable Executable)
@@ -39,8 +39,11 @@
 #include "biewlib/kbd_code.h"
 #include "biewlib/pmalloc.h"
 
+#define ARRAY_SIZE(x)       (sizeof(x)/sizeof(x[0]))
+
 static PE_ADDR * peVA = NULL;
-PEHEADER pe;
+static PEHEADER pe;
+static PERVA *peDir;
 
 static BGLOBAL pe_cache = &bNull;
 static BGLOBAL pe_cache1 = &bNull;
@@ -86,12 +89,12 @@ static unsigned long __NEAR__ __FASTCALL__ RVA2Phys(unsigned long rva)
    if(rva >= peVA[i].rva) break;
    if(IsKbdTerminate()) return 0;
  }
- if(i < 0) return 0;
+ if (i < 0) return rva;         // low RVAs fix -XF
  pphys = peVA[i].phys;
  obj_rva = peVA[i].rva;
  /* Added by Kostya Nosov <k-nosov@yandex.ru> */
  if (i < pe.peObjects - 1)
- { 
+ {
    size = peVA[i+1].phys - pphys;
    if (rva - obj_rva > size) return 0;
  }
@@ -124,30 +127,32 @@ static unsigned long __NEAR__ __FASTCALL__ fioReadDWord2Phys(BGLOBAL handle,unsi
  return RVA2Phys(dword);
 }
 
-const char * __pecputype[] =
-{
- "80386",
- "80486",
- "80586",
- "80686",
- "80787",
- "80887",
- "80987",
- "801087"
-};
-
 static const char * __NEAR__ __FASTCALL__ PECPUType(void)
 {
-  const char * cptr;
-  if(pe.peCPUType >= 0x014C && pe.peCPUType < 0x0150) cptr = __pecputype[(pe.peCPUType - 0x014C) & 0x0007];
-  else
-    if(pe.peCPUType == 0x0162) cptr = "__MIPS Mark I (R2000, R3000)";
-    else
-      if(pe.peCPUType == 0x0163) cptr = "__MIPS Mark II (R6000)";
-      else
-        if(pe.peCPUType == 0x0166) cptr = "__MIPS Mark III (R4000)";
-        else                            cptr = __pecputype[0];
-  return cptr;
+    static const char * __pecputype[] =
+    {
+     "80386",
+     "80486",
+     "80586",
+     "80686",
+     "80787",
+     "80887",
+     "80987",
+     "801087"
+    };
+    const char * cptr;
+
+    if (pe.peCPUType >= 0x014C && pe.peCPUType < 0x0150)
+        cptr = __pecputype[(pe.peCPUType - 0x014C) & 0x0007];
+    else if (pe.peCPUType == 0x0162)
+        cptr = "__MIPS Mark I (R2000, R3000)";
+    else if (pe.peCPUType == 0x0163)
+        cptr = "__MIPS Mark II (R6000)";
+    else if (pe.peCPUType == 0x0166)
+        cptr = "__MIPS Mark III (R4000)";
+    else cptr = __pecputype[0];
+
+    return cptr;
 }
 
 static unsigned long entryPE = 0L;
@@ -199,20 +204,27 @@ static void __NEAR__ PaintNewHeaderPE_1( void )
            ,pe.peObjectAlign);
 }
 
-const char * __pesubsyst[] =
-{
-  "Unknown",
-  "Native",
-  "Windows GUI",
-  "Windows Character",
-  "OS/2 GUI",
-  "OS/2 Character",
-  "Posix GUI",
-  "Posix Character"
-};
-
 static void __NEAR__ PaintNewHeaderPE_2( void )
 {
+  static const char * subSystem[] =
+  {
+    "Unknown",
+    "Native",
+    "Windows GUI",
+    "Windows Character",
+    "OS/2 GUI",
+    "OS/2 Character",
+    "Posix GUI",
+    "Posix Character",
+    "Win9x Driver",
+    "Windows CE",
+    "EFI application",
+    "EFI boot service driver",
+    "EFI runtime driver",
+    "EFI ROM",
+    "X-Box",
+  };
+
   twPrintF("File align                     = %08lXH\n"
            "OS version                     = %hu.%hu\n"
            "User version                   = %hu.%hu\n"
@@ -220,7 +232,7 @@ static void __NEAR__ PaintNewHeaderPE_2( void )
            "Image size                     = %lu bytes\n"
            "Header size                    = %lu bytes\n"
            "File checksum                  = %08lXH\n"
-           "NT subsystem                   = %s\n"
+           "Subsystem                      = %s\n"
            "DLL Flags :                      [%04hXH]\n"
            " [%c] Per-Process Library initialization\n"
            " [%c] Per-Process Library termination\n"
@@ -230,10 +242,7 @@ static void __NEAR__ PaintNewHeaderPE_2( void )
            "Stack commit size              = %lu bytes\n"
            "Heap reserve size              = %lu bytes\n"
            "Heap commit size               = %lu bytes\n"
-           "Number interesting VA/Sizes    = %lu bytes\n"
-           "Export Table RVA               = %08lXH\n"
-           "Total export data size         = %lu bytes\n"
-           "Import Table RVA               = %08lXH"
+           "Number of directory entries    = %lu bytes"
            ,pe.peFileAlign
            ,pe.peOSMajor,pe.peOSMinor
            ,pe.peUserMajor,pe.peUserMinor
@@ -241,7 +250,7 @@ static void __NEAR__ PaintNewHeaderPE_2( void )
            ,pe.peImageSize
            ,pe.peHeaderSize
            ,pe.peFileChecksum
-           ,pe.peSubSystem <= 0x0007 ? __pesubsyst[pe.peSubSystem] : "Unknown"
+           ,pe.peSubSystem < ARRAY_SIZE(subSystem) ? subSystem[pe.peSubSystem] : "Unknown"
            ,pe.peDLLFlags
            ,GetBool(pe.peDLLFlags & 0x0001)
            ,GetBool(pe.peDLLFlags & 0x0002)
@@ -251,55 +260,13 @@ static void __NEAR__ PaintNewHeaderPE_2( void )
            ,pe.peStackCommitSize
            ,pe.peHeapReserveSize
            ,pe.peHeapCommitSize
-           ,pe.peInterestingVASize
-           ,pe.peExportTableRVA
-           ,pe.peTotalExportDataSize
-           ,pe.peImportTableRVA);
+           ,pe.peDirSize);
 }
 
-static void __NEAR__ PaintNewHeaderPE_3( void )
+static const void (__NEAR__ * pephead[])(void) =
 {
-  twPrintF("Total import data size            = %lu bytes\n"
-           "Resource Table RVA                = %08lXH\n"
-           "Total resource data size          = %lu bytes\n"
-           "Exception Table RVA               = %08lXH\n"
-           "Total exception data size         = %lu bytes\n"
-           "Security Table RVA                = %08lXH\n"
-           "Total security data size          = %lu bytes\n"
-           "Fixup Table RVA                   = %08lXH\n"
-           "Total fixup data size             = %lu bytes\n"
-           "Debug Table RVA                   = %08lXH\n"
-           "Total number of debug directories = %lu bytes\n"
-           "Image Description string RVA      = %08lXH\n"
-           "Total description data size       = %lu bytes\n"
-           "Machine Specific value RVA        = %08lXH\n"
-           "Machine specific value size       = %lu bytes\n"
-           "Thread Local Storage RVA          = %08lXH\n"
-           "Total Thread Local Storage size   = %lu bytes"
-           ,pe.peTotalImportDataSize
-           ,pe.peResourceTableRVA
-           ,pe.peTotalResourceDataSize
-           ,pe.peExceptionTableRVA
-           ,pe.peTotalExceptionDataSize
-           ,pe.peSecurityTableRVA
-           ,pe.peTotalSecurityDataSize
-           ,pe.peFixupTableRVA
-           ,pe.peTotalFixupDataSize
-           ,pe.peDebugTableRVA
-           ,pe.peTotalDebugDirectories
-           ,pe.peImageDescriptionRVA
-           ,pe.peTotalDescriptionSize
-           ,pe.peMachineSpecificRVA
-           ,pe.peMachineSpecificSize
-           ,pe.peThreadLocalStorageRVA
-           ,pe.peTotalTLSSize);
-}
-
-static void (__NEAR__ * pephead[])( void ) =
-{
- PaintNewHeaderPE_1,
- PaintNewHeaderPE_2,
- PaintNewHeaderPE_3
+    PaintNewHeaderPE_1,
+    PaintNewHeaderPE_2
 };
 
 static void __FASTCALL__ PaintNewHeaderPE(TWindow * win,const void **ptr,unsigned npage,unsigned tpage)
@@ -312,7 +279,7 @@ static void __FASTCALL__ PaintNewHeaderPE(TWindow * win,const void **ptr,unsigne
   sprintf(text," Portable Executable Header [%d/%d] ",npage + 1,tpage);
   twSetTitleAttr(win,text,TW_TMODE_CENTER,dialog_cset.title);
   twSetFooterAttr(win,PAGEBOX_SUB,TW_TMODE_RIGHT,dialog_cset.selfooter);
-  if(npage < 3)
+  if(npage < 2)
   {
     twGotoXY(1,1);
     (*(pephead[npage]))();
@@ -325,7 +292,7 @@ static unsigned long __FASTCALL__ ShowNewHeaderPE( void )
  long fpos;
  fpos = BMGetCurrFilePos();
  entryPE = RVA2Phys(pe.peEntryPointRVA);
- if(PageBox(70,21,NULL,3,PaintNewHeaderPE) != -1 && entryPE && entryPE < bmGetFLength()) fpos = entryPE;
+ if(PageBox(70,21,NULL,2,PaintNewHeaderPE) != -1 && entryPE && entryPE < bmGetFLength()) fpos = entryPE;
  return fpos;
 }
 
@@ -341,44 +308,55 @@ static void __FASTCALL__ ObjPaintPE(TWindow * win,const void ** names,unsigned s
  twSetTitleAttr(win,buffer,TW_TMODE_CENTER,dialog_cset.title);
  twSetFooterAttr(win,PAGEBOX_SUB,TW_TMODE_RIGHT,dialog_cset.selfooter);
  twGotoXY(1,1);
+
+ memcpy(buffer, objs->oName, 8);
+ buffer[8] = 0;
+
  twPrintF("Object Name                    = %8s\n"
           "Virtual Size                   = %lX bytes\n"
           "RVA (relative virtual address) = %08lX\n"
           "Physical size                  = %08lX bytes\n"
           "Physical offset                = %08lX bytes\n"
-          "<Pointer to relocations>       = %08lX\n"
-          "<Pointer to line numbers>      = %08lX\n"
-          "<Number of relocations>        = %hu\n"
-          "<Number of line numbers>       = %hu\n"
+          "<Relocations>                  = %hu at %08lX\n"
+          "<Line numbers>                 = %hu at %08lX\n"
           "FLAGS: %lX\n"
-          "   [%c] Code object\n"
-          "   [%c] Initialized data object\n"
-          "   [%c] Uninitialized data object\n"
-          "   [%c] Object must not be cashed\n"
-          "   [%c] Object is not pageable\n"
-          "   [%c] Object is shared\n"
-          "   [%c] Executable object\n"
-          "   [%c] Readable object\n"
-          "   [%c] Writable object"
-          ,objs->oName
+          "   [%c] Executable code          "   "   [%c] Shared object\n"
+          "   [%c] Initialized data         "   "   [%c] Executable object\n"
+          "   [%c] Uninitialized data       "   "   [%c] Readable object\n"
+          "   [%c] Contains COMDAT          "   "   [%c] Writable object\n"
+          "   [%c] Contains comments or other info\n"
+          "   [%c] Won't become part of the image\n"
+          "   [%c] Contains extended relocations\n"
+          "   [%c] Discardable as needed\n"
+          "   [%c] Must not be cashed\n"
+          "   [%c] Not pageable\n"
+          "   Alignment                   = %u %s\n"
+
+          ,buffer
           ,objs->oVirtualSize
           ,objs->oRVA
           ,objs->oPhysicalSize
           ,objs->oPhysicalOffset
-          ,objs->oRelocPtr
-          ,objs->oLineNumbPtr
           ,objs->oNReloc
+          ,objs->oRelocPtr
           ,objs->oNLineNumb
+          ,objs->oLineNumbPtr
           ,objs->oFlags
-          ,GetBool((objs->oFlags & 0x00000020UL) == 0x00000020UL)
-          ,GetBool((objs->oFlags & 0x00000040UL) == 0x00000040UL)
-          ,GetBool((objs->oFlags & 0x00000080UL) == 0x00000080UL)
-          ,GetBool((objs->oFlags & 0x04000000UL) == 0x04000000UL)
-          ,GetBool((objs->oFlags & 0x08000000UL) == 0x08000000UL)
-          ,GetBool((objs->oFlags & 0x10000000UL) == 0x10000000UL)
-          ,GetBool((objs->oFlags & 0x20000000UL) == 0x20000000UL)
-          ,GetBool((objs->oFlags & 0x40000000UL) == 0x40000000UL)
-          ,GetBool((objs->oFlags & 0x80000000UL) == 0x80000000UL));
+
+          ,GetBool(objs->oFlags & 0x00000020UL), GetBool(objs->oFlags & 0x10000000UL)
+          ,GetBool(objs->oFlags & 0x00000040UL), GetBool(objs->oFlags & 0x20000000UL)
+          ,GetBool(objs->oFlags & 0x00000080UL), GetBool(objs->oFlags & 0x40000000UL)
+          ,GetBool(objs->oFlags & 0x00001000UL), GetBool(objs->oFlags & 0x80000000UL)
+          ,GetBool(objs->oFlags & 0x00000200UL)
+          ,GetBool(objs->oFlags & 0x00000800UL)
+          ,GetBool(objs->oFlags & 0x01000000UL)
+          ,GetBool(objs->oFlags & 0x02000000UL)
+          ,GetBool(objs->oFlags & 0x04000000UL)
+          ,GetBool(objs->oFlags & 0x08000000UL)
+
+          ,objs->oFlags&0x00F00000 ? 1 << (((objs->oFlags&0x00F00000)>>20)-1) : 0
+          ,objs->oFlags&0x00F00000 ? "byte(s)" : "(default)");
+
  twRefreshFullWin(win);
 }
 
@@ -435,80 +413,24 @@ static unsigned __NEAR__ __FASTCALL__ GetImportCountPE(BGLOBAL handle,unsigned l
   return count;
 }
 
-static char * __NEAR__ __FASTCALL__ __ReadASCIIZ(BGLOBAL handle,unsigned long offset,int align)
+/* returns really readed number of characters */
+unsigned __NEAR__ __FASTCALL__ __peReadASCIIZName(BGLOBAL handle,unsigned long offset,char *buff, unsigned cb_buff)
 {
-  int j,size,shift;
-  tBool key;
+  unsigned j;
   char ch;
-  char * asciiz;
   unsigned long fpos;
   fpos = bioTell(handle);
   j = 0;
-  shift = 0;
-  if(align == 2) shift = 2;
-  bioSeek(handle,offset + shift,SEEKF_START);
+  bioSeek(handle,offset,SEEKF_START);
   while(1)
   {
-    j++;
-    if(j > 300 || bioEOF(handle)) break;
     ch = bioReadByte(handle);
-    if(ch == 0) break;
+    buff[j++] = ch;
+    if(!ch || j >= cb_buff || bioEOF(handle)) break;
   }
-  size = j;
-  key = False;
-  if(align == 1)
-  {
-    size = j > 66 ? 73 : j + 7;
-    j = size - 7;
-    if(size == 73) key = True;
-  }
-  asciiz = PMalloc(size);
-  if( asciiz != 0 )
-  {
-    bioSeek(handle,offset + shift,SEEKF_START);
-    j--;
-    bioReadBuffer(handle,asciiz,j);
-    if(key) { asciiz[j++] = TWC_RT_ARROW; asciiz[j++] = TWC_RT_ARROW; asciiz[j++] = TWC_RT_ARROW; }
-    bioSeek(handle,fpos,SEEKF_START);
-    if(align == 1) asciiz[j++] = '';
-    asciiz[j] = 0;
-  }
-  return asciiz;
-}
-
-static char * __NEAR__ __FASTCALL__ __ReadASCIIZFull(BGLOBAL handle,unsigned long offset,int align)
-{
-  int j,size,shift;
-  char ch;
-  char * asciiz;
-  unsigned long fpos;
-  fpos = bioTell(handle);
-  j = 0;
-  shift = 0;
-  if(align == 2) shift = 2;
-  bioSeek(handle,offset + shift,SEEKF_START);
-  while(1)
-  {
-    j++;
-    if(j > 300 || bioEOF(handle)) break;
-    ch = bioReadByte(handle);
-    if(ch == 0) break;
-  }
-  size = j;
-  if(align == 1)
-  {
-    if(size > 80) j = size = 80;
-  }
-  asciiz = PMalloc(size);
-  if( asciiz != 0 )
-  {
-    bioSeek(handle,offset + shift,SEEKF_START);
-    j--;
-    bioReadBuffer(handle,asciiz,j);
-    bioSeek(handle,fpos,SEEKF_START);
-    asciiz[j] = 0;
-  }
-  return asciiz;
+  if(j) buff[j-1]=0;
+  bioSeek(handle,fpos,SEEKF_START);
+  return j;
 }
 
 static unsigned __NEAR__ __FASTCALL__ __ReadImportPE(BGLOBAL handle,unsigned long phys,memArray *obj,unsigned nnames)
@@ -519,16 +441,15 @@ static unsigned __NEAR__ __FASTCALL__ __ReadImportPE(BGLOBAL handle,unsigned lon
   bioSeek(handle,phys,SEEKF_START);
   for(i = 0;i < nnames;i++)
   {
-    char *tmp;
+    char tmp[300];
     tBool is_eof;
     rva = fioReadDWord(handle,12L,SEEKF_CUR);
     bioSeek(handle,4L,SEEKF_CUR);
     addr = RVA2Phys(rva);
-    if(!(tmp = __ReadASCIIZ(handle,addr,0))) break;
+    __peReadASCIIZName(handle,addr,tmp,sizeof(tmp));
     if(IsKbdTerminate()) break;
     is_eof = bioEOF(handle);
-    if(!ma_AddString(obj,is_eof ? CORRUPT_BIN_MSG : tmp,True)) { PFree(tmp); break; }
-    PFree(tmp);
+    if(!ma_AddString(obj,is_eof ? CORRUPT_BIN_MSG : tmp,True)) break;
     if(is_eof) break;
   }
   bioSeek(handle,fpos,SEEKF_START);
@@ -557,41 +478,40 @@ static unsigned __FASTCALL__ GetImpCountPE(BGLOBAL handle)
 
 static tBool __FASTCALL__  __ReadImpContPE(BGLOBAL handle,memArray * obj,unsigned nnames)
 {
-  unsigned i;
+  unsigned i,VA;
   unsigned long Hint,rphys;
   bioSeek(handle,addr_shift_pe,SEEKF_START);
+  VA = pePA2VA(addr_shift_pe);
   for(i = 0;i < nnames;i++)
   {
+    char stmp[300];
+    tBool is_eof;
+    sprintf(stmp,".%08X: ", VA);
+    VA += 4;
     Hint = bioReadDWord(handle);
     if(!(Hint & 0x80000000UL))
     {
-       rphys = RVA2Phys(Hint & 0x7FFFFFFFUL);
-       if(rphys > bmGetFLength() || bioEOF(handle))
-       {
-          if(!ma_AddString(obj,CORRUPT_BIN_MSG,True)) break;
-       }
-       else
-       {
-         char *tmp;
-         tBool is_eof;
-         if(!(tmp = __ReadASCIIZ(handle,rphys,2))) break;
-         is_eof = bioEOF(handle);
-         if(!ma_AddString(obj,is_eof ? CORRUPT_BIN_MSG : tmp,True)) { PFree(tmp); break; }
-         PFree(tmp);
-         if(IsKbdTerminate() || is_eof) break;
-       }
+      rphys = RVA2Phys(Hint & 0x7FFFFFFFUL);
+      if(rphys > bmGetFLength() || bioEOF(handle))
+      {
+        if(!ma_AddString(obj,CORRUPT_BIN_MSG,True)) break;
+      }
+      else
+      {
+        char tmp[256];
+        __peReadASCIIZName(handle,rphys+2,tmp,sizeof(tmp));
+        strcat(stmp,tmp);
+      }
     }
     else
     {
-      char stmp[80];
-      tBool is_eof;
-      is_eof = bioEOF(handle);
-      if(is_eof) strcpy(stmp,CORRUPT_BIN_MSG);
-      sprintf(stmp,"< By ordinal >   @%lu",Hint & 0x7FFFFFFFUL);
-      if(IsKbdTerminate()) break;
-      if(!ma_AddString(obj,stmp,True)) break;
-      if(is_eof) break;
+      char tmp[80];
+      sprintf(tmp,"< By ordinal >   @%lu",Hint & 0x7FFFFFFFUL);
+      strcat(stmp,tmp);
     }
+    is_eof = bioEOF(handle);
+    if(!ma_AddString(obj,is_eof ? CORRUPT_BIN_MSG : stmp,True)) break;
+    if(IsKbdTerminate() || is_eof) break;
   }
   return True;
 }
@@ -604,10 +524,10 @@ static unsigned long __FASTCALL__ ShowModRefPE( void )
   unsigned nnames;
   unsigned long phys,fret;
   fret = BMGetCurrFilePos();
-  if(!pe.peImportTableRVA) { not_found: NotifyBox(NOT_ENTRY," Module References "); return fret; }
+  if(!peDir[PE_IMPORT].rva) { not_found: NotifyBox(NOT_ENTRY," Module References "); return fret; }
   handle = pe_cache;
   bioSeek(handle,0L,SEEK_SET);
-  phys = RVA2Phys(pe.peImportTableRVA);
+  phys = RVA2Phys(peDir[PE_IMPORT].rva);
   if(!(nnames = GetImportCountPE(handle,phys))) goto not_found;
   if(!(obj = ma_Build(nnames,True))) goto exit;
   if(__ReadImportPE(handle,phys,obj,nnames))
@@ -639,58 +559,68 @@ static unsigned long __FASTCALL__ ShowModRefPE( void )
 
 static ExportTablePE et;
 
+static void inline writeExportVA(unsigned long va, BGLOBAL handle, char *buf, unsigned long bufsize)
+{
+    // check for forwarded export
+    if (va>=peDir[PE_EXPORT].rva && va<peDir[PE_EXPORT].rva+peDir[PE_EXPORT].size)
+        __peReadASCIIZName(handle, RVA2Phys(va), buf, bufsize);
+    // normal export
+    else sprintf(buf, ".%08lX", va + pe.peImageBase);
+}
+
 static tBool __FASTCALL__ PEExportReadItems(BGLOBAL handle,memArray * obj,unsigned nnames)
 {
   unsigned long nameaddr;
-  unsigned long expaddr,nameptr;
+  unsigned long expaddr,nameptr,*addr;
   unsigned i,ord;
-  char buff[10];
+  char buff[80];
+
   nameptr = RVA2Phys(et.etNamePtrTableRVA);
-  expaddr  = RVA2Phys(et.etOrdinalTableRVA);
-  for(i = 0;i < nnames;i++)
+  expaddr = RVA2Phys(et.etOrdinalTableRVA);
+  if(!(addr = PMalloc(sizeof(unsigned long)*nnames))) return True;
+  bioSeek(handle,RVA2Phys(et.etAddressTableRVA),SEEKF_START);
+  bioReadBuffer(handle,addr,sizeof(unsigned long)*nnames);
+
+  for(i = 0;i < et.etNumNamePtrs;i++)
   {
-    char *stmp;
+    char stmp[300];
     tBool is_eof;
     nameaddr = fioReadDWord2Phys(handle,nameptr + 4*i,SEEKF_START);
-    if(!(stmp = __ReadASCIIZ(handle,nameaddr,1)) || IsKbdTerminate()) break;
-    ord = fioReadWord(handle,expaddr + i*2,SEEKF_START) + (unsigned)et.etOrdinalBase;
+    __peReadASCIIZName(handle,nameaddr, stmp, sizeof(stmp)-sizeof(buff)-1);
+    if(IsKbdTerminate()) break;
+    ord = fioReadWord(handle,expaddr + i*2,SEEKF_START);
     is_eof = bioEOF(handle);
-    sprintf(buff,"%i",ord);
+    sprintf(buff,"%c%-9lu ", LB_ORD_DELIMITER, ord+(unsigned long)et.etOrdinalBase);
+    writeExportVA(addr[ord], handle, buff+11, sizeof(buff)-11);
+    addr[ord] = 0;
     strcat(stmp,buff);
-    if(!ma_AddString(obj,is_eof ? CORRUPT_BIN_MSG : stmp,True)) { PFree(stmp); break; }
-    PFree(stmp);
+    if(!ma_AddString(obj,is_eof ? CORRUPT_BIN_MSG : stmp,True)) break;  // -XF removed PFree(stmp)
     if(is_eof) break;
   }
+
+  for(i = 0;i < nnames;i++)
+  {
+    if(addr[i])
+    {
+      ord = i+et.etOrdinalBase;
+      sprintf(buff," < by ordinal > %c%-9lu ",LB_ORD_DELIMITER, (unsigned long)ord);
+      writeExportVA(addr[i], handle, buff+27, sizeof(buff)-27);
+      if(!ma_AddString(obj,buff,True)) break;
+    }
+  }
+
+  PFree(addr);
   return True;
 }
-#if 0
-static unsigned __FASTCALL__ PEExportReadFullName(BGLOBAL handle,char * names,unsigned index)
-{
-  unsigned long nameaddr;
-  unsigned long expaddr,nameptr;
-  unsigned ord;
-  char * name;
-  nameptr = RVA2Phys(et.etNamePtrTableRVA);
-  expaddr  = RVA2Phys(et.etOrdinalTableRVA);
-  nameaddr = fioReadDWord2Phys(handle,nameptr + 4*index,SEEKF_START);
-  name = __ReadASCIIZFull(handle,nameaddr,1);
-  if(name)
-  {
-    strncpy(names,name,255);
-    PFree(name);
-  }
-  ord = fioReadWord(handle,expaddr + index*2,SEEKF_START);
-  return ord;
-}
-#endif
+
 static unsigned __FASTCALL__ PEExportNumItems(BGLOBAL handle)
 {
   unsigned long addr;
-  if(!pe.peExportTableRVA) return 0;
-  addr = RVA2Phys(pe.peExportTableRVA);
+  if(!peDir[PE_EXPORT].rva) return 0;
+  addr = RVA2Phys(peDir[PE_EXPORT].rva);
   bioSeek(handle,addr,SEEKF_START);
   bioReadBuffer(handle,(void *)&et,sizeof(et));
-  return (unsigned)(et.etNumNamePtrs);
+  return (unsigned)(et.etNumEATEntries);
 }
 
 static unsigned long __NEAR__ __FASTCALL__ CalcEntryPE(unsigned ordinal,tBool dispmsg)
@@ -717,27 +647,26 @@ static unsigned long __FASTCALL__ ShowExpNamPE( void )
   int ret;
   unsigned ordinal;
   unsigned long addr;
-  char exp_nam[256];
+  char exp_nam[256], exp_buf[300];
   fpos = BMGetCurrFilePos();
   strcpy(exp_nam,EXP_TABLE);
-  if(pe.peExportTableRVA)
+  if(peDir[PE_EXPORT].rva)
   {
-    addr = RVA2Phys(pe.peExportTableRVA);
+    addr = RVA2Phys(peDir[PE_EXPORT].rva);
     bmSeek(addr,SEEKF_START);
     bmReadBuffer((void *)&et,sizeof(et));
     if(et.etNameRVA)
     {
-      char *name,sftime[80];
+      char sftime[80];
       struct tm * tm;
-      name = __ReadASCIIZFull(bmbioHandle(),RVA2Phys(et.etNameRVA),1);
-      if(strlen(name) > 50) strcpy(&name[50],"...");
+      __peReadASCIIZName(bmbioHandle(),RVA2Phys(et.etNameRVA),exp_buf, sizeof(exp_buf));
+      if(strlen(exp_buf) > 50) strcpy(&exp_buf[50],"...");
       tm = localtime((time_t *)&et.etDateTime);
       strftime(sftime,sizeof(sftime),"%x",tm);
       sprintf(exp_nam," %s (ver=%hX.%hX %s) "
-              ,name
+              ,exp_buf
               ,et.etMajVer, et.etMinVer
               ,sftime);
-      PFree(name);
     }
   }
   ret = fmtShowList(PEExportNumItems,PEExportReadItems,
@@ -750,6 +679,61 @@ static unsigned long __FASTCALL__ ShowExpNamPE( void )
   return fpos;
 }
 
+
+static tBool __FASTCALL__ PEReadRVAs(BGLOBAL handle, memArray * obj, unsigned nnames)
+{
+  unsigned i;
+  static const char *rvaNames[] =
+  {
+      "~Export Table        ",
+      "~Import Table        ",
+      "~Resource Table      ",
+      "E~xception Table     ",
+      "Sec~urity Table      ",
+      "Re~location Table    ",
+      "~Debug Information   ",
+      "Image De~scription   ",
+      "~Machine Specific    ",
+      "~Thread Local Storage",
+      "Load Confi~guration  ",
+      "~Bound Import Table  ",
+      "Import ~Adress Table ",
+      "Dela~y Import Table  ",
+      "~COM+                ",
+      "Reser~ved            "
+  };
+
+  for (i=0; i<pe.peDirSize; i++)
+  {
+    char foo[80];
+
+    sprintf(foo, "%s  %08lX  %8lu",
+        i<ARRAY_SIZE(rvaNames) ? rvaNames[i] : "Unknown             ",
+        peDir[i].rva,
+        peDir[i].size);
+    if (!ma_AddString(obj, foo, True))
+        return False;
+  }
+
+  return True;
+}
+
+static unsigned __FASTCALL__ PENumRVAs(BGLOBAL handle)
+{
+  return pe.peDirSize;
+}
+
+static unsigned long __FASTCALL__ ShowPERVAs( void )
+{
+  unsigned long fpos = BMGetCurrFilePos();
+  int ret;
+  ret = fmtShowList(PENumRVAs, PEReadRVAs, " Directory Entry       RVA           size ", LB_SELECTIVE|LB_USEACC, NULL);
+  if (ret!=-1 && peDir[ret].rva)
+    fpos = RVA2Phys(peDir[ret].rva);
+
+  return fpos;
+}
+
 /***************************************************************************/
 /************************  FOR PE  *****************************************/
 /***************************************************************************/
@@ -771,14 +755,9 @@ static linearArray *CurrPEChain = NULL;
 static tCompare __FASTCALL__ compare_pe_reloc_s(const void __HUGE__ *e1,const void __HUGE__ *e2)
 {
   const RELOC_PE __HUGE__ *p1,__HUGE__ *p2;
-  tCompare ret;
   p1 = (const RELOC_PE __HUGE__ *)e1;
   p2 = (const RELOC_PE __HUGE__ *)e2;
-  if(p1->laddr < p2->laddr) ret = -1;
-  else
-    if(p1->laddr > p2->laddr) ret = 1;
-    else                      ret = 0;
-  return ret;
+  return __CmpLong__(p1->laddr,p2->laddr);
 }
 
 static void __NEAR__ __FASTCALL__ BuildPERefChain( void )
@@ -791,20 +770,20 @@ static void __NEAR__ __FASTCALL__ BuildPERefChain( void )
   TWindow *w,*usd;
   usd = twUsedWin();
   if(!(CurrPEChain = la_Build(0,sizeof(RELOC_PE),MemOutBox))) return;
-  w = CrtDlgWndnls(" System is busy ",49,1);
+  w = CrtDlgWndnls(SYSTEM_BUSY,49,1);
   if(!PubNames) pe_ReadPubNameList(bmbioHandle(),MemOutBox);
   twUseWin(w);
   twGotoXY(1,1);
-  twPutS(" Building reference chains for PE format");
+  twPutS(BUILD_REFS);
   twUseWin(usd);
   handle = pe_cache;
   bioSeek(handle,0L,SEEK_SET);
   /**
      building references chain for external
   */
-  if(pe.peImportTableRVA)
+  if(peDir[PE_IMPORT].rva)
   {
-    phys = RVA2Phys(pe.peImportTableRVA);
+    phys = RVA2Phys(peDir[PE_IMPORT].rva);
     nnames = GetImportCountPE(handle,phys);
   }
   else
@@ -845,14 +824,14 @@ static void __NEAR__ __FASTCALL__ BuildPERefChain( void )
   /**
      building references chain for internal
   */
-  if(pe.peTotalFixupDataSize)
+  if(peDir[PE_FIXUP].size)
   {
-    phys = RVA2Phys(pe.peFixupTableRVA);
+    phys = RVA2Phys(peDir[PE_FIXUP].rva);
     bioSeek(handle,phys,SEEKF_START);
     cpos = bioTell(handle);
-    while(bioTell(handle) < cpos + pe.peTotalFixupDataSize)
+    while(bioTell(handle) < cpos + peDir[PE_FIXUP].size)
     {
-      uint16_t typeoff;
+      tUInt16 typeoff;
       unsigned long page,physoff,size,ccpos;
       tBool is_eof;
       ccpos = bioTell(handle);
@@ -886,17 +865,17 @@ static RELOC_PE __HUGE__ * __NEAR__ __FASTCALL__ __found_RPE(unsigned long laddr
   return la_Find(CurrPEChain,&key,compare_pe_reloc_s);
 }
 
-static int __NEAR__ __FASTCALL__ BuildReferStrPE(char *str,RELOC_PE __HUGE__ *rpe,int flags)
+static unsigned long __NEAR__ __FASTCALL__ BuildReferStrPE(char *str,RELOC_PE __HUGE__ *rpe,int flags)
 {
    BGLOBAL handle,handle2,handle3;
    unsigned long phys,rva,magic,Hint;
-   int retrf;
+   unsigned long retrf;
    ImportDirPE ipe;
    char buff[400];
    handle = pe_cache;
    handle2 = pe_cache4;
    handle3 = pe_cache3;
-   phys = RVA2Phys(pe.peImportTableRVA);
+   phys = RVA2Phys(peDir[PE_IMPORT].rva);
    bioSeek(handle,phys + 20L*rpe->modidx,SEEKF_START);
    rva = fioReadDWord(handle,12L,SEEKF_CUR);
    retrf = RAPREF_DONE;
@@ -907,17 +886,17 @@ static int __NEAR__ __FASTCALL__ BuildReferStrPE(char *str,RELOC_PE __HUGE__ *rp
      bioSeek(handle2,RVA2Phys(rva),SEEKF_START);
      bioReadBuffer(handle2,buff,400);
      buff[399] = 0;
-     /* 
-        Removing extension .dll from import name. 
-        Modified by "Kostya Nosov" <k-nosov@yandex.ru> 
+     /*
+        Removing extension .dll from import name.
+        Modified by "Kostya Nosov" <k-nosov@yandex.ru>
      */
      is_ext = strrchr(buff,'.');
-     if(is_ext && !strcmp(is_ext,".dll")) 
+     if(is_ext && !strcmp(is_ext,".dll"))
      {
           *is_ext = 0;
           sprintf(&str[strlen(str)]," %s.",buff);
      }
-     else      
+     else
        sprintf(&str[strlen(str)]," <%s>.",buff);
      bioSeek(handle3,phys + rpe->modidx*sizeof(ImportDirPE),SEEKF_START);
      bioReadBuffer(handle3,&ipe,sizeof(ImportDirPE));
@@ -954,14 +933,12 @@ static int __NEAR__ __FASTCALL__ BuildReferStrPE(char *str,RELOC_PE __HUGE__ *rp
    {
      unsigned long delta,value,point_to;
      const char *pe_how;
-       retrf = RAPREF_PROCEEDCMT;
-       strcat(str,"*this.");
-       bioSeek(handle3,rpe->laddr,SEEKF_START);
-       value = bioReadDWord(handle3);
-       delta = pe.peImageBase;
-       point_to = 0;
-       switch(rpe->import.type)
-       {
+     bioSeek(handle3,rpe->laddr,SEEKF_START);
+     value = bioReadDWord(handle3);
+     delta = pe.peImageBase;
+     point_to = 0;
+     switch(rpe->import.type)
+     {
           default:
           case 0: /** ABSOLUTE, fixup is skipped */
                   pe_how = "(";
@@ -986,25 +963,32 @@ static int __NEAR__ __FASTCALL__ BuildReferStrPE(char *str,RELOC_PE __HUGE__ *rp
           case 5: /** MIPS JUMP ADDR */
                   pe_how = "((mips)";
                   break;
-       }
-     if(flags & APREF_USE_TYPE)  strcat(str,pe_how);
-     /** if out of physical image */
-     strcat(str,Get8Digit(point_to ? point_to : value-delta));
-     if(flags & APREF_USE_TYPE)  strcat(str,")");
+     }
+     retrf = point_to ? point_to : value-delta;
+     if(!(flags & APREF_SAVE_VIRT))
+     {
+       strcat(str,"*this.");
+       if(flags & APREF_USE_TYPE) strcat(str,pe_how);
+       /** if out of physical image */
+       strcat(str,Get8Digit(retrf));
+       if(flags & APREF_USE_TYPE)  strcat(str,")");
+     }
+     else strcat(str,Get8Digit(value));
    }
    return retrf;
 }
 
-static int __FASTCALL__ AppendPERef(char *str,unsigned long ulShift,int flags,int codelen,unsigned long r_sh)
+static unsigned long __FASTCALL__ AppendPERef(char *str,unsigned long ulShift,int flags,int codelen,unsigned long r_sh)
 {
   RELOC_PE __HUGE__ *rpe;
-  int retrf;
+  unsigned long retrf;
   BGLOBAL b_cache;
   char buff[400];
   UNUSED(codelen);
   b_cache = pe_cache3;
   retrf = RAPREF_NONE;
-  if(pe.peImportTableRVA || pe.peFixupTableRVA)
+  if(flags & APREF_TRY_PIC) return RAPREF_NONE;
+  if(peDir[PE_IMPORT].rva || peDir[PE_FIXUP].rva)
   {
     bioSeek(b_cache,
             RVA2Phys(bmReadDWordEx(ulShift,SEEK_SET) - pe.peImageBase),
@@ -1043,20 +1027,30 @@ static void __FASTCALL__ initPE( void )
    int i;
    BGLOBAL main_handle;
    PMRegLowMemCallBack(peLowMemFunc);
+
    bmReadBufferEx(&pe,sizeof(PEHEADER),headshift,SEEKF_START);
+
+   if(!(peDir = PMalloc(sizeof(PERVA)*pe.peDirSize)))
+   {
+     MemOutBox("PE initialization");
+     exit(EXIT_FAILURE);
+   }
+   bmReadBuffer(peDir, sizeof(PERVA)*pe.peDirSize);
+
    if(!(peVA = PMalloc(sizeof(PE_ADDR)*pe.peObjects)))
    {
      MemOutBox("PE initialization");
      exit(EXIT_FAILURE);
    }
-   main_handle = bmbioHandle();
-   bioSeek(main_handle,0x18 + pe.peNTHdrSize + headshift,SEEKF_START);
+   bmSeek(0x18 + pe.peNTHdrSize + headshift,SEEKF_START);
    for(i = 0;i < pe.peObjects;i++)
    {
-     peVA[i].rva = fioReadDWord(main_handle,12L,SEEKF_CUR);
-     peVA[i].phys = fioReadDWord(main_handle,4L,SEEKF_CUR);
-     bioSeek(main_handle,16L,SEEKF_CUR);
+     peVA[i].rva = bmReadDWordEx(12L,SEEKF_CUR);
+     peVA[i].phys = bmReadDWordEx(4L,SEEKF_CUR);
+     bmSeek(16L,SEEKF_CUR);
    }
+
+   main_handle = bmbioHandle();
    if((pe_cache = bioDupEx(main_handle,BBIO_SMALL_CACHE_SIZE)) == &bNull) pe_cache = main_handle;
    if((pe_cache1 = bioDupEx(main_handle,BBIO_SMALL_CACHE_SIZE)) == &bNull) pe_cache1 = main_handle;
    if((pe_cache2 = bioDupEx(main_handle,BBIO_SMALL_CACHE_SIZE)) == &bNull) pe_cache2 = main_handle;
@@ -1067,7 +1061,8 @@ static void __FASTCALL__ initPE( void )
 static void __FASTCALL__ destroyPE( void )
 {
   BGLOBAL main_handle;
-  if(peVA) PFree(peVA);
+  if(peVA) PFREE(peVA);
+  if(peDir) PFREE(peDir);
   if(PubNames) { la_Destroy(PubNames); PubNames = 0; }
   if(CurrPEChain) { la_Destroy(CurrPEChain); CurrPEChain = 0; } /* Fixed by "Kostya Nosov" <k-nosov@yandex.ru> */
   main_handle = bmbioHandle();
@@ -1099,8 +1094,8 @@ static tBool __FASTCALL__ peAddressResolv(char *addr,unsigned long cfpos)
  /* Since this function is used in references resolving of disassembler
     it must be seriously optimized for speed. */
  tBool bret = True;
- uint32_t res;
- if(cfpos >= headshift && cfpos < headshift + sizeof(PEHEADER))
+ tUInt32 res;
+ if(cfpos >= headshift && cfpos < headshift + sizeof(PEHEADER) + pe.peDirSize*sizeof(PERVA))
  {
     strcpy(addr,"PEH :");
     strcpy(&addr[5],Get4Digit(cfpos - headshift));
@@ -1113,7 +1108,7 @@ static tBool __FASTCALL__ peAddressResolv(char *addr,unsigned long cfpos)
     strcpy(&addr[5],Get4Digit(cfpos - headshift - pe.peNTHdrSize - 0x18));
  }
  else  /* Added by "Kostya Nosov" <k-nosov@yandex.ru> */
-   if((res=pePA2VA(cfpos))!=0) 
+   if((res=pePA2VA(cfpos))!=0)
    {
      addr[0] = '.';
      strcpy(&addr[1],Get8Digit(res));
@@ -1152,15 +1147,10 @@ static unsigned long __FASTCALL__ pePA2VA(unsigned long pa)
 static void __FASTCALL__ pe_ReadPubName(BGLOBAL b_cache,const struct PubName *it,
                            char *buff,unsigned cb_buff)
 {
-    char *stmp;
-    stmp = __ReadASCIIZFull(b_cache,it->nameoff,0);
-    if(stmp)
-    {
-      strncpy(buff,stmp,cb_buff);
-      buff[cb_buff-1] = 0;
-      PFree(stmp);
-    }
-    else buff[0] = 0;
+    char stmp[300];
+    __peReadASCIIZName(b_cache,it->nameoff,stmp, sizeof(stmp));
+    strncpy(buff,stmp,cb_buff);
+    buff[cb_buff-1] = 0;
 }
 
 static tBool __NEAR__ __FASTCALL__ FindPubName(char *buff,unsigned cb_buff,unsigned long pa)
@@ -1203,12 +1193,12 @@ static unsigned long __FASTCALL__ peGetPubSym(char *str,unsigned cb_str,unsigned
 }
 
 static unsigned __FASTCALL__ peGetObjAttr(unsigned long pa,char *name,unsigned cb_name,
-                      unsigned long *start,unsigned long *end,int *class,int *bitness)
+                      unsigned long *start,unsigned long *end,int *_class,int *bitness)
 {
   unsigned i,nitems,ret;
   *start = 0;
   *end = bmGetFLength();
-  *class = OC_NOOBJECT;
+  *_class = OC_NOOBJECT;
   *bitness = bitnessPE(pa);
   name[0] = 0;
   nitems = pe.peObjects;
@@ -1230,7 +1220,7 @@ static unsigned __FASTCALL__ peGetObjAttr(unsigned long pa,char *name,unsigned c
     {
       *start = po.oPhysicalOffset;
       *end = *start + po.oPhysicalSize;
-      *class = po.oFlags & 0x00000020L ? OC_CODE : OC_DATA;
+      *_class = po.oFlags & 0x00000020L ? OC_CODE : OC_DATA;
       strncpy(name,(const char *)po.oName,cb_name);
       ret = i+1;
       break;
@@ -1245,8 +1235,8 @@ static int __FASTCALL__ pePlatform( void ) { return DISASM_CPU_IX86; }
 REGISTRY_BIN peTable =
 {
   "PE (Portable Executable)",
-  { "PEHelp", "Import", "Export", NULL, NULL, NULL, NULL, "PEHead", NULL, "Object" },
-  { PEHelp, ShowModRefPE, ShowExpNamPE, NULL, NULL, NULL, NULL, ShowNewHeaderPE, NULL, ShowObjectsPE },
+  { "PEHelp", "Import", "Export", NULL, NULL, NULL, NULL, "PEHead", "Dir   ", "Object" },
+  { PEHelp, ShowModRefPE, ShowExpNamPE, NULL, NULL, NULL, NULL, ShowNewHeaderPE, ShowPERVAs, ShowObjectsPE },
   IsPE, initPE, destroyPE,
   NULL,
   AppendPERef,
