@@ -1,4 +1,4 @@
-/** 
+/**
  * @namespace   biew_plugins_auto
  * @file        plugins/bin/rdoff.h
  * @brief       This file contains RDOFF v1 file definitions.
@@ -28,17 +28,17 @@
 /** RDOFF v1 fixed header */
 typedef struct tag_rdoff_Header
 {
-  uint8_t  id[6]; /**< RDOFF1 or RDOFF\01 */
-  uint32_t header_len;
+  tUInt8  id[6]; /**< RDOFF1 or RDOFF\01 */
+  tUInt32 header_len;
 }rdoff_Header;
 
 typedef struct tag_RDOFF_RELOC
 {
-  uint8_t  reflen; /**< length of references */
-  uint8_t  is_rel; /**< is relative fixup */
-  uint16_t segto;  /**< logical # of segment or ext. reference
+  tUInt8  reflen; /**< length of references */
+  tUInt8  is_rel; /**< is relative fixup */
+  tUInt16 segto;  /**< logical # of segment or ext. reference
                             i.e. max external refers = 65536-2 */
-  uint32_t offset; /**< absolute offset within file */
+  tUInt32 offset; /**< absolute offset within file */
 }RDOFF_RELOC;
 
 #ifdef __HAVE_PRAGMA_PACK__
