@@ -1,4 +1,4 @@
-/** 
+/**
  * @namespace   biew_plugins_I
  * @file        plugins/disasm.c
  * @brief       This file contains universal interface for any disassembler.
@@ -33,6 +33,8 @@
 #include "bconsole.h"
 #include "editor.h"
 #include "codeguid.h"
+#include "search.h"
+#include "tstrings.h"
 #include "biewlib/file_ini.h"
 #include "biewlib/biewlib.h"
 #include "biewlib/pmalloc.h"
@@ -40,11 +42,15 @@
 
 extern REGISTRY_DISASM ix86_Disasm;
 extern REGISTRY_DISASM Null_Disasm;
+extern REGISTRY_DISASM AVR_Disasm;
+extern REGISTRY_DISASM Java_Disasm;
 
 static REGISTRY_DISASM *mainDisasmTable[] =
 {
   &ix86_Disasm,
-  &Null_Disasm
+  &Null_Disasm,
+  &AVR_Disasm,
+  &Java_Disasm
 };
 
 static unsigned DefDisasmSel = __DEFAULT_DISASM;
@@ -59,7 +65,8 @@ static unsigned char *CurrStrLenBuff = NULL;
 static unsigned long *PrevStrLenAddr = NULL;
 static char          LastPrevLen;
 static char          PrevStrCount = 0;
-static char *        disCodeBuffer = NULL,*disCodeBuf2 = NULL,*disCodeBufPredict = NULL;
+static char *        disCodeBuffer = NULL,*disCodeBuf2 = NULL;
+static char *        disCodeBufPredict = NULL;
 static int           disMaxCodeLen;
 
 tBool DisasmPrepareMode = False;
@@ -94,11 +101,12 @@ static void __FASTCALL__ FillPrevAsmPage(unsigned long bound,unsigned predist)
   unsigned long distin,addr;
   unsigned j;
   unsigned totallen;
+  tAbsCoord height = twGetClientHeight(MainWnd);
   char showref,addrdet;
-  if(!predist) predist = Height*disMaxCodeLen;
+  if(!predist) predist = height*disMaxCodeLen;
   predist = (predist/16)*16+16;   /** align on 16-byte boundary */
   distin = bound >= predist ? bound - predist : 0;
-  memset(PrevStrLenAddr,TWC_DEF_FILLER,tvioHeight*sizeof(long));
+  memset(PrevStrLenAddr,TWC_DEF_FILLER,height*sizeof(long));
   PrevStrCount = 0;
   totallen = 0;
   showref = disNeedRef;
@@ -114,14 +122,14 @@ static void __FASTCALL__ FillPrevAsmPage(unsigned long bound,unsigned predist)
        dret = Disassembler(distin,(void *)disCodeBuffer,__DISF_SIZEONLY);
        if(addr >= bound) break;
        totallen += dret.codelen;
-       if(j < Height) PrevStrLenAddr[j] = addr;
+       if(j < height) PrevStrLenAddr[j] = addr;
        else
        {
-          memmove(PrevStrLenAddr,&PrevStrLenAddr[1],(Height - 1)*sizeof(long));
-          PrevStrLenAddr[Height - 1] = addr;
+          memmove(PrevStrLenAddr,&PrevStrLenAddr[1],(height - 1)*sizeof(long));
+          PrevStrLenAddr[height - 1] = addr;
        }
   }
-  PrevStrCount = j < Height ? j : Height;
+  PrevStrCount = j < height ? j : height;
   LastPrevLen = PrevStrCount ? bound - PrevStrLenAddr[PrevStrCount - 1] : 0;
   disNeedRef = showref;
   DisasmPrepareMode = False;
@@ -130,13 +138,14 @@ static void __FASTCALL__ FillPrevAsmPage(unsigned long bound,unsigned predist)
 
 static void __FASTCALL__ PrepareAsmLines(int keycode,unsigned long cfpos)
 {
+  tAbsCoord height = twGetClientHeight(MainWnd);
   switch(keycode)
   {
     case KE_DOWNARROW:
                     PrevStrLen = CurrStrLenBuff[0];
                     CurrStrLen = CurrStrLenBuff[1];
-                    if((unsigned)PrevStrCount < Height) PrevStrCount++;
-                    else                                memmove(PrevStrLenAddr,&PrevStrLenAddr[1],sizeof(long)*(Height - 1));
+                    if((unsigned)PrevStrCount < height) PrevStrCount++;
+                    else                                memmove(PrevStrLenAddr,&PrevStrLenAddr[1],sizeof(long)*(height - 1));
                     PrevStrLenAddr[PrevStrCount - 1] = cfpos - CurrStrLenBuff[0];
                     PrevPageSize = PrevStrLenAddr[PrevStrCount - 1] - PrevStrLenAddr[0] + PrevStrLen;
                     break;
@@ -145,17 +154,17 @@ static void __FASTCALL__ PrepareAsmLines(int keycode,unsigned long cfpos)
                       size_t i;
                       unsigned size;
                       unsigned long prevpos;
-                      size = Summ(CurrStrLenBuff,Height);
+                      size = Summ(CurrStrLenBuff,height);
                       prevpos = cfpos - size;
                       size = 0;
-                      for(i = 0;i < Height;i++)
+                      for(i = 0;i < height;i++)
                       {
                           size += CurrStrLenBuff[i];
                           PrevStrLenAddr[i] = prevpos + size;
                       }
-                      PrevStrLen = CurrStrLenBuff[Height - 1];
+                      PrevStrLen = CurrStrLenBuff[height - 1];
                       PrevPageSize = size;
-                      PrevStrCount = Height;
+                      PrevStrCount = height;
                     }
                     break;
    case KE_UPARROW:
@@ -206,33 +215,40 @@ static unsigned __FASTCALL__ drawAsm( unsigned keycode, unsigned textshift )
    hexAddressResolv = addrdet;
  }
  PrepareAsmLines(keycode,cfpos);
- if(amocpos != cfpos || keycode == KE_SUPERKEY)
+ if(amocpos != cfpos || keycode == KE_SUPERKEY || keycode == KE_JUSTFIND)
  {
+   tAbsCoord height = twGetClientHeight(MainWnd);
+   tAbsCoord width = twGetClientWidth(MainWnd);
    GidResetGoAddress(keycode);
-   I = 0;   
+   I = 0;
+   twFreezeWin(MainWnd);
    if(keycode == KE_UPARROW)
    {
      dir = -1;
      Limit = -1;
      /* All checks we have done above */
      twScrollWinDn(MainWnd,1,1);
-     memmove(&CurrStrLenBuff[1],CurrStrLenBuff,Height-1);
+     memmove(&CurrStrLenBuff[1],CurrStrLenBuff,height-1);
    }
    else
    {
      dir = 1;
-     Limit = Height;
+     Limit = height;
    }
    if(keycode == KE_DOWNARROW)
    {
      if(CurrStrLenBuff[1])
      {
-       I = Height-1;
+       I = height-1;
        twScrollWinUp(MainWnd,I,1);
        memmove(CurrStrLenBuff,&CurrStrLenBuff[1],I);
        cfpos += Summ(CurrStrLenBuff,I);
      }
-     else goto bye;
+     else
+     {
+       twRefreshWin(MainWnd);
+       goto bye;
+     }
    }
    if(cfpos > flen) cfpos = flen;
    amocpos = cfpos;
@@ -269,18 +285,22 @@ static unsigned __FASTCALL__ drawAsm( unsigned keycode, unsigned textshift )
        if(disPanelMode < PANMOD_MEDIUM)
        {
          unsigned full_off,med_off,tmp_off;
+         ColorAttr opc;
          med_off = disMaxCodeLen*2+1;
          full_off = med_off+10;
          for(j = 0;j < dret.codelen;j++,len+=2)
             memcpy(&outstr[len],Get2Digit(disCodeBuffer[j]),2);
          tmp_off = disPanelMode < PANMOD_FULL ? full_off : med_off;
          if(len < tmp_off) len = tmp_off;
-         twSetColorAttr(disasm_cset.opcodes);
+         if(activeDisasm->GetOpcodeColor) 
+		opc = HiLight ? activeDisasm->GetOpcodeColor(dret.pro_clone) : disasm_cset.opcodes;
+	 else	opc = disasm_cset.opcodes;
+         twSetColorAttr(opc);
          twDirectWrite(disPanelMode < PANMOD_FULL ? 11 : 1,
                        i + 1,
                        &outstr[10],
                        disPanelMode < PANMOD_FULL ? len - 11 : len - 1);
-         if(isHOnLine(cfpos,dret.codelen))  
+         if(isHOnLine(cfpos,dret.codelen))
          {
             hli.text = &outstr[10];
             HiLightSearch(MainWnd,cfpos,10,dret.codelen,i,&hli,HLS_USE_DOUBLE_WIDTH);
@@ -295,6 +315,7 @@ static unsigned __FASTCALL__ drawAsm( unsigned keycode, unsigned textshift )
        /* Here adding commentaries */
        savstring[0] = 0;
        orig_commoff = orig_commpos = 0;
+       if(j > 5)
        if(dret.str[j-5] == codeguid_image[0] &&
           dret.str[j-4] == codeguid_image[1] &&
           dret.str[j-3] == codeguid_image[2] &&
@@ -307,6 +328,7 @@ static unsigned __FASTCALL__ drawAsm( unsigned keycode, unsigned textshift )
          strcpy(savstring,&dret.str[new_idx]);
          dret.str[new_idx--] = 0;
          while(dret.str[new_idx] == ' ' && new_idx) new_idx--;
+         if(dret.str[new_idx] != ' ') new_idx++;
          dret.str[new_idx] = 0;
          j = strlen(dret.str);
        }
@@ -331,7 +353,7 @@ static unsigned __FASTCALL__ drawAsm( unsigned keycode, unsigned textshift )
          }
        }
        twSetColorAttr(browser_cset.main);
-       if(len < Width)
+       if(len < width)
        {
          twGotoXY(len,i + 1);
          twClrEOL();
@@ -341,12 +363,13 @@ static unsigned __FASTCALL__ drawAsm( unsigned keycode, unsigned textshift )
      }
      else
      {
-       twDirectWrite(1,i + 1,outstr,Width);
+       twDirectWrite(1,i + 1,outstr,width);
        CurrStrLenBuff[i] = 0;
      }
    }
+   twRefreshWin(MainWnd);
    twSetColorAttr(browser_cset.main);
-   lastbyte = TopCFPos + Summ(CurrStrLenBuff,Height);
+   lastbyte = TopCFPos + Summ(CurrStrLenBuff,height);
    CurrPageSize = lastbyte-TopCFPos;
  }
  bye:
@@ -373,7 +396,7 @@ static tBool __FASTCALL__ disReferenceResolving( void )
   int retval;
   tBool ret;
   nModes = sizeof(refsdepth_names)/sizeof(char *);
-  retval = SelBoxA(refsdepth_names,nModes," References resolving depth: ",disNeedRef);
+  retval = SelBoxA(refsdepth_names,nModes," Reference resolving depth: ",disNeedRef);
   if(retval != -1)
   {
     disNeedRef = retval;
@@ -455,6 +478,7 @@ static void __FASTCALL__ DisasmScreen(TWindow* ewnd,unsigned long cp,unsigned lo
 {
  int i,j,len,lim;
  char outstr[__TVIO_MAXSCREENWIDTH+1],outstr1[__TVIO_MAXSCREENWIDTH+1];
+ tAbsCoord width = twGetClientWidth(MainWnd);
  DisasmRet dret;
  twFreezeWin(ewnd);
  for(i = st;i < stop;i++)
@@ -467,8 +491,8 @@ static void __FASTCALL__ DisasmScreen(TWindow* ewnd,unsigned long cp,unsigned lo
    twDirectWrite(1,i + 1,outstr,9);
    dret = Disassembler(cp + start,&EditorMem.buff[start],__DISF_NORMAL);
    EditorMem.alen[i] = dret.codelen;
-   memset(outstr,TWC_DEF_FILLER,Width);
-   memset(outstr1,TWC_DEF_FILLER,Width);
+   memset(outstr,TWC_DEF_FILLER,width);
+   memset(outstr1,TWC_DEF_FILLER,width);
    len = 0; for(j = 0;j < EditorMem.alen[i];j++) { memcpy(&outstr1[len],Get2Digit(EditorMem.save[start + j]),2); len += 2; }
    len = 0; for(j = 0;j < EditorMem.alen[i];j++) { memcpy(&outstr[len],Get2Digit(EditorMem.buff[start + j]),2); len += 2; }
    twUseWin(ewnd);
@@ -479,12 +503,12 @@ static void __FASTCALL__ DisasmScreen(TWindow* ewnd,unsigned long cp,unsigned lo
      twDirectWrite(j + 1,i + 1,&outstr[j],1);
    }
    len = strlen(dret.str);
-   memset(outstr,TWC_DEF_FILLER,Width);
+   memset(outstr,TWC_DEF_FILLER,width);
    memcpy(outstr,dret.str,len);
    twUseWin(MainWnd);
    twSetColorAttr(browser_cset.main);
    lim = disMaxCodeLen*2+11;
-   twDirectWrite(lim+1,i + 1,outstr,Width-lim);
+   twDirectWrite(lim+1,i + 1,outstr,width-lim);
    start += EditorMem.alen[i];
   }
   else
@@ -507,6 +531,7 @@ static int __NEAR__ __FASTCALL__ FullAsmEdit(TWindow * ewnd)
  unsigned rlen,len,flags;
  unsigned long flen;
  unsigned max_buff_size = disMaxCodeLen*tvioHeight;
+ tAbsCoord height = twGetClientHeight(MainWnd);
  tBool redraw = False;
  char outstr[__TVIO_MAXSCREENWIDTH],owork[__TVIO_MAXSCREENWIDTH];
 
@@ -517,9 +542,9 @@ static int __NEAR__ __FASTCALL__ FullAsmEdit(TWindow * ewnd)
  rlen = edit_cp + max_buff_size < flen ? max_buff_size : (int)(flen - edit_cp);
  BMReadBufferEx((void *)EditorMem.buff,rlen,edit_cp,BM_SEEK_SET);
  memcpy(EditorMem.save,EditorMem.buff,max_buff_size);
- memset(EditorMem.alen,TWC_DEF_FILLER,tvioHeight);
+ memset(EditorMem.alen,TWC_DEF_FILLER,height);
 
- DisasmScreen(ewnd,edit_cp,flen,0,Height,start);
+ DisasmScreen(ewnd,edit_cp,flen,0,height,start);
  PaintETitle(0,True);
  start = 0;
  twShowWin(ewnd);
@@ -533,7 +558,7 @@ static int __NEAR__ __FASTCALL__ FullAsmEdit(TWindow * ewnd)
    flags = __ESS_FILLER_7BIT | __ESS_WANTRETURN | __ESS_HARDEDIT;
    if(!redraw) flags |= __ESS_NOREDRAW;
    _lastbyte = eeditstring(outstr,&legalchars[2],&len,(unsigned)(edit_y + 1),(unsigned *)&edit_x,
-                          flags,owork);
+                          flags,owork,NULL);
    CompressHex(&EditorMem.buff[start],outstr,EditorMem.alen[edit_y],False);
    switch(_lastbyte)
    {
@@ -544,7 +569,7 @@ static int __NEAR__ __FASTCALL__ FullAsmEdit(TWindow * ewnd)
                        {
                          char code[81];
                          if(GetStringDlg(code,activeDisasm->name,
-                                         NULL,"Input assembler instruction:"))
+                                         NULL,"Enter assembler instruction:"))
                          {
                            aret = (*activeDisasm->asm_f)(code);
                            if(aret.err_code)
@@ -571,11 +596,11 @@ static int __NEAR__ __FASTCALL__ FullAsmEdit(TWindow * ewnd)
                          {
                            bioSeek(bHandle,edit_cp,BIO_SEEK_SET);
                            if(!bioWriteBuffer(bHandle,(void *)EditorMem.buff,rlen))
-                              errnoMessageBox("Can not write",NULL,errno);
+                              errnoMessageBox(WRITE_FAIL,NULL,errno);
                            bioClose(bHandle);
                            BMReRead();
                          }
-                         else errnoMessageBox("Can not reopen",NULL,errno);
+                         else errnoMessageBox("Can't reopen",NULL,errno);
                       }
      case KE_F(10):
      case KE_ESCAPE: goto bye;
@@ -588,7 +613,7 @@ static int __NEAR__ __FASTCALL__ FullAsmEdit(TWindow * ewnd)
      DisasmRet dret;
      dret = Disassembler(edit_cp + start,&EditorMem.buff[start],__DISF_NORMAL);
      EditorMem.alen[edit_y] = dret.codelen;
-     DisasmScreen(ewnd,edit_cp,flen,0,Height,0);
+     DisasmScreen(ewnd,edit_cp,flen,0,height,0);
    }
    PaintETitle(start + edit_x/2,True);
    CheckXYBounds();
@@ -602,11 +627,12 @@ static int __NEAR__ __FASTCALL__ FullAsmEdit(TWindow * ewnd)
 static void __FASTCALL__ disEdit( void )
 {
  TWindow * ewnd;
+ if(!BMGetFLength()) { ErrMessageBox(NOTHING_EDIT,NULL); return; }
  ewnd = WindowOpen(11,2,disMaxCodeLen*2+11,tvioHeight-1,TWS_CURSORABLE);
  twSetColorAttr(browser_cset.edit.main); twClearWin();
  edit_x = edit_y = 0;
  EditMode = EditMode ? False : True;
- if(editInitBuffs(disMaxCodeLen))
+ if(editInitBuffs(disMaxCodeLen,NULL,0))
  {
    FullAsmEdit(ewnd);
    editDestroyBuffs();
@@ -701,13 +727,93 @@ DisasmRet Disassembler(unsigned long ulShift,MBuffer buffer,unsigned flags)
 
 static unsigned __FASTCALL__ disCharSize( void ) { return 1; }
 
+static unsigned long __FASTCALL__ disSearch(TWindow *pwnd, unsigned long start,
+                                            unsigned long *slen, unsigned flags,
+                                            tBool is_continue, tBool *is_found)
+{
+  DisasmRet dret;
+  unsigned long tsize, retval, flen, dfpos, cfpos, sfpos; /* If search have no result */
+  char *disSearchBuff;
+  unsigned len, lw, proc, pproc, pmult;
+  int cache[UCHAR_MAX+1];
+  cfpos = sfpos = BMGetCurrFilePos();
+  flen = BMGetFLength();
+  retval = ULONG_MAX;
+  disSearchBuff  = PMalloc(1002+DISCOM_SIZE);
+  DumpMode = True;
+  if(!disSearchBuff)
+  {
+     MemOutBox("Disassembler search initialization");
+     goto bye;
+  }
+  cfpos = start;
+  tsize = flen;
+  pmult = 100;
+  if(tsize > ULONG_MAX/100) { tsize /= 100; pmult = 1; }
+  pproc = proc = 0;
+  /* Attempt to balance disassembler output */
+  PrepareAsmLines(KE_SUPERKEY, cfpos);
+  lw = disPrevLineWidth();
+  if(cfpos && cfpos >= lw) cfpos -= lw;
+  if(!(is_continue && (flags & SF_REVERSE))) cfpos += disCurrLineWidth();
+  /* end of attempt */
+  fillBoyerMooreCache(cache, search_buff, search_len, flags & SF_CASESENS);
+  while(1)
+  {
+    proc = (unsigned)((cfpos*pmult)/tsize);
+    if(proc != pproc)
+    {
+      if(!ShowPercentInWnd(pwnd,pproc=proc))  break;
+    }
+    if(flags & SF_REVERSE)
+    {
+       PrepareAsmLines(KE_UPARROW, cfpos);
+       lw = disPrevLineWidth();
+       if(cfpos && lw && cfpos >= lw)
+       {
+         len = cfpos + disMaxCodeLen < flen ? disMaxCodeLen : (int)(flen - cfpos);
+         memset(disCodeBuffer,0,disMaxCodeLen);
+         dfpos = cfpos;
+         BMReadBufferEx((void *)disCodeBuffer,len,cfpos,BM_SEEK_SET);
+         dret = Disassembler(cfpos,(void *)disCodeBuffer,__DISF_NORMAL);
+         cfpos -= lw;
+       }
+       else break;
+    }
+    else
+    {
+         len = cfpos + disMaxCodeLen < flen ? disMaxCodeLen : (int)(flen - cfpos);
+         memset(disCodeBuffer,0,disMaxCodeLen);
+         dfpos = cfpos;
+         BMReadBufferEx((void *)disCodeBuffer,len,cfpos,BM_SEEK_SET);
+         dret = Disassembler(cfpos,(void *)disCodeBuffer,__DISF_NORMAL);
+         cfpos += dret.codelen;
+         if(cfpos >= flen) break;
+    }
+    strcpy(disSearchBuff, dret.str);
+    strcat(disSearchBuff, dis_comments);
+    if(strFind(disSearchBuff, strlen(disSearchBuff), search_buff, search_len, cache, flags))
+    {
+      *is_found = True;
+      retval = dfpos;
+      *slen = dret.codelen;
+      break;
+    }
+  }
+  PFREE(disSearchBuff);
+  bye:
+  BMSeek(sfpos, SEEK_SET);
+  DumpMode = False;
+  return retval;
+}
+
 REGISTRY_MODE disMode =
 {
   "~Dissasembler",
   { NULL, "Disasm", NULL, NULL, NULL, "AResol", "PanMod", "ResRef", "HiLght", NULL },
   { NULL, disSelect_Disasm, NULL, NULL, NULL, hexAddressResolution, disSelectPanelMode, disReferenceResolving, disSelectHiLight, NULL },
   disDetect,
-  __MF_USECODEGUIDE,
+  __MF_USECODEGUIDE | __MF_DISASM,
   drawAsm,
   NULL,
   disCharSize,
@@ -721,7 +827,8 @@ REGISTRY_MODE disMode =
   disReadIni,
   disSaveIni,
   disInit,
-  disTerm
+  disTerm,
+  disSearch
 };
 
 static void __NEAR__ __FASTCALL__ disAcceptActions( void )
@@ -780,7 +887,7 @@ void  __FASTCALL__ disSetModifier(char *str,const char *modf)
 int __FASTCALL__ disAppendDigits(char *str,unsigned long ulShift,int flags,
                      char codelen,void *defval,unsigned type)
 {
- int app;
+ unsigned long app;
  char comments[DISCOM_SIZE];
  const char *appstr;
  unsigned dig_type;
@@ -795,14 +902,15 @@ int __FASTCALL__ disAppendDigits(char *str,unsigned long ulShift,int flags,
      {
        strncpy(sout,str,sizeof(sout)-1);
        sout[sizeof(sout)-1] = 0;
-       ErrMessageBox(sout," Detected internal disassembler error ");
+       ErrMessageBox(sout," Internal disassembler error detected ");
        displayed = True;
      }
   }
 #endif
+  if(hexAddressResolv && detectedFormat->AddressResolving) flags |= APREF_SAVE_VIRT;
   app = disNeedRef >= NEEDREF_ALL ? AppendAsmRef(str,ulShift,flags,codelen,0L) :
                                     RAPREF_NONE;
-  if(app == RAPREF_NONE || app == RAPREF_PROCEEDCMT)
+  if(app != RAPREF_DONE)
   {
     dig_type = type & 0x00FFU;
     comments[0] = 0;
@@ -815,9 +923,10 @@ int __FASTCALL__ disAppendDigits(char *str,unsigned long ulShift,int flags,
       if(*(unsigned long *)defval)         /* Do not perform operation on NULL */
       {
       unsigned long pa,psym;
-      unsigned class;
-      pa = detectedFormat->va2pa ? detectedFormat->va2pa(*(unsigned long *)defval) :
-                                   *(unsigned long *)defval;
+      unsigned _class;
+      if(!app) pa = detectedFormat->va2pa ? detectedFormat->va2pa(*(unsigned long *)defval) :
+                                           *(unsigned long *)defval;
+      else pa = app;
       if(pa)
       {
         /* 1. Try to determine immediate as offset to public symbol */
@@ -827,7 +936,7 @@ int __FASTCALL__ disAppendDigits(char *str,unsigned long ulShift,int flags,
           if(detectedFormat->GetPubSym)
           {
             psym = detectedFormat->GetPubSym(&comments[2],sizeof(comments)-2,
-                                             &class,pa,False);
+                                             &_class,pa,False);
             if(psym != pa) comments[0] = 0;
             else
             {
@@ -844,17 +953,17 @@ int __FASTCALL__ disAppendDigits(char *str,unsigned long ulShift,int flags,
           unsigned char rch;
           index = 0;
           strcat(comments,"->\"");
-          for(index = 0;index < sizeof(comments)-6;index++)
+          for(index = 3;index < sizeof(comments)-5;index++)
           {
-            bmSeek(pa+index,BM_SEEK_SET);
+            bmSeek(pa+index-3,BM_SEEK_SET);
             rch = bmReadByte();
-            if(isprint(rch)) comments[index+3] = rch;
+            if(isprint(rch)) comments[index] = rch;
             else break;
           }
-          if(!index) comments[0] = 0;
-          else 
+          if(!comments[3]) comments[0] = 0;
+          else
           {
-            comments[index+3] = '"'; comments[index+4] = 0; 
+            comments[index++] = '"'; comments[index] = 0;
             dis_severity = DISCOMSEV_STRPTR;
             strcpy(dis_comments,comments);
           }
@@ -867,14 +976,41 @@ int __FASTCALL__ disAppendDigits(char *str,unsigned long ulShift,int flags,
     {
      switch(dig_type)
      {
-      case DISARG_LONG:  appstr = Get8SignDig(*(long *)defval);
-                         if(type & DISARG_IMM && 
+      case DISARG_LLONG: 
+#ifdef INT64_C
+			 appstr = Get16SignDig(*(tInt64 *)defval);
+#else
+			 appstr = Get16SignDig(((tInt32 *)defval)[0],((tInt32 *)defval)[1]);
+#endif
+                         if(type & DISARG_IMM &&
                             disNeedRef >= NEEDREF_PREDICT &&
                             dis_severity < DISCOMSEV_STRING &&
                             isprint(((unsigned char *)defval)[0]) &&
                             isprint(((unsigned char *)defval)[1]) &&
                             isprint(((unsigned char *)defval)[2]) &&
-                            isprint(((unsigned char *)defval)[3])) 
+                            isprint(((unsigned char *)defval)[3]) &&
+                            isprint(((unsigned char *)defval)[4]) &&
+                            isprint(((unsigned char *)defval)[5]) &&
+                            isprint(((unsigned char *)defval)[6]) &&
+                            isprint(((unsigned char *)defval)[7]))
+                            sprintf(comments,"\"%c%c%c%c%c%c%c%c\""
+                                            ,((unsigned char *)defval)[0]
+                                            ,((unsigned char *)defval)[1]
+                                            ,((unsigned char *)defval)[2]
+                                            ,((unsigned char *)defval)[3]
+                                            ,((unsigned char *)defval)[4]
+                                            ,((unsigned char *)defval)[5]
+                                            ,((unsigned char *)defval)[6]
+                                            ,((unsigned char *)defval)[7]);
+                         break;
+      case DISARG_LONG:  appstr = Get8SignDig(*(long *)defval);
+                         if(type & DISARG_IMM &&
+                            disNeedRef >= NEEDREF_PREDICT &&
+                            dis_severity < DISCOMSEV_STRING &&
+                            isprint(((unsigned char *)defval)[0]) &&
+                            isprint(((unsigned char *)defval)[1]) &&
+                            isprint(((unsigned char *)defval)[2]) &&
+                            isprint(((unsigned char *)defval)[3]))
                             sprintf(comments,"\"%c%c%c%c\""
                                             ,((unsigned char *)defval)[0]
                                             ,((unsigned char *)defval)[1]
@@ -882,54 +1018,81 @@ int __FASTCALL__ disAppendDigits(char *str,unsigned long ulShift,int flags,
                                             ,((unsigned char *)defval)[3]);
                          break;
       case DISARG_SHORT: appstr = Get4SignDig(*(short *)defval);
-                         if(type & DISARG_IMM && 
+                         if(type & DISARG_IMM &&
                             disNeedRef >= NEEDREF_PREDICT &&
                             dis_severity < DISCOMSEV_STRING &&
                             isprint(((unsigned char *)defval)[0]) &&
-                            isprint(((unsigned char *)defval)[1])) 
+                            isprint(((unsigned char *)defval)[1]))
                             sprintf(comments,"\"%c%c\""
                                             ,((unsigned char *)defval)[0]
                                             ,((unsigned char *)defval)[1]);
                          break;
       case DISARG_CHAR:  appstr = Get2SignDig(*(char *)defval);
-                         if(type & DISARG_IMM && 
+                         if(type & DISARG_IMM &&
                             disNeedRef >= NEEDREF_PREDICT &&
                             dis_severity < DISCOMSEV_STRING &&
-                            isprint(*(unsigned char *)defval)) 
+                            isprint(*(unsigned char *)defval))
                             sprintf(comments,"'%c'",*(unsigned char *)defval);
                          break;
       case DISARG_BYTE:  appstr = Get2Digit(*(unsigned char *)defval);
-                         if(type & DISARG_IMM && 
+                         if(type & DISARG_IMM &&
                             disNeedRef >= NEEDREF_PREDICT &&
                             dis_severity < DISCOMSEV_STRING &&
-                            isprint(*(unsigned char *)defval)) 
+                            isprint(*(unsigned char *)defval))
                             sprintf(comments,"'%c'",*(unsigned char *)defval);
                          break;
       case DISARG_WORD:  appstr = Get4Digit(*(unsigned short *)defval);
-                         if(type & DISARG_IMM && 
+                         if(type & DISARG_IMM &&
                             disNeedRef >= NEEDREF_PREDICT &&
                             dis_severity < DISCOMSEV_STRING &&
                             isprint(((unsigned char *)defval)[0]) &&
-                            isprint(((unsigned char *)defval)[1])) 
+                            isprint(((unsigned char *)defval)[1]))
                             sprintf(comments,"\"%c%c\""
                                             ,((unsigned char *)defval)[0]
                                             ,((unsigned char *)defval)[1]);
                          break;
       default:
-      case DISARG_DWORD: appstr = Get8Digit(*(unsigned long *)defval); 
-                         if(type & DISARG_IMM && 
+      case DISARG_DWORD: appstr = Get8Digit(*(unsigned long *)defval);
+                         if(type & DISARG_IMM &&
                             disNeedRef >= NEEDREF_PREDICT &&
                             dis_severity < DISCOMSEV_STRING &&
                             isprint(((unsigned char *)defval)[0]) &&
                             isprint(((unsigned char *)defval)[1]) &&
                             isprint(((unsigned char *)defval)[2]) &&
-                            isprint(((unsigned char *)defval)[3])) 
+                            isprint(((unsigned char *)defval)[3]))
                             sprintf(comments,"\"%c%c%c%c\""
                                             ,((unsigned char *)defval)[0]
                                             ,((unsigned char *)defval)[1]
                                             ,((unsigned char *)defval)[2]
                                             ,((unsigned char *)defval)[3]);
                          break;
+      case DISARG_QWORD:
+#ifdef INT64_C
+			 appstr = Get16Digit(*(tUInt64 *)defval);
+#else
+			 appstr = Get16Digit(((tUInt32 *)defval)[0],((tUInt32 *)defval)[1]);
+#endif
+                         if(type & DISARG_IMM &&
+                            disNeedRef >= NEEDREF_PREDICT &&
+                            dis_severity < DISCOMSEV_STRING &&
+                            isprint(((unsigned char *)defval)[0]) &&
+                            isprint(((unsigned char *)defval)[1]) &&
+                            isprint(((unsigned char *)defval)[2]) &&
+                            isprint(((unsigned char *)defval)[3]) &&
+                            isprint(((unsigned char *)defval)[4]) &&
+                            isprint(((unsigned char *)defval)[5]) &&
+                            isprint(((unsigned char *)defval)[6]) &&
+                            isprint(((unsigned char *)defval)[7]))
+                            sprintf(comments,"\"%c%c%c%c%c%c%c%c\""
+                                            ,((unsigned char *)defval)[0]
+                                            ,((unsigned char *)defval)[1]
+                                            ,((unsigned char *)defval)[2]
+                                            ,((unsigned char *)defval)[3]
+                                            ,((unsigned char *)defval)[4]
+                                            ,((unsigned char *)defval)[5]
+                                            ,((unsigned char *)defval)[6]
+                                            ,((unsigned char *)defval)[7]);
+                         break;
     }
     strcat(str,appstr);
    }
@@ -950,10 +1113,13 @@ int __FASTCALL__ disAppendFAddr(char * str,long ulShift,long distin,unsigned lon
  char *modif_to;
  DisasmRet dret;
  int appended = RAPREF_NONE;
+ int flags;
  fpos = bmGetCurrFilePos();
  /* Prepare insn type */
  if(disNeedRef > NEEDREF_NONE)
  {
+   /* Forward prediction: ulShift = offset of binded field but r_sh is
+      pointer where this field is referenced. */
    memset(disCodeBufPredict,0,disMaxCodeLen*PREDICT_DEPTH);
    bmSeek(r_sh, SEEK_SET);
    bmReadBuffer(disCodeBufPredict,disMaxCodeLen*PREDICT_DEPTH);
@@ -968,20 +1134,23 @@ int __FASTCALL__ disAppendFAddr(char * str,long ulShift,long distin,unsigned lon
      {
        strncpy(sout,str,sizeof(sout)-1);
        sout[sizeof(sout)-1] = 0;
-       ErrMessageBox(sout," Detected internal disassembler error ");
+       ErrMessageBox(sout," Internal disassembler error detected ");
        displayed = True;
      }
   }
 #endif
  if(disNeedRef > NEEDREF_NONE)
  {
-   if(AppendAsmRef(str,ulShift,APREF_TRY_LABEL,codelen,r_sh)) appended = RAPREF_DONE;
+   if(dret.pro_clone == __INSNT_JMPPIC) goto try_pic; /* skip defaults for PIC */
+   flags = APREF_TRY_LABEL;
+   if(hexAddressResolv && detectedFormat->AddressResolving) flags |= APREF_SAVE_VIRT;
+   if(AppendAsmRef(str,ulShift,flags,codelen,r_sh)) appended = RAPREF_DONE;
    else
    {
       /*
          Forwarding references.
          Dereferencing ret instruction.
-         Idea and PE release of "Kostya Nosov" <k-nosov@yandex.ru> 
+         Idea and PE implementation by "Kostya Nosov" <k-nosov@yandex.ru>
       */
        if(dret.pro_clone == __INSNT_JMPVVT) /* jmp (mod r/m) */
        {
@@ -989,7 +1158,7 @@ int __FASTCALL__ disAppendFAddr(char * str,long ulShift,long distin,unsigned lon
             {
               appended = RAPREF_DONE;
               modif_to = strchr(str,' ');
-              if(modif_to) 
+              if(modif_to)
               {
                 while(*modif_to == ' ') modif_to++;
                 *(modif_to-1) = '*';
@@ -997,10 +1166,20 @@ int __FASTCALL__ disAppendFAddr(char * str,long ulShift,long distin,unsigned lon
               if(!DumpMode && !EditMode) GidAddGoAddress(str,r_sh);
             }
        }
+       else
+       if(dret.pro_clone == __INSNT_JMPPIC) /* jmp [ebx+offset] */
+       {
+            try_pic:
+            if(AppendAsmRef(str,r_sh+dret.field,APREF_TRY_PIC,dret.codelen,r_sh))
+            {
+              appended = RAPREF_DONE; /* terminate appending any info anyway */
+              if(!DumpMode && !EditMode) GidAddGoAddress(str,r_sh);
+            }
+       }
    }
  }
    /*
-      Idea and PE release of "Kostya Nosov" <k-nosov@yandex.ru>: 
+      Idea and PE release of "Kostya Nosov" <k-nosov@yandex.ru>:
       make virtual address as argument of "jxxx" and "callx"
    */
  if(!appended)
@@ -1010,7 +1189,7 @@ int __FASTCALL__ disAppendFAddr(char * str,long ulShift,long distin,unsigned lon
      r_sh = r_sh ? r_sh : ulShift;
      appended = detectedFormat->AddressResolving(&str[strlen(str)],r_sh) ? RAPREF_DONE : RAPREF_NONE;
    }
-   else
+   if(!appended)
    {
      needref = disNeedRef;
      disNeedRef = NEEDREF_NONE;
@@ -1027,16 +1206,18 @@ int __FASTCALL__ disAppendFAddr(char * str,long ulShift,long distin,unsigned lon
      else
      {
        const char * pstr = "";
-       if(type & 0x04)
+       if(type & DISADR_USESEG)
        {
          strcat(str,Get4Digit(seg));
          strcat(str,":");
        }
        if(!type) pstr = Get2SignDig((char)distin);
        else
-         if(type & 0x01) pstr = type & 0x04 ? Get4Digit((unsigned)distin) : Get4SignDig((unsigned)distin);
+         if(type & DISADR_NEAR16)
+              pstr = type & DISADR_USESEG ? Get4Digit((unsigned)distin) :
+                                            Get4SignDig((unsigned)distin);
          else
-          if(type & 0x02)   pstr = Get8SignDig(distin);
+          if(type & DISADR_NEAR32)   pstr = Get8SignDig(distin);
        strcat(str,pstr);
      }
      disNeedRef = needref;
