@@ -211,6 +211,10 @@ extern const char * k86_WordRegs[];
 extern const char * k86_DWordRegs[];
 extern const char * k86_QWordRegs[];
 extern const char * k86_XMMXRegs[];
+extern const char * k86_CrxRegs[];
+extern const char * k86_DrxRegs[];
+extern const char * k86_TrxRegs[];
+extern const char * k86_XrxRegs[];
 extern unsigned char k86_REX;
 extern int has_REX;
 extern tBool has67_in64;
@@ -274,6 +278,7 @@ extern void   __FASTCALL__ ix86_ArgAXDigit(char *str,ix86Param *);
 extern void   __FASTCALL__ ix86_ArgRMDigit(char *str,ix86Param *);
 extern void   __FASTCALL__ ix86_ArgIRegDigit(char *str,ix86Param *);
 extern void   __FASTCALL__ ix86_ArgIReg(char *str,ix86Param *);
+extern void   __FASTCALL__ ix86_ArgIReg64(char *str,ix86Param *);
 extern void   __FASTCALL__ ix86_ArgAXIReg(char *str,ix86Param *);
 extern void   __FASTCALL__ ix86_ArgSInt(char *str,ix86Param *);
 extern void   __FASTCALL__ ix86_ArgInt(char *str,ix86Param *);
