@@ -1,4 +1,4 @@
-/** 
+/**
  * @namespace   biew_plugins_II
  * @file        plugins/nls/russian.c
  * @brief       This file contains support of russian codepages.
@@ -72,7 +72,7 @@ static unsigned cp_mode = TXT_CURRCP;
 static unsigned char cp_symb_len = 1;
 
 #if 0
-static const short cp866_2_Unicode[] =
+static const tUInt16 cp866_2_Unicode[] =
 {
   0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007, /*0x00*/
   0x0008, 0x0009, 0x000A, 0x000B, 0x000C, 0x000D, 0x000E, 0x000F,
@@ -372,7 +372,7 @@ static void __NEAR__ __FASTCALL__ Translit7Bit(char *str,int size)
   }
 }
 
-static unsigned __FASTCALL__ ru_convert_buffer(char *buff,unsigned size)
+static unsigned __FASTCALL__ ru_convert_buffer(char *buff,unsigned size, tBool use_fs_nls)
 {
   switch(cp_mode)
   {
@@ -395,7 +395,11 @@ static unsigned __FASTCALL__ ru_convert_buffer(char *buff,unsigned size)
                         break;
       case TXT_TRANSLIT:Translit7Bit(buff,size); break;
   };
-  if(cp_mode != TXT_CURRCP) __nls_OemToOsdep((unsigned char *)buff,size);
+  if(cp_mode != TXT_CURRCP)
+  {
+    if(use_fs_nls) __nls_OemToFs((unsigned char *)buff,size);
+    else           __nls_OemToOsdep((unsigned char *)buff,size);
+  }
   return size / cp_symb_len;
 }
 
