@@ -31,7 +31,9 @@ int main( int argc, char * argv[] )
      printf("Too few\n\r");
      return EXIT_FAILURE;
   }
+  __init_sys();
   FiProgress(argv[1],MyProc);
+  __term_sys();
   return EXIT_SUCCESS;
 }
 
